﻿local addon, ns = ...
local oUF = ns.oUF or oUF

local GUIS = LibStub("gCore-3.0"):NewModule("gBags: Core")

-- Lua API
local tonumber, type = tonumber, type
local gsub = gsub
local unpack = unpack

local M = LibStub("gMedia-3.0")
local L = LibStub("gLocale-1.0"):GetLocale(addon)
local C = LibStub("gDB-1.0"):GetDataBase("gBags: Colors")
local F = LibStub("gDB-1.0"):GetDataBase("gBags: Functions")
local gPanel = LibStub("gPanel-2.0")
local CreateUIPanel = function(...) return LibStub("gPanel-2.0"):CreateUIPanel(...) end
local CreateChatCommand = function(...) GUIS:CreateChatCommand(...) end

local template

_G.gBAGS_DB = _G.gBAGS_DB or {}

--
-- our panel template(s)
template = function(object, arg1, arg2)
	if (arg1 == "border") then
		if (arg2) then
			object:SetUIBackdrop(M["Backdrop"]["PixelBorder"], true)
			object:SetUIBackdropBorderColor(unpack(C["border"]))
			object:CreateUIShadow()
			object:ClearAllPoints()
			object:SetPoint("TOPLEFT", arg2, "TOPLEFT", -3, 3)
			object:SetPoint("BOTTOMRIGHT", arg2, "BOTTOMRIGHT", 3, -3)
			
			return object
		else
			object._UIPANEL = object._UIPANEL or object:CreateUIPanel()
			object._UIPANEL:SetUIBackdrop(M["Backdrop"]["PixelBorder"], true)
			object._UIPANEL:SetUIBackdropBorderColor(unpack(C["border"]))
			object._UIPANEL:CreateUIShadow()
			object._UIPANEL:ClearAllPoints()
			object._UIPANEL:SetPoint("TOPLEFT", object, "TOPLEFT", -3, 3)
			object._UIPANEL:SetPoint("BOTTOMRIGHT", object, "BOTTOMRIGHT", 3, -3)

			return object._UIPANEL
		end
		
	elseif (arg1 == "backdrop") then
		object._UIPANEL = object._UIPANEL or object:CreateUIPanel()
		object._UIPANEL:SetUIBackdrop(M["Backdrop"]["PixelBorder-Satin"], true)
		object._UIPANEL:SetUIBackdropColor(unpack(C["overlay"]))
		object._UIPANEL:SetUIBackdropBorderColor(unpack(C["border"]))
		object._UIPANEL:CreateUIShadow()
		object._UIPANEL:ClearAllPoints()
		object._UIPANEL:SetPoint("TOPLEFT", arg2 or object, "TOPLEFT", -3, 3)
		object._UIPANEL:SetPoint("BOTTOMRIGHT", arg2 or object, "BOTTOMRIGHT", 3, -3)
		
		return object._UIPANEL
		
	elseif (arg1 == "itembackdrop") then
		object._UIPANEL = object._UIPANEL or object:CreateUIPanel()
		object._UIPANEL:SetUIBackdrop(M["Backdrop"]["ItemButton"], true)
		object._UIPANEL:SetUIBackdropColor(C["backdrop"][1], C["backdrop"][2], C["backdrop"][3], 1)
		object._UIPANEL:SetUIBackdropBorderColor(C["border"][1], C["border"][2], C["border"][3], 1)
		object._UIPANEL:CreateUIShadow()
		object._UIPANEL:ClearAllPoints()
		object._UIPANEL:SetPoint("TOPLEFT", arg2 or object, "TOPLEFT", -3, 3)
		object._UIPANEL:SetPoint("BOTTOMRIGHT", arg2 or object, "BOTTOMRIGHT", 3, -3)
		
		return object._UIPANEL
		
	elseif (arg1 == "blackbackdrop") then
		object._UIPANEL = object._UIPANEL or object:CreateUIPanel()
		object._UIPANEL:SetUIBackdrop(M["Backdrop"]["PixelBorder-Satin"], true)
		object._UIPANEL:SetUIBackdropColor(unpack(C["background"]))
		object._UIPANEL:SetUIBackdropBorderColor(unpack(C["border"]))
		object._UIPANEL:CreateUIShadow()
		object._UIPANEL:ClearAllPoints()
		object._UIPANEL:SetPoint("TOPLEFT", arg2 or object, "TOPLEFT", -3, 3)
		object._UIPANEL:SetPoint("BOTTOMRIGHT", arg2 or object, "BOTTOMRIGHT", 3, -3)
		
		return object._UIPANEL
		
	elseif (arg1 == "itembutton") then
		object:SetUIBackdrop(M["Backdrop"]["ItemButton"], true)
		object:SetUIBackdropColor(C["backdrop"][1], C["backdrop"][2], C["backdrop"][3], 1)
		object:SetUIBackdropBorderColor(C["border"][1], C["border"][2], C["border"][3], 1)
		object:CreateUIShadow()

		return object
		
	elseif (arg1 == "blank") then
		object:SetUIBackdrop(M["Backdrop"]["PixelBorder-Blank"], true)
		object:SetUIBackdropColor(unpack(C["background"]))
		object:SetUIBackdropBorderColor(unpack(C["border"]))
		object:CreateUIShadow()

		return object
	elseif (arg1 == "targetborder") then
		object:SetUIBackdrop(M["Backdrop"]["TargetBorder"], true)
		object:SetUIBackdropBorderColor(unpack(C["target"]))

		return object

	else
		object:SetUIBackdrop(M["Backdrop"]["PixelBorder-Satin"], true)
		object:SetUIBackdropColor(unpack(C["overlay"]))
		object:SetUIBackdropBorderColor(unpack(C["border"]))
		object:CreateUIShadow()

		return object
	end
end

GUIS.Init = function(self)
	-- validate saved settings
	gBAGS_DB.load = ValidateTable(gBAGS_DB.load, load)
	
	-- set the gPanel textures and colors to what gBags uses
	gPanel:SetColor("background", C["background"])
	gPanel:SetColor("border", C["border"])
	gPanel:SetColor("index", C["index"])
	gPanel:SetColor("overlay", C["overlay"])
	gPanel:SetColor("shadow", C["shadow"])
	gPanel:SetColor("value", C["value"])
	gPanel:SetFontObject(gBAGS_PanelFont)

	-- localize gPanel
	gPanel.L["PANEL_BAGS"] 						= L["PANEL_BAGS"]
	gPanel.L["PANEL_MAIL_NEW"] 					= L["PANEL_MAIL_NEW"]
	gPanel.L["PANEL_MAIL_NONE"] 				= L["PANEL_MAIL_NONE"]
	gPanel.L["PANEL_RESTED"] 					= L["PANEL_RESTED"]
	gPanel.L["PANEL_RIGHTCLICKFOROPTIONS"] 		= L["PANEL_RIGHTCLICKFOROPTIONS"] 
	gPanel.L["PANEL_TOTAL"] 					= L["PANEL_TOTAL"]
	gPanel.L["PANEL_TOTAL_USAGE"] 				= L["PANEL_TOTAL_USAGE"]
	gPanel.L["PANEL_TRACKED_CURRENCIES"] 		= L["PANEL_TRACKED_CURRENCIES"]
	gPanel.L["PANEL_UNLISTED_ADDONS"] 			= L["PANEL_UNLISTED_ADDONS"]
	
	gPanel:RegisterUITemplate(addon, template)
	
end
