﻿local addon, ns = ...

local F = LibStub("gDB-1.0"):NewDataBase("gBags: Functions")

local M = LibStub("gMedia-3.0")
local L = LibStub("gLocale-1.0"):GetLocale(addon)

--
-- returns the ideal uiScale
function F.ideal()
	local num = strmatch(GetCVar("gxResolution"), "%d+x(%d+)")
	if not(num) or ((tonumber(num) == 0)) then
		num = 1024
	end
	
	return 768/num
end

--
-- returns 1px in the current scale
function F.Pixel(parent)
	local num = strmatch(GetCVar("gxResolution"), "%d+x(%d+)")
	if not(num) or ((tonumber(num) == 0)) then
		return 1
	else
		return (F.ideal() / (parent or UIParent):GetEffectiveScale())
	end
end

--
-- transforms X into actual pixels, dependant on the 'parent' scale
function F.scale(x, parent)
	return x and floor(x * F.Pixel(parent) + 0.5)
end

local properRightAlign
do
	local dummy = UIParent:CreateFontString()
	dummy:Hide()

	local removePunctuation = function(msg)
		msg = gsub(msg, "[%.+]$", "")
		return msg
	end

	--
	-- Blizzard doesn't hide spaces at the end of a wrapped line
	-- when aligned to the right. This is a problem.
	properRightAlign = function(fontString)
		fontString:SetText(gsub(fontString:GetText(), "[%s]+", " "))
		fontString:SetText(removePunctuation(fontString:GetText()))

		local width = fontString:GetWidth()
		local fullWidth = fontString:GetStringWidth()
		if (fullWidth <= width) then
			return 
		end
		
		-- we need our dummy to have exact same font as the original
		dummy:SetFontObject(fontString:GetFontObject())
		dummy:SetFont(fontString:GetFontObject():GetFont())
		dummy:SetText(" ")
		
		local space = dummy:GetStringWidth()
		local verticalSpace = fontString:GetSpacing()
		local stringHeight = dummy:GetStringHeight()
		local words = { strsplit(" ", fontString:GetText()) }
		local wordWidths = {}
		
		for i = 1, #words do
			dummy:SetText(words[i])
			wordWidths[i] = dummy:GetStringWidth()
			dummy:SetText(nil)
		end
		
		local newWord = ""
		local lineLength, totalLength, height, counter = 0, 0, 0, 0
		while (counter < #words) do
			if (counter > 0) then
				height = height + verticalSpace
			end
			
			lineLength = 0
			while (lineLength + wordWidths[counter + 1] <= width) do
				counter = counter + 1
				newWord = newWord .. words[counter]
				lineLength = lineLength + wordWidths[counter]
				
				if (counter == #words) then
					break
				end

				if (lineLength + space + wordWidths[counter + 1] <= width) then
					newWord = newWord .. " "
					lineLength = lineLength + space
				else
					newWord = newWord .. "|n"
					break
				end
			end
			
			height = height + stringHeight
		end
		
		fontString:SetText(newWord)
		fontString:SetHeight(height)
--		return newWord, height
	end
end
F.properRightAlign = properRightAlign