﻿local addon, ns = ...

local MAJOR, MINOR = "gActionButtons-2.0", 2
local gAB, oldminor = LibStub:NewLibrary(MAJOR, MINOR)

if not gAB then return end 

assert(LibStub("gFrameHandler-1.0"), MAJOR .. " couldn't find an instance of gFrameHandler-1.0")

-- Lua API
local pairs, select = pairs, select
local strfind = string.find
local type = type

gAB.BUTTONS = gAB.BUTTONS or {}

local styleFunc = noop
local hotkeyFunc = noop

local rangetimer = 0.1
local showhotkey = 1 
local showmacro = 1

local playerClass = (select(2, UnitClass("player")))

local isSafe
local FixButtonGrid
local ButtonIsUsable, ButtonHasRange
local UpdateHotKey
local StyleActionButton
local StylePetActionButtons, StyleShapeShiftButtons, StyleMultiCastButtons, StyleSummonSpellButton, StyleRecallSpellButton
local UpdateAllHotkeys, UpdateAllNames

local isEnabled, hasEnteredWorld

local shiftClass = {
	["DRUID"] 			= true;
	["WARRIOR"] 		= true;
	["PALADIN"] 		= true;
	["DEATHKNIGHT"] 	= true;
	["ROGUE"] 			= true;
	["PRIEST"] 			= true;
	["HUNTER"] 			= true;
	["WARLOCK"] 		= true;
}

--
-- idea here is the have a safelist available, to decide which buttons should always have names/hotkeys
-- this is because the styling functions can be used by all itembuttons, not just actionbuttons
-- we build a table instead of doing a manual check on each call
local safeList = {}
do
	local buttons = {
		["ActionButton"] 				= NUM_ACTIONBAR_BUTTONS; 
		["MultiBarBottomLeftButton"] 	= NUM_ACTIONBAR_BUTTONS; 
		["MultiBarBottomRightButton"] 	= NUM_ACTIONBAR_BUTTONS; 
		["MultiBarRightButton"] 		= NUM_ACTIONBAR_BUTTONS; 
		["MultiBarLeftButton"] 			= NUM_ACTIONBAR_BUTTONS;
		["PetActionButton"] 			= NUM_PET_ACTION_SLOTS;
		["ShapeshiftButton"] 			= NUM_SHAPESHIFT_SLOTS;
		["MultiCastSummonSpellButton"] 	= 1;
		["MultiCastSlotButton"] 		= MAX_TOTEMS;
		["MultiCastRecallSpellButton"] 	= 1; 
	}
	for key, numButtons in pairs(buttons) do
		for index = 1, numButtons do
			safeList[key .. index] = true
		end
	end
	buttons = nil
end

--
-- ..aaand here's the function that performs the actual checking
isSafe = function(button)
	return safeList[button:GetName()]
end

-- make sure the style functions get hooked
local EventFrame = CreateFrame("Frame")
EventFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
EventFrame:SetScript("OnEvent", function(self, event, ...) 
	local arg1 = ...
	if (event == "PLAYER_ENTERING_WORLD") then
		gAB:OnEnter()
		
		-- we only need this once
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")
		
		hasEnteredWorld = true
				
	elseif (event == "UPDATE_SHAPESHIFT_FORMS")
		or (event == "UPDATE_SHAPESHIFT_USABLE") 
		or (event == "UPDATE_SHAPESHIFT_COOLDOWN")
		or (event == "UPDATE_SHAPESHIFT_FORM")
		or (event == "ACTIONBAR_PAGE_CHANGED") then
	
		StyleShapeShiftButtons()
		
	elseif (event == "PET_BAR_UPDATE") 
		or (event == "PLAYER_CONTROL_GAINED") 
		or (event == "PLAYER_CONTROL_LOST") 
		or (event == "PLAYER_FARSIGHT_FOCUS_CHANGED") 
		or (event == "PET_BAR_HIDE")
		or (event == "PET_UI_UPDATE")
		or (event == "PET_BAR_UPDATE_USABLE")
		or (event == "PET_BAR_UPDATE_COOLDOWN")
		or (event == "UNIT_FLAGS")
		or ((event == "UNIT_PET") and (arg1 == "player")) 
		or ((event == "UNIT_AURA") and (arg1 == "pet")) then
		
		StylePetActionButtons()
	end
end)


-- default colors
local C = {}
C["normal"] 	= { r = 0.15; g = 0.15; b = 0.15; } -- normal state, when nothing else applies
C["equipped"] 	= { r = 0.10; g = 0.85; b = 0.10; } -- equipped items
C["mana"] 		= { r = 0.10; g = 0.10; b = 0.85; } -- you haven't enough mana
C["range"] 		= { r = 0.85; g = 0.10; b = 0.10; } -- target is out of range
C["usable"] 	= { r = 1.00; g = 1.00; b = 1.00; } -- an item or ability is usable 
C["unusable"] 	= { r = 0.35; g = 0.35; b = 0.35; } -- an item or ability is not usable

FixButtonGrid = function(button)
	if not self then return end

	local nt = _G[button:GetName().."NormalTexture"]

	if (IsEquippedAction(button.action)) then
		nt:SetVertexColor(C["equipped"].r, C["equipped"].g, C["equipped"].b, 1)
	else
		nt:SetVertexColor(C["normal"].r, C["normal"].g, C["normal"].b, 1)
	end  
end

ButtonIsUsable = function(self)
	if not self or not self.action then return end
	
	local nt = _G[self:GetName().."NormalTexture"]
	local ic = _G[self:GetName().."Icon"]

	if IsEquippedAction(self.action) then
		nt:SetVertexColor(C["equipped"].r, C["equipped"].g, C["equipped"].b, 1)
	else
		nt:SetVertexColor(C["normal"].r, C["normal"].g, C["normal"].b, 1)
	end  
	
	local isUsable, notEnoughMana = IsUsableAction(self.action)
	
	if ActionHasRange(self.action) and IsActionInRange(self.action) == 0 then
		ic:SetVertexColor(C["range"].r, C["range"].g, C["range"].b, 1)
	elseif notEnoughMana then
		ic:SetVertexColor(C["mana"].r, C["mana"].g, C["mana"].b, 1)
	elseif isUsable then
		ic:SetVertexColor(C["usable"].r, C["usable"].g, C["usable"].b, 1)
	else
		ic:SetVertexColor(C["unusable"].r, C["unusable"].g, C["unusable"].b, 1)
	end
end

ButtonHasRange = function(self, elapsed )
	if not self then return end

	local t = self.rangetimer
	
	if not t then
		self.rangetimer = 0
		return
	end
	
	t = t + elapsed
	
	if t < (rangetimer or 0.1) then
		self.rangetimer = t
		return
	else
		self.rangetimer = self.rangetimer - (rangetimer or 0.1)
	end
	
	ButtonIsUsable(self)
end

UpdateHotKey = function(button, actionButtonType)
	if not button then return end
	
	local hotkey = _G[button:GetName().."HotKey"]
	if not hotkey then return end
	
	local text = hotkey:GetText()
	if text then
		hotkeyFunc(text, hotkey, button, actionButtonType)
	end
	
	if GetBoolean(showhotkey) then
		hotkey:Show()
	else
		hotkey:Hide()
	end
end 

StyleActionButton = function(self)
	if not self then return end

	local name = self:GetName()
	
	if not(gAB.BUTTONS[name]) then
		gAB.BUTTONS[name] = self
	end
	
	local actionType, ID, subType
	if self.action then
		actionType, ID, subType = GetActionInfo(self.action)
	end

	self.ActionType 	= actionType
	self.Autocast 		= _G[name.."AutoCastable"]
	self.Background 	= _G[name].background
	self.Border 		= _G[name.."Border"]
	self.Button 		= _G[name]
	self.Cooldown 		= _G[name.."Cooldown"]
	self.Count 			= _G[name.."Count"]
	self.Flash 			= _G[name.."Flash"]
	self.FloatingBG 	= _G[name.."FloatingBG"]
	self.Frame 			= _G[name.."Frame"]
	self.Highlight 		= _G[name.."Highlight"]
	self.Hotkey 		= _G[name.."HotKey"]
	self.Hover 			= _G[name.."HoverTex"]
	self.Icon 			= _G[name.."Icon"] or _G[name.."IconTexture"]
	self.ItemID 		= actionType == "item" and ID
	self.ItemRarity 	= actionType == "item" and (select(3,GetItemInfo(ID)))
	self.Macro 			= _G[name.."Name"]
	self.Name 			= name
	self.NormalTexture 	= _G[name.."NormalTexture"] or _G[name.."NormalTexture2"]
	self.Overlay 		= _G[name].overlay
	self.Panel 			= _G[name.."Panel"]
	self.Pushed 		= _G[name.."PushedTex"]
	self.Shine 			= _G[name.."Shine"]
	self.TotemBorder 	= (strfind(name, "MultiCastActionButton")) ~= nil and (select(11, self:GetRegions())) or (strfind(name, "MultiCastSlotButton")) ~= nil and (select(2, self:GetRegions())) 

	if self.Macro then
		if GetBoolean(showmacro) or (not isSafe(self)) then
			self.Macro:Show()
		else
			self.Macro:Hide()
		end
	end
	
	if self.Hotkey then
		if GetBoolean(showhotkey) then
			self.Hotkey:Show()
		else
			self.Hotkey:Hide()
		end
	end

	styleFunc(self)
	UpdateHotKey(self)
end
gAB.StyleActionButton = StyleActionButton

StylePetActionButtons = function()
	if not(isEnabled) then
		return
	end

	for i = 1, NUM_PET_ACTION_SLOTS, 1 do
		StyleActionButton(_G["PetActionButton"..i])
	end  
end

StyleShapeShiftButtons = function()
	if not(isEnabled) then
		return
	end

	for i = 1, NUM_SHAPESHIFT_SLOTS, 1 do
		StyleActionButton(_G["ShapeshiftButton"..i])
	end
end

StyleMultiCastButtons = function()
	if not(isEnabled) then
		return
	end

	for i = 1, MAX_TOTEMS do
		StyleActionButton(_G["MultiCastSlotButton"..i])
	end
	
	for i = 1, NUM_MULTI_CAST_PAGES * NUM_MULTI_CAST_BUTTONS_PER_PAGE do
		StyleActionButton(_G["MultiCastActionButton"..i])
	end
end

StyleSummonSpellButton = function()
	if not(isEnabled) then
		return
	end

	StyleActionButton(MultiCastSummonSpellButton)
end

StyleRecallSpellButton = function()
	if not(isEnabled) then
		return
	end

	StyleActionButton(MultiCastRecallSpellButton)
end

UpdateAllHotkeys = function()
	if not(isEnabled) then
		return
	end

	for name, button in pairs(gAB.BUTTONS) do
		if _G[name.."HotKey"] then
			if GetBoolean(showhotkey) then
				_G[name.."HotKey"]:Show()
			else
				_G[name.."HotKey"]:Hide()
			end
		end
	end
end

UpdateAllNames = function()
	if not(isEnabled) then
		return
	end

	for name, button in pairs(gAB.BUTTONS) do
		if _G[name.."Name"] then
			if GetBoolean(showmacro) or (not isSafe(button)) then
				_G[name.."Name"]:Show()
			else
				_G[name.."Name"]:Hide()
			end
		end
	end
end

function gAB:Enable()
	if (self.enabled) or (isEnabled) then 
		return 
	else
		self.enabled = true
		isEnabled = true
	end
	
	hooksecurefunc("ActionButton_Update", StyleActionButton)
	hooksecurefunc("ActionButton_UpdateUsable", ButtonIsUsable)
	hooksecurefunc("ActionButton_OnUpdate", ButtonHasRange)
	hooksecurefunc("ActionButton_ShowGrid", FixButtonGrid)
	hooksecurefunc("ActionButton_UpdateHotkeys", UpdateHotKey)
	
	if (playerClass == "SHAMAN") then
		hooksecurefunc("MultiCastActionBarFrame_Update", StyleMultiCastButtons)
		hooksecurefunc("MultiCastSlotButton_Update", StyleMultiCastButtons)
		hooksecurefunc("MultiCastSummonSpellButton_Update", StyleSummonSpellButton)
		hooksecurefunc("MultiCastRecallSpellButton_Update", StyleRecallSpellButton)
	end

	EventFrame:RegisterEvent("PET_UI_UPDATE")
	EventFrame:RegisterEvent("PET_BAR_HIDE")
	EventFrame:RegisterEvent("PET_BAR_UPDATE")
	EventFrame:RegisterEvent("PET_BAR_UPDATE_USABLE")
	EventFrame:RegisterEvent("PET_BAR_UPDATE_COOLDOWN")
	EventFrame:RegisterEvent("PLAYER_CONTROL_GAINED")
	EventFrame:RegisterEvent("PLAYER_CONTROL_LOST")
	EventFrame:RegisterEvent("PLAYER_FARSIGHT_FOCUS_CHANGED")
	EventFrame:RegisterEvent("UNIT_FLAGS")
	EventFrame:RegisterEvent("UNIT_PET")
	EventFrame:RegisterEvent("UNIT_AURA")
	
	if (shiftClass[playerClass]) then
		EventFrame:RegisterEvent("UPDATE_SHAPESHIFT_FORMS")
		EventFrame:RegisterEvent("UPDATE_SHAPESHIFT_USABLE")
		EventFrame:RegisterEvent("UPDATE_SHAPESHIFT_COOLDOWN")
		EventFrame:RegisterEvent("UPDATE_SHAPESHIFT_FORM")
		EventFrame:RegisterEvent("ACTIONBAR_PAGE_CHANGED")

	end

--		hooksecurefunc("ShapeshiftBar_OnLoad", StyleShapeShiftButtons)
--		hooksecurefunc("ShapeshiftBar_Update", StyleShapeShiftButtons)
--		hooksecurefunc("ShapeshiftBar_UpdateState", StyleShapeShiftButtons)
	hooksecurefunc("PetActionBar_Update", StylePetActionButtons)
	
	if (hasEnteredWorld) then
		self:OnEnter()
	end
end

function gAB:OnEnter()
	if not(isEnabled) then
		return
	end
	
	if (playerClass == "SHAMAN") then
		StyleMultiCastButtons()
	end
	
	if (shiftClass[playerClass]) then
		StyleShapeShiftButtons()
	end
	
	StylePetActionButtons()
end

------------------------------------------------------------------------------------------------------------
-- 	API Calls
------------------------------------------------------------------------------------------------------------
function gAB:SetStyleFunction(func, cold)
	styleFunc = type(func) == "function" and func or noop

	if not(cold) then 
		self:Enable()
	end
end

function gAB:SetHotkeyFunction(func, cold)
	hotkeyFunc = type(func) == "function" and func or noop

	if not(cold) then 
		self:Enable()
	end
end

function gAB:SetHotkeyState(state)
	showhotkey = GetBinary(state)

	UpdateAllHotkeys()
end

function gAB:SetNameState(state)
	showmacro = GetBinary(state)
	
	UpdateAllNames()
end
