﻿local MAJOR, MINOR = "gLocale-1.0", 1
local gLocale, oldminor = LibStub:NewLibrary(MAJOR, MINOR)

if not gLocale then return end 

-- Lua APIs
local setmetatable = setmetatable

gLocale.L = gLocale.L or {} 
gLocale.defaults = gLocale.defaults or {}

-- Just not that big a difference between these two to bother
local clientLocale = GetLocale()
if clientLocale == "enGB" then
	clientLocale = "enUS"
end

--
-- The first locale to be registered for an addon will be its default
-- Not very functional, but will do
--
-- 'addon' can be anything, as long as its name is unique with this handler
-- 'locale' is the name of the locale, typically "enUS", "esMX", etc 
-- 		* there is no check to see whether or not the locale is a "real" one, so use caution
function gLocale:NewLocale(addon, locale)
	if type(addon) ~= "string" then
		geterrorhandler()((MAJOR .. ": NewLocale(addon, locale) 'addon' - string expected, got %s"):format(type(addon)), 2)
		return 
	end

	if type(locale) ~= "string" then
		geterrorhandler()((MAJOR .. ": NewLocale(addon, locale) 'locale' - string expected, got %s"):format(type(locale)), 2)
		return 
	end

	self.L[addon] = self.L[addon] or {}

	if self.L[addon][locale] then
		geterrorhandler()((MAJOR .. ": NewLocale(addon, locale) 'locale' - data for locale '%s' already exists"):format(type(locale)), 2)
		return 
	end
	
	if not self.defaults[addon] then
		self.L[addon][locale] = {}
		self.defaults[addon] = locale
	else
		-- if this is neither the default locale, nor the one of the client language,
		-- then we don't need it at all
		if (locale ~= self.defaults[addon]) and (locale ~= clientLocale) then
			return 
		end

		-- the new locale will get its missing values from the default (first registered) locale
		self.L[addon][locale] = setmetatable({}, { __index = self.L[addon][self.defaults[addon]]})
	end

	-- return a pointer to the new locale table
	-- which the user can add values to
	return self.L[addon][locale]
end

--
-- retrieve the current locale for the game client
-- will create an empty locale if none already exist in the clients language
function gLocale:GetLocale(addon)
	if type(addon) ~= "string" then
		geterrorhandler()((MAJOR .. ": GetLocale(addon) 'addon' - string expected, got %s"):format(type(addon)), 2)
		return 
	end
	
	if (not self.L[addon]) then
		geterrorhandler()((MAJOR .. ": GetLocale(addon) 'addon' - couldn't find any registered locales for addon '%s'"):format(type(addon)), 2)
		return 
	end

	return self.L[addon][clientLocale] or self:NewLocale(addon, clientLocale)
end

