﻿local addon, ns = ...

assert(LibStub("gFrameHandler-1.0"), "Couldn't find an instance of gFrameHandler-1.0")
assert(LibStub("gString-1.0"), "Couldn't find an instance of gString-1.0")
assert(LibStub("gPanel-2.0"), "Couldn't find an instance of gPanel-2.0")

-- Lua API
local floor, min = math.floor, math.min
local ipairs, select, unpack = ipairs, select, unpack

local RegisterPlugin = function(...) LibStub("gPanel-2.0"):RegisterPlugin(...) end

local FACTION_BAR_COLORS = LibStub("gPanel-2.0"):GetColor("FACTION_BAR_COLORS")

local L = LibStub("gPanel-2.0").L
L["PANEL_BAGS"] 					= L["PANEL_BAGS"] 			or "Bags"
L["PANEL_GUILD_NONE"] 				= L["PANEL_GUILD_NONE"] 	or "No Guild"
L["PANEL_MAIL_NEW"] 				= L["PANEL_MAIL_NEW"] 		or "New Mail!"
L["PANEL_MAIL_NONE"] 				= L["PANEL_MAIL_NONE"] 		or "No Mail"

local M = LibStub("gMedia-3.0")

local loadModule = function(name)
	local module = LibStub("gCore-3.0"):GetModule(name)
	local dead = LibStub("gDB-1.0"):GetDataBase("GUIS-gUI: Functions").kill(name)

	if ((module) and not(dead)) then
		return module
	end
end

local FixDropDown = function(self)
	DropDownList1:ClearAllPoints()
	DropDownList1:SetPoint("BOTTOM", self:GetParent(), "TOP", 0, 8)

	GameTooltip:Hide()
end
------------------------------------------------------------------------------------------------------------
-- 	Plugins
------------------------------------------------------------------------------------------------------------
--[[
	RegisterPlugin("NAME_OF_PLUGIN" , {
		onclick = nil or function(self) end;
		onevent = nil or function(self, event) end;
		oninit = nil or function(self) end;

		interval = update_interval_in_secs;
		onupdate = nil or function(self) end;

		tooltip = nil or "Name_of_Tip";

		events = { ... };
		variables = { ... };
	})
]]--

local friendsMenuFrame = CreateFrame("Frame", "GUIS-gUI_Panel_FriendsDropdown", self, "UIDropDownMenuTemplate")
local friendsMenuList = function() 
	local L = LibStub("gLocale-1.0"):GetLocale(addon)
	return {
	{
		text = ADD_FRIEND;
		notCheckable = true;
		keepShownOnClick = false;
		func = function()
			FriendsFrameAddFriendButton:Click()
			-- StaticPopup_Show("ADD_FRIEND")
		end
	};
} end
RegisterPlugin("Friends" , {
	events = {
		"FRIENDLIST_UPDATE";
		"PLAYER_ENTERING_WORLD";
		"BN_FRIEND_ACCOUNT_ONLINE";
		"BN_FRIEND_ACCOUNT_OFFLINE";
	};
	onclick = function(self, button)
		if (button == "LeftButton") then
			ToggleFriendsFrame(1) 
		elseif (button == "RightButton") then
			if (DropDownList1:IsShown()) then
				DropDownList1:Hide()
			else
				EasyMenu(friendsMenuList(), friendsMenuFrame, "cursor", 0, 4, "MENU", 2)
				FixDropDown(self)
			end
		end
	end;
	onevent = function(self, event)
		local numBNetTotal, numBNetOnline = BNGetNumFriends()
		local numWoWTotal, numWoWOnline = GetNumFriends()
		local t = self.lib.text
		
		local BFriends = {}
		for i = 1, numBNetTotal do
			local presenceID, givenName, surname, toonName, toonID, client, isOnline, lastOnline, AFK, DND, broadcastText, noteText, isFriend, broadcastTime = BNGetFriendInfo(i)
		
			local hasFocus, toonName, client, realmName, weirdNumber, smallNumber, race, class, emptyString, zoneName, level, gameText, faction, trueBoolean, twentySixNumber = BNGetToonInfo(presenceID)
		
			if (isOnline) then
				if (client == BNET_CLIENT_WOW) and (isFriend) and (realmName == GetRealmName()) then
					local playerFaction = (UnitFactionGroup("player"))
					local sameFaction = ((playerFaction == "Alliance") and (faction == 1)) or ((playerFaction == "Horde") and (faction == 0))
					
					if (sameFaction) then
						BFriends[toonName] = true
					end
				end
			end
		end

		if (NumWoWOnline) and (NumWoWOnline > 0) then
			for i = 1, numWoWTotal do
				local name, level, class, area, connected, status, note = GetFriendInfo(i)
			
				if (BFriends[name]) then
					if (connected == 1) then 
						numWoWOnline = max(0, NumWoWOnline - 1)
					end
				end
			end
		end
		
		local text = t(FRIENDS .. ": ")
		text = text .. t((numBNetOnline + numWoWOnline), true)
		text = text .. t("/")
		text = text .. t((numBNetTotal + numWoWTotal), true)

		self.text:SetText(text)
	end;
	tooltip = "Friends";
})

local guildMenuFrame = CreateFrame("Frame", "GUIS-gUI_Panel_GuildDropdown", self, "UIDropDownMenuTemplate")
local guildMenuList = function() 
	local L = LibStub("gLocale-1.0"):GetLocale(addon)
	return {
	{
		text = GUILDCONTROL_OPTION7;
		notCheckable = true;
		notClickable = not(CanGuildInvite());
		keepShownOnClick = false;
		func = function()
			StaticPopup_Show("ADD_GUILDMEMBER")
		end
	};
	{
		text = GUILDCONTROL_OPTION9;
		notCheckable = true;
		notClickable = not(CanEditMOTD());
		keepShownOnClick = false;
		func = function()
			if not(GuildFrame) then 
				LoadAddOn("Blizzard_GuildUI") 
			end 
			if not(GuildFrame:IsShown()) then
				GuildFrame_Toggle() 
			end
			GuildTextEditFrame_Show("motd")
		end
	};
} end
RegisterPlugin("Guild" , {
	events = {
		"GUILD_ROSTER_UPDATE";
		"PLAYER_ENTERING_WORLD";
		"PLAYER_GUILD_UPDATE";
	};
	onclick = function(self, button)
		if (button == "LeftButton") then
			if (IsInGuild()) then 
				if not(GuildFrame) then 
					LoadAddOn("Blizzard_GuildUI") 
				end 
				GuildFrame_Toggle() 
			else 
				if not(LookingForGuildFrame) then 
					LoadAddOn("Blizzard_LookingForGuildUI") 
				end 
				LookingForGuildFrame_Toggle() 
			end
			
		elseif (button == "RightButton") then
			if (IsInGuild()) then 
				if (DropDownList1:IsShown()) then
					DropDownList1:Hide()
				else
					EasyMenu(guildMenuList(), guildMenuFrame, "cursor", 0, 4, "MENU", 2)
					FixDropDown(self)
				end
			end
		end
	end;
	oninit = function(self)
		GuildRoster()
		self.text:SetText(self.lib.text(self.GetGuildMembers(self)))
	end;
	--
	-- arg1 is nil if the event is triggered by GuildRoster()
	onevent = function(self, event, arg1) 
		if (event == "PLAYER_ENTERING_WORLD") then
			self:UnregisterEvent("PLAYER_ENTERING_WORLD")

			if (IsInGuild()) and not(GuildFrame) then 
				LoadAddOn("Blizzard_GuildUI") 
			end
		end
		
		self.text:SetText(self.lib.text(self.GetGuildMembers(self)))
	end;
	onupdate = function(self)
		GuildRoster()
	end;
	interval = 15;
	variables = {
		-- we need to pass a pointer to "self" here
		-- to get access to the Plugins libraries
		GetGuildMembers = function(self)
			local t = self.lib.text
			if (IsInGuild()) then
				local total, online = GetNumGuildMembers(true), 0

				for i = 1, total do
					online = online + ((( select(9, GetGuildRosterInfo(i))) == 1) and 1 or 0)
				end
				
				local text = t(GUILD .. ": ")
				text = text .. t(online, true)
				text = text .. t("/")
				text = text .. t(total, true)

				return text
			else
				return t(L["PANEL_GUILD_NONE"])
			end
		end
	};
	tooltip = "Guild";
})

RegisterPlugin("msfps" , {
	onupdate = function(self)
		local _, _, home, world = GetNetStats();
		local t = self.lib.text

		local text = t(home, true)
		text = text .. t(MILLISECONDS_ABBR)
		text = text .. t(" - ")
		text = text .. t(world, true)
		text = text .. t(MILLISECONDS_ABBR)
		text = text .. t(" - ")
		text = text .. t(floor(GetFramerate()), true)
		text = text .. t(FPS_ABBR)

		self.text:SetText(text)
	end;
	variables = {
		interval = 1.0;
	};
	tooltip = "Memory";
})

local bagMenuFrame = CreateFrame("Frame", "GUIS-gUI_Panel_BagsDropdown", self, "UIDropDownMenuTemplate")
local bagMenuList = function() 
	local module = loadModule("GUIS-gUI: Bags")

	-- just bail out if the bag module isn't loaded
	if not(module) then
		return
	end

	local L = LibStub("gLocale-1.0"):GetLocale(addon)
	return {
	{
		text = L["BAGS_TOGGLE_COMPRESSION"];
		checked = (GUIS_DB["bags"].compressemptyspace == 1);
		keepShownOnClick = true;
		isNotRadio = true;
		func = function()
			if (GUIS_DB["bags"].compressemptyspace == 0) then
				GUIS_DB["bags"].compressemptyspace = 1 
				module.updateAllContainers()
				
			elseif (GUIS_DB["bags"].compressemptyspace == 1) then
				GUIS_DB["bags"].compressemptyspace = 0 
				module.updateAllContainers()
			end
		end
	};
	{
		text = L["BAGS_TOGGLE_LOCK"];
		checked = (GUIS_DB["bags"].locked == 1);
		keepShownOnClick = true;
		isNotRadio = true;
		func = function()
			if (GUIS_DB["bags"].locked == 1) then
				GUIS_DB["bags"].locked = 0
			else
				GUIS_DB["bags"].locked = 1
			end
		end
	};
	{
		text = L["RESTACK_AUTO"];
		checked = (GUIS_DB["bags"].autorestack == 1);
		keepShownOnClick = true;
		isNotRadio = true;
		func = function()
			if (GUIS_DB["bags"].autorestack == 1) then
				GUIS_DB["bags"].autorestack = 0
			else
				GUIS_DB["bags"].autorestack = 1
			end
		end
	};
	{
		text = L["RESTACK_CRAFTING"];
		checked = (GUIS_DB["bags"].autorestackcrafted == 1);
		keepShownOnClick = true;
		isNotRadio = true;
		func = function()
			if (GUIS_DB["bags"].autorestackcrafted == 1) then
				GUIS_DB["bags"].autorestackcrafted = 0
			else
				GUIS_DB["bags"].autorestackcrafted = 1
			end
		end
	};
	
} end


RegisterPlugin("Bags" , {
	events = {
		"PLAYER_LOGIN"; 
		"BAG_UPDATE";
	};
	onclick = function(self, button) 
		if (button == "LeftButton") then
			OpenAllBags()
			
		elseif (button == "RightButton") then
			if (loadModule("GUIS-gUI: Bags")) then
				if (DropDownList1:IsShown()) then
					DropDownList1:Hide()
				else
					EasyMenu(bagMenuList(), bagMenuFrame, "cursor", 0, 4, "MENU", 2)
					FixDropDown(self)
				end
			end
		end
	end;
	onevent = function(self, event) 
		local t = self.lib.text
		
		local text = t(L["PANEL_BAGS"]..": ")
		text = text .. t (("[free:backpack+bags]"):tag(), true)
		text = text .. t("/")
		text = text .. t(("[max:backpack+bags]"):tag(), true)
		
		self.text:SetText(text)
	end;
	tooltip = "Bags";
})

RegisterPlugin("Time" , {
	onupdate = function(self) 
		self.text:SetText(self.lib.text(("[time]"):tag(), false))
	end;
	variables = {
		interval = 1.0;
	};
	tooltip = "Time";
})

RegisterPlugin("Gold" , {
	events = {
		"addon_LOADED";
		"PLAYER_MONEY";
		"PLAYER_ENTERING_WORLD";
		"PLAYER_LOGIN";
	};
	onclick = function(self) 
		ToggleCharacter("TokenFrame")
	end;
	onevent = function(self, event)
		self.text:SetText(self.lib.text(("[money]"):tag(), false))
	end;
	tooltip = "Currency";
})

RegisterPlugin("Mail" , {
	events = {
		"UPDATE_PENDING_MAIL";
	};
	
	onevent = function(self, event)
		self.GetMail(self)
	end;
	
	oninit = function(self) 
		self.GetMail(self)
	end;

	variables = {
		GetMail = function(self) 
			local t = self.lib.text
			if HasNewMail() then
				self.text:SetText(t(L["PANEL_MAIL_NEW"], false))
			else
				self.text:SetText(t(L["PANEL_MAIL_NONE"], false))
			end
		end
	};
--	tooltip = nil;
})

RegisterPlugin("MeleeAttackPower" , {
	onupdate = function(self) 
		local t = self.lib.text
		
		self.text:SetText(t(("[ap]"):tag(), true) .. t(" AP", false))
	end;
	variables = { 
		interval = 0.1;
	};
	tooltip = "MeleeStats";
})

RegisterPlugin("MeleeCritical" , {
	onupdate = function(self) 
		local t = self.lib.text
		
		self.text:SetText(t(("[mcrit]"):tag(), true) .. t("% " .. CRIT_ABBR, false))
	end;
	variables = { 
		interval = 0.1;
	};
	tooltip = "MeleeStats";
})

RegisterPlugin("MeleeHaste" , {
	onupdate = function(self) 
		local t = self.lib.text
		
		self.text:SetText(t(("[mhaste]"):tag(), true) .. t("% " .. SPELL_HASTE_ABBR, false))
	end;
	variables = { 
		interval = 0.1;
	};
	tooltip = "MeleeStats";
})

RegisterPlugin("RangedAttackPower" , {
	onupdate = function(self) 
		local t = self.lib.text
		
		self.text:SetText(t(("[rap]"):tag(), true) .. t(" RAP", false))
	end;
	variables = { 
		interval = 0.1;
	};
	tooltip = "RangedStats";
})

RegisterPlugin("RangedCritical" , {
	onupdate = function(self) 
		local t = self.lib.text
		
		self.text:SetText(t(("[rcrit]"):tag(), true) .. t("% " .. CRIT_ABBR, false))
	end;
	variables = { 
		interval = 0.1;
	};
	tooltip = "RangedStats";
})

RegisterPlugin("RangedHaste" , {
	onupdate = function(self) 
		local t = self.lib.text
		
		self.text:SetText(t(("[rhaste]"):tag(), true) .. t("% " .. SPELL_HASTE_ABBR, false))
	end;
	variables = { 
		interval = 0.1;
	};
	tooltip = "RangedStats";
})

RegisterPlugin("SpellPower" , {
	onupdate = function(self) 
		local t = self.lib.text
		
		self.text:SetText(t(("[sp]"):tag(), true) .. t(" SP", false))
	end;
	variables = {
		interval = 0.1;
	};
	tooltip = "CasterStats";
})

RegisterPlugin("SpellCritical" , {
	onupdate = function(self) 
		local t = self.lib.text
		
		self.text:SetText(t(("[scrit]"):tag(), true) .. t("% " .. CRIT_ABBR, false))
	end;
	variables = {
		interval = 0.1;
	};
	tooltip = "CasterStats";
})

RegisterPlugin("SpellHaste" , {
	onupdate = function(self) 
		local t = self.lib.text
		
		self.text:SetText(t(("[shaste]"):tag(), true) .. t("% " .. SPELL_HASTE_ABBR, false))
	end;
	variables = {
		interval = 0.1;
	};
	tooltip = "CasterStats";
})

RegisterPlugin("Experience" , {
	events = {
		"PLAYER_ALIVE";
		"PLAYER_ENTERING_WORLD";
		"PLAYER_LEVEL_UP";
		"PLAYER_LOGIN";
		"PLAYER_XP_UPDATE";
	};
	horizontalTooltip = true;
	onclick = nil;
	onevent = function(self, event, ...)
		if (event == "PLAYER_LEVEL_UP") then
			if (GetXPExhaustion() or 0) > 0 then 
				self.bar:SetStatusBarColor(unpack(self.XPRestedColor))
			else
				self.bar:SetStatusBarColor(unpack(self.XPColor))
			end

			self.bar:SetMinMaxValues(0, UnitXPMax("player"))
			self.bar:SetValue(UnitXP("player"))
			self.rested:SetMinMaxValues(0, UnitXPMax("player"))
			self.rested:SetValue(min(UnitXPMax("player"), UnitXP("player") + (GetXPExhaustion() or 0)))

			self.text:SetText(self.XPText())

			--[[
			if (UnitLevel("player") == MAX_PLAYER_LEVEL) or (IsXPUserDisabled()) then
				self:Hide()
			else
				self:Show()
			end
			]]--
		end
		
		if (event == "PLAYER_XP_UPDATE") or (event == "PLAYER_ALIVE") or (event == "PLAYER_ENTERING_WORLD") or (event == "PLAYER_LOGIN") then
			if (GetXPExhaustion() or 0) > 0 then 
				self.bar:SetStatusBarColor(unpack(self.XPRestedColor))
			else
				self.bar:SetStatusBarColor(unpack(self.XPColor))
			end
			
			self.bar:SetMinMaxValues(0, UnitXPMax("player"))
			self.bar:SetValue(UnitXP("player"))

			self.rested:SetMinMaxValues(0, UnitXPMax("player"))
			self.rested:SetValue(min(UnitXPMax("player"), UnitXP("player") + (GetXPExhaustion() or 0)))

			self.text:SetText(self.XPText())
		end
	end;
	oninit = function(self)
		self.background = self:CreateTexture(nil, "BACKGROUND")
		self.background:SetAllPoints(self:GetParent())
		self.background:SetTexture(M["StatusBar"]["StatusBar"])
		self.background:SetVertexColor(0.15, 0.15, 0.15, 1)
		
		self.rested = CreateFrame("StatusBar", nil, self)
		self.rested:SetStatusBarTexture(M["StatusBar"]["StatusBar"])
		self.rested:SetStatusBarColor(unpack(self.XPRestedBonusColor))
		self.rested:SetMinMaxValues(0, UnitXPMax("player"))
		self.rested:SetValue(min(UnitXPMax("player"), UnitXP("player") + (GetXPExhaustion() or 0)))
		self.rested:SetAllPoints(self:GetParent())

		self.bar = CreateFrame("StatusBar", nil, self.rested)
		self.bar:SetStatusBarTexture(M["StatusBar"]["StatusBar"])
		self.bar:SetAllPoints(self:GetParent())
		
		if (GetXPExhaustion() or 0) > 0 then 
			self.bar:SetStatusBarColor(unpack(self.XPRestedColor))
		else
			self.bar:SetStatusBarColor(unpack(self.XPColor))
		end

		self.bar:SetMinMaxValues(0, UnitXPMax("player"))
		self.bar:SetValue(UnitXP("player"));
		self.bar:EnableMouse(true)

		self.text:SetParent(self.bar)
		self.text:SetAllPoints(self.bar)
		self.text:SetDrawLayer("OVERLAY")
		self.text:SetText(self.XPText())
		self.text:SetFontObject(GUIS_NumberFontSmall or NumberFontNormalSmallGray)
		self.text:SetTextColor(1, 1, 1, 1)
		
		self.overlay:SetParent(self.bar)
		
		--[[
		if (UnitLevel("player") == MAX_PLAYER_LEVEL) or (IsXPUserDisabled()) then
			self:Hide()
		else
			self:Show()
		end
		]]--
	end;
	onupdate = nil;
	variables = {
		XPColor 			= { 0.4, 0.0, 0.4, 1.0 };
		XPRestedColor 		= { 0.3, 0.3, 0.8, 1.0 };
		XPRestedBonusColor 	= { 0.1, 0.1, 0.4, 1.0 };
		
		XPText 	= function() 
			return ("[shortvalue:%d]"):format(UnitXP("player")):tag().." / "..("[shortvalue:%d]"):format(UnitXPMax("player")):tag().." - "..(floor(UnitXP("player") / UnitXPMax("player") * 100).."%") 
		end;
	};
	tooltip = "Experience";
})

RegisterPlugin("Reputation" , {
	events = {
		"UPDATE_FACTION";
	};
	onevent = function(self, event, arg1, arg2)
		local RPName, RPStanding, RPMin, RPMax, RPValue = GetWatchedFactionInfo()

		self.bar:SetStatusBarColor(unpack(self.RPColor()))
		self.bar:SetMinMaxValues(RPMin, RPMax)
		self.bar:SetValue(RPValue)
		self.text:SetText(self.RPText())
	end;
	onclick = function(self, button)
		if (button == "LeftButton") then
			ToggleCharacter("ReputationFrame")
			
			local RPName, RPStanding, RPMin, RPMax, RPValue = GetWatchedFactionInfo()
			
			if (RPName) then
				local headersToClose = {}
				local headerID, headerName, headerToExpand, headerToExpandName, faction
				
				ExpandAllFactionHeaders()
				
				for i = 1, GetNumFactions() do
					local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild = GetFactionInfo(i)

					if (isHeader) then
						headerID = i
						headerName = name
					end
					
					if (RPName == name) then
						faction = i
						headerToExpand = headerID
						headerToExpandName = headerName
					end
				end
				
				SetSelectedFaction(faction)
				ReputationDetailFrame:Show()
				ReputationFrame_Update()
			end
		end
	end;
	oninit = function(self)
		self.background = self:CreateTexture(nil, "BACKGROUND")
		self.background:SetAllPoints(self:GetParent())
		self.background:SetTexture(M["StatusBar"]["StatusBar"])
		self.background:SetVertexColor(0.15, 0.15, 0.15, 1)

		local RPName, RPStanding, RPMin, RPMax, RPValue = GetWatchedFactionInfo()
		self.bar = CreateFrame("StatusBar", nil, self)
		self.bar:SetStatusBarTexture(M["StatusBar"]["StatusBar"])
		self.bar:SetStatusBarColor(unpack(self.RPColor()))
		self.bar:SetAllPoints(self:GetParent())
		self.bar:SetMinMaxValues(RPMin, RPMax)
		self.bar:SetValue(RPValue);
		self.bar:EnableMouse(true)
		
		self.text:SetParent(self.bar)
		self.text:SetDrawLayer("OVERLAY")
		self.text:SetText(self.RPText())
		self.text:SetFontObject(GUIS_NumberFontSmall or NumberFontNormalSmallGray)
		self.text:SetTextColor(1, 1, 1, 1)
		
		hooksecurefunc("ReputationWatchBar_Update", function() 
			self:ForceEvent("UPDATE_FACTION") 
		end)
	end;
	onupdate = nil;
	horizontalTooltip = true;
	variables = {
		RPText = function() 
			local RPName, RPStanding, RPMin, RPMax, RPValue = GetWatchedFactionInfo()
			if (not RPName) then 
				return "" 
			end
			return (RPName and ("[shortvalue:%d]"):format(RPValue - RPMin):tag().." / "..("[shortvalue:%d]"):format(RPMax - RPMin):tag().." - "..floor((RPValue - RPMin) / (RPMax - RPMin) * 100)).."%" or ""
		end;
		RPColor = function() 
			local RPName, RPStanding, RPMin, RPMax, RPValue = GetWatchedFactionInfo()
			if not(RPName) then 
				return { 0, 0, 0, 1 }
			end
			return { FACTION_BAR_COLORS[RPStanding].r, FACTION_BAR_COLORS[RPStanding].g, FACTION_BAR_COLORS[RPStanding].b, 1 }
		end;
	};
	tooltip = "Reputation";
})

local durabilityMenuFrame = CreateFrame("Frame", "GUIS-gUI_Panel_MerchantOptionsDropdown", self, "UIDropDownMenuTemplate")
durabilityMenuList = function() 
	local L = LibStub("gLocale-1.0"):GetLocale(addon)
	return {
	{
		text = L["MERCHANT_OPTIONS_AUTOREPAIR"]; 
		checked = GUIS_DB["merchant"].autorepair;
		keepShownOnClick = true;
		isNotRadio = true;
		func = function()
			GUIS_DB["merchant"].autorepair = not(GUIS_DB["merchant"].autorepair)
		end
	};
	{
		text = L["MERCHANT_OPTIONS_GUILDREPAIR"]; 
		checked = GUIS_DB["merchant"].guildrepair;
		keepShownOnClick = true;
		isNotRadio = true;
		func = function()
			GUIS_DB["merchant"].guildrepair = not(GUIS_DB["merchant"].guildrepair)
		end
	};
	{
		text = L["MERCHANT_OPTIONS_AUTOSELL"]; 
		checked = GUIS_DB["merchant"].autosell;
		keepShownOnClick = true;
		isNotRadio = true;
		func = function()
			GUIS_DB["merchant"].autosell = not(GUIS_DB["merchant"].autosell)
		end
	};
	{
		text = L["MERCHANT_OPTIONS_DETAILS"]; 
		checked = GUIS_DB["merchant"].detailedreport;
		keepShownOnClick = true;
		isNotRadio = true;
		func = function()
			GUIS_DB["merchant"].detailedreport = not(GUIS_DB["merchant"].detailedreport)
		end
	};
} end

RegisterPlugin("Durability" , {
	events = {
		"UPDATE_INVENTORY_DURABILITY";
		"PLAYER_LOGIN";
		"PLAYER_ENTERING_WORLD";
	};
	onevent = function(self, event)
		if (event == "UPDATE_INVENTORY_DURABILITY") then
			local t = self.lib.text
		
			self.durability.current = 0
			self.durability.total = 0

			for _, slot in ipairs(self.slots) do
				local item = _G["Character" .. slot]
				local current, total = GetInventoryItemDurability(item:GetID())
				
				if current and total then
					self.durability.current = self.durability.current + current
					self.durability.total = self.durability.total + total
				end
			end
			
			local percent = 100
			if (self.durability.total > 0) then
				percent = max(0, self.durability.current / self.durability.total * 100)
			end
			
			local text = t(ARMOR .. ": ")
			text = text .. t("%d", true):format(percent)
			text = text .. t("%")
	
			self.text:SetText(text)
		end
	end;
	onclick = function(self, button)
		if (button == "RightButton") then
			if (loadModule("GUIS-gUI: Merchant")) then
				if (DropDownList1:IsShown()) then
					DropDownList1:Hide()
				else
					EasyMenu(durabilityMenuList(), durabilityMenuFrame, "cursor", 0, 4, "MENU", 2)
					FixDropDown(self)
				end
			end
		end
	end;
	variables = {
		slots = { "HeadSlot", "ShoulderSlot", "ChestSlot", "WaistSlot", "WristSlot", "HandsSlot", "LegsSlot", "FeetSlot", "MainHandSlot", "SecondaryHandSlot", "RangedSlot" };		
		durability = { current = 0; total = 0; };
		tooltip = "Durability";
	};
})

