﻿assert(LibStub("gFrameHandler-1.0"), "Couldn't find an instance of gFrameHandler-1.0")
assert(LibStub("gString-1.0"), "Couldn't find an instance of gString-1.0")
assert(LibStub("gPanel-2.0"), "Couldn't find an instance of gPanel-2.0")

local RegisterTooltip = function(...) LibStub("gPanel-2.0"):RegisterTooltip(...) end

-- Lua API
local format, gsub = string.format, string.gsub
local ipairs, pairs, select = ipairs, pairs, select
local abs, floor, log, max = math.abs, math.floor, math.log, math.max
local collectgarbage = collectgarbage

local RAID_CLASS_COLORS = LibStub("gPanel-2.0"):GetColor("RAID_CLASS_COLORS")

local L = LibStub("gPanel-2.0").L
L["PANEL_BAGS"] 					= L["PANEL_BAGS"] 					or "Bags"
L["PANEL_LEFTCLICKFORBAGS"] 		= L["PANEL_LEFTCLICKFORBAGS"] 		or "Left-Click to open all bags"
L["PANEL_RESTED"] 					= L["PANEL_RESTED"] 				or "Rested"
L["PANEL_RIGHTCLICKFOROPTIONS"] 	= L["PANEL_RIGHTCLICKFOROPTIONS"] 	or "Right-Click for options"
L["PANEL_TOTAL"] 					= L["PANEL_TOTAL"] 					or "Total"
L["PANEL_TOTAL_USAGE"] 				= L["PANEL_TOTAL_USAGE"] 			or "Total usage" 
L["PANEL_TRACKED_CURRENCIES"] 		= L["PANEL_TRACKED_CURRENCIES"] 	or "Tracked Currencies"
L["PANEL_UNLISTED_ADDONS"] 			= L["PANEL_UNLISTED_ADDONS"] 		or "Unlisted AddOns"

------------------------------------------------------------------------------------------------------------
-- 	Tooltips
------------------------------------------------------------------------------------------------------------
--[[
	RegisterTooltip("Name_of_Tip", function(self)
		-- set up tooltip, positioning is optional as this will be done by library
		-- show the tip. not done by library
	end
]]--

RegisterTooltip("Bags", function(self)
	GameTooltip:AddLine(L["PANEL_LEFTCLICKFORBAGS"])

	local module = LibStub("gCore-3.0"):GetModule("GUIS-gUI: Bags")
	local dead = LibStub("gDB-1.0"):GetDataBase("GUIS-gUI: Functions").kill("GUIS-gUI: Bags")

	if (module) and not(dead) then
		GameTooltip:AddLine(L["PANEL_RIGHTCLICKFOROPTIONS"])
	end
	
	GameTooltip:Show()
end)

RegisterTooltip("Friends", function(self)
	ShowFriends()
	
	local numBNetTotal, numBNetOnline = BNGetNumFriends()
	local numberOfFriends = GetNumFriends()
	
	local friendsonline = 0

	if ((numberOfFriends + numBNetTotal) > 0) then
		GameTooltip:AddLine("|cFFFFFFFF" .. FRIENDS .. "|r", 1,1,1)
		GameTooltip:AddLine(" ")
	end
	
	local BFriends = {}
	for i = 1, numBNetTotal do
		local presenceID, givenName, surname, toonName, toonID, client, isOnline, lastOnline, AFK, DND, broadcastText, noteText, isFriend, broadcastTime = BNGetFriendInfo(i)
	
		local hasFocus, toonName, client, realmName, weirdNumber, smallNumber, race, class, emptyString, zoneName, level, gameText, faction, trueBoolean, twentySixNumber = BNGetToonInfo(presenceID)
	
		if (isOnline) then
			local left, right = "", ""
			local t = self.lib.text
			
			if (client == BNET_CLIENT_WOW) then
				if (isFriend) and (realmName == GetRealmName()) then
					local playerFaction = (UnitFactionGroup("player"))
					local sameFaction = ((playerFaction == "Alliance") and (faction == 1)) or ((playerFaction == "Horde") and (faction == 0))
					
					if (sameFaction) then
						BFriends[toonName] = true
					end
				end

				for k,v in pairs(LOCALIZED_CLASS_NAMES_MALE) do 
					if (class == v) then 
						class = k 
					end 
				end

				if (GetLocale() ~= "enUS") and (GetLocale() ~= "enGB") then -- feminine class localization
					for k,v in pairs(LOCALIZED_CLASS_NAMES_FEMALE) do 
						if (class == v) then 
							class = k 
						end 
					end
				end
				
				local classc = (RAID_CLASS_COLORS)[class] or { r = 0.35, g = 0.75, b = 1}
				local lvlcol = (level) and (tonumber(level)) and GetQuestDifficultyColor(tonumber(level)) or { r = 1, g = 1, b = 1 }
				local zonecol = (GetRealZoneText() == zoneName) and {r = 0, g = 1, b = 0} or {r = 1, g = 1, b = 1}
				
				left = "|cFF" .. format("%02x%02x%02x", lvlcol.r*255, lvlcol.g*255, lvlcol.b*255) .. level .. "|r"
				left = left .. "|cFFFFFFFF:|r "
				left = left .. toonName .. " " .. (AFK and CHAT_FLAG_AFK or "") .. (DND and CHAT_FLAG_DND or "")  
				left = left .. t("(" .. givenName .. " " .. surname .. ")", true)

				if (noteText) and (noteText ~= "") then 
					left = left.. t("(" .. noteText .. ")", true) 
				end

				if (realmName == GetRealmName()) then
					right = zoneName
				else
					right = gameText
				end
				GameTooltip:AddDoubleLine(left, right, classc.r, classc.g, classc.b, zonecol.r, zonecol.g, zonecol.b)
			else
				left = givenName .. " " .. surname .. (toonName and "("..toonName..")" or "")..(noteText and "|cFFFFFFFF ("..noteText..") |r")
				right = gameText .. "(" .. client .. ")"
				GameTooltip:AddDoubleLine(left, right)
			end
		end
	end
	
	if (numberOfFriends > 0) then
		for i = 1, numberOfFriends do
			local name, level, class, area, connected, status, note = GetFriendInfo(i)
			if (connected == 1) and not(BFriends[name]) then 
				friendsonline = friendsonline + 1

				for k,v in pairs(LOCALIZED_CLASS_NAMES_MALE) do 
					if (class == v) then 
						class = k 
					end 
				end
				
				if (GetLocale() ~= "enUS") then -- feminine class localization
					for k,v in pairs(LOCALIZED_CLASS_NAMES_FEMALE) do if class == v then class = k end end
				end
				
				local classc, lvlcol, zonecol = (RAID_CLASS_COLORS)[class],GetQuestDifficultyColor(level), (GetRealZoneText() == area) and {r = 0, g = 1, b = 0} or {r = 1, g = 1, b = 1}
				local left = "|cFF"..format("%02x%02x%02x", lvlcol.r*255, lvlcol.g*255, lvlcol.b*255)..level.."|r|cFFFFFFFF:|r "..name.." "..status

				if (note) then 
					left = left .. "|cFFFFFFFF (" .. note .. ") |r" 
				end

				local right = area

				GameTooltip:AddDoubleLine(left, right, classc.r, classc.g, classc.b, zonecol.r, zonecol.g, zonecol.b)
			end
		end
		
	end
	
	friendsonline = friendsonline + numBNetOnline
	if (friendsonline > 0) then 
		GameTooltip:Show()
	end
end)

RegisterTooltip("Guild", function(self) 
	local numberofguildies = GetNumGuildMembers(false)
	if numberofguildies > 0 then

		GameTooltip:AddLine(GetGuildInfo("player") or "N/A", 1, 1, 1)

		local gmotd = GetGuildRosterMOTD()
		if (gmotd and not gmotd == "") then
			GameTooltip:AddLine(gmotd)
		end

		GameTooltip:AddLine(" ")

		for i = 1, numberofguildies do
			local name, rank, rankIndex, level, class, zone, note, officernote, connected, status, classFileName = GetGuildRosterInfo(i)
			if connected == 1 then  
				local zonecolor = {r = 1, g = 1, b = 1}
				if GetRealZoneText() == zone then 
					zonecolor = {r = 0, g = 1, b = 0} 
				end
				
				local lvlcol = GetQuestDifficultyColor(level)
				
				local left = "|cFF"..format("%02x%02x%02x", lvlcol.r*255, lvlcol.g*255, lvlcol.b*255)..level.."|r|cFFFFFFFF:|r "..name.." "..status
				if note ~= "" then 
					left = left.."|cFFFFFFFF ("..note..") |r" 
				end
				
				local right = zone
				
				GameTooltip:AddDoubleLine(left, right, RAID_CLASS_COLORS[classFileName].r, RAID_CLASS_COLORS[classFileName].g, RAID_CLASS_COLORS[classFileName].b, zonecolor.r, zonecolor.g, zonecolor.b)
			end
		end

		GameTooltip:Show()
	end
end)

RegisterTooltip("Memory", function(self) 
	collectgarbage("collect")
	UpdateAddOnMemoryUsage()
	
	local memory = {}
	local total = 0
	
	-- Memory Usage
	local formatMemory = function(bytes)
			bytes = bytes * 1024
			local s, e = {'B', 'Kb', 'MB', 'GB', 'TB'}, floor(abs(log(bytes+1)/log(1024)))
			return format("%.1f "..s[e+1], (bytes/1024^e))
	end

	-- Store addon memory usage
	for i = 1, GetNumAddOns() do
		local mem = GetAddOnMemoryUsage(i)
		memory[i] = { select(2, GetAddOnInfo(i)), mem, IsAddOnLoaded(i) }
		total = total + mem
	end
	
	-- Sort the addons by memory usage
	sort(memory, function(a, b)
		if a and b then
			return a[2] > b[2]
		end
	end)
	
	-- Setup tooltip
	
	
	GameTooltip:AddDoubleLine("|cFFFFFFFF"..L["PANEL_TOTAL_USAGE"].."|r", formatMemory(total), 1, 1, 1, 1, 1, 1)
	GameTooltip:AddLine(" ")
	
	-- limit the number of addons displayed
	local maxShown = 15
	local Shown = 0
	local leftOvers = total
	
	for i = 1, #memory do
		if memory[i][3] then 
			if Shown < maxShown then
				local red = memory[i][2] / total * 2
				local green = 1 - red
				GameTooltip:AddDoubleLine(memory[i][1], formatMemory(memory[i][2]), 1, 1, 1, red, green+1, 0)
				
				leftOvers = leftOvers - memory[i][2]
			end
			Shown = Shown + 1
		end
	end
	
	if (Shown > maxShown) then
		local red = leftOvers / total * 2
		local green = 1 - red
--		GameTooltip:AddLine("+" .. (Shown - maxShown) .. " " .. ADDONS)
		GameTooltip:AddDoubleLine("..." .. (Shown - maxShown) .. L["PANEL_UNLISTED_ADDONS"], formatMemory(leftOvers), 1, 1, 1, red, green + 1, 0)
	end
	
	GameTooltip:Show()
end)

RegisterTooltip("Time", function(self) 
	GameTooltip:AddLine("|cFFFFFFFF"..TIMEMANAGER_TOOLTIP_TITLE.."|r")
	GameTooltip:AddDoubleLine(TIMEMANAGER_TOOLTIP_LOCALTIME, ("[time:local]"):tag())
	GameTooltip:AddDoubleLine(TIMEMANAGER_TOOLTIP_REALMTIME, ("[time:game]"):tag())
	GameTooltip:Show()
end)

RegisterTooltip("Currency", function(self) 
	local faction = (UnitFactionGroup("player"))
	local showtip
	
	for i = 1, GetNumWatchedTokens()  do
		local name, count, icon = GetBackpackCurrencyInfo(i)
		if count then
			local tcount = count
			icon = "  |T"..icon..":0:0:0:0|t"
			tcount = "|cFF"..((count > 0) and "FFFFFF" or "666666")..tcount.."|r"
			if (i == 1) then
				GameTooltip:AddLine(L["PANEL_TRACKED_CURRENCIES"], 1, 1, 1)
				GameTooltip:AddLine(" ")
			end
			showtip = true
			local left = "|cFF"..((count > 0) and "FFAA00" or "666666")..name.."|r"
			local right = tcount.."  "..icon
			GameTooltip:AddDoubleLine(left, right)
		end
	end
	GameTooltip:Show()
end)

local dummy = CreateFrame("GameTooltip")
RegisterTooltip("Durability", function(self)

	local slots = { "HeadSlot", "ShoulderSlot", "ChestSlot", "WaistSlot", "WristSlot", "HandsSlot", "LegsSlot", "FeetSlot", "MainHandSlot", "SecondaryHandSlot", "RangedSlot" }		

	GameTooltip:AddLine(DURABILITY, 1, 1, 1)
	GameTooltip:AddLine(" ")
	
	local eqcurrent, eqtotal, eqcost = 0, 0, 0
	for _, slot in ipairs(slots) do
		local item = _G["Character" .. slot]
		local exist, _, cost = dummy:SetInventoryItem("player", item:GetID())
		local current, total = GetInventoryItemDurability(item:GetID())

		if exist and cost and cost > 0 then
			eqcost = eqcost + cost
		end

		if current and total then
			eqcurrent = eqcurrent + current
			eqtotal = eqtotal + total
		end
	end

	local bgcurrent, bgtotal, bgcost = 0, 0, 0
	for bag = 0, 4 do
		for slot = 1, GetContainerNumSlots(bag) do
			local cooldown, cost = dummy:SetBagItem(bag, slot)
			local current, total = GetContainerItemDurability(bag, slot)

			if cost and cost > 0 then
				bgcost = bgcost + cost
			end

			if current and total and current > 0 and total > 0 then
				bgcurrent = bgcurrent + current
				bgtotal = bgtotal + total
			end
		end
	end
	bgcost = max(bgcost, 0)
	
	-- division by zero cause funny funny results
	-- if total durabilities are 0, there are no items to show
	bgcurrent 	= bgtotal == 0 and 1 or bgcurrent
	bgtotal 	= bgtotal == 0 and 1 or bgtotal
	eqcurrent 	= eqtotal == 0 and 1 or eqcurrent
	eqtotal 	= eqtotal == 0 and 1 or eqtotal

	GameTooltip:AddDoubleLine("|cFFFFFFFF"..CURRENTLY_EQUIPPED..": (|r" .. ("%d"):format(eqcurrent / eqtotal * 100) .. "|cFFFFFFFF%)|r", ("|cFFFFFFFF[money:" .. eqcost .. "]|r"):tag())
	GameTooltip:AddDoubleLine("|cFFFFFFFF"..L["PANEL_BAGS"]..": (|r" .. ("%d"):format(bgcurrent / bgtotal * 100) .. "|cFFFFFFFF%)|r", ("|cFFFFFFFF[money:" .. bgcost .. "]|r"):tag())
	GameTooltip:AddDoubleLine("|cFFFFFFFF"..L["PANEL_TOTAL"]..": (|r" .. ("%d"):format((bgcurrent + eqcurrent) / (bgtotal + eqtotal) * 100) .. "|cFFFFFFFF%)|r", ("|cFFFFFFFF[money:" .. (eqcost + bgcost) .. "]|r"):tag())
	
	local module = LibStub("gCore-3.0"):GetModule("GUIS-gUI: Merchant")
	local dead = LibStub("gDB-1.0"):GetDataBase("GUIS-gUI: Functions").kill("GUIS-gUI: Merchant")

	if (module) and not(dead) then
		GameTooltip:AddLine(" ")
		GameTooltip:AddLine(L["PANEL_RIGHTCLICKFOROPTIONS"])
	end

	GameTooltip:Show()
	
end)

RegisterTooltip("MeleeStats", function(self) 
	mep, oep = GetExpertisePercent()
	me, oe = GetExpertise()
	
	GameTooltip:AddDoubleLine(ITEM_MOD_MELEE_ATTACK_POWER_SHORT..":", ("[ap]"):tag().." ("..("[mcrit]"):tag().."% "..CRIT_ABBR..")", 1,1,1, 1,1,1)
	GameTooltip:AddDoubleLine(ITEM_MOD_HASTE_MELEE_RATING_SHORT..":", GetCombatRating(18).." ("..("[mhaste]"):tag().."%)")
	GameTooltip:AddDoubleLine(ITEM_MOD_EXPERTISE_RATING_SHORT..":", GetCombatRating(24).." ("..format("%.2f", mep).."% / "..format("%.2f", oep).."%)")
	GameTooltip:AddDoubleLine(ITEM_MOD_HIT_MELEE_RATING_SHORT..":", GetCombatRating(6).." ("..format("%.2f", GetCombatRatingBonus(6)).."%)")

	GameTooltip:Show()
end)

RegisterTooltip("TankStats", function(self) 
end)

RegisterTooltip("CasterStats", function(self) 
	local sp, spcrit = {}, {}
	local currentsp, currentspcrit = GetSpellBonusHealing(), 0
	local iconpath = " |TInterface\\PaperDollInfoFrame\\SpellSchoolIcon"
	local iconpath2 = " |TInterface\\Icons\\Spell_Nature_HealingTouch"
	local iconcrop = ":16:16:0:0|t"

	sp[0] = GetSpellBonusHealing()
	for spid = 2,7 do 
		sp[spid] = GetSpellBonusDamage(spid)
		spcrit[spid] = GetSpellCritChance(spid)
		currentsp = max(currentsp, sp[spid])
		currentspcrit = max(currentspcrit, spcrit[spid])
	end
	
	-- ITEM_MOD_HIT_SPELL_RATING_SHORT -- spell hit

	GameTooltip:AddDoubleLine(ITEM_MOD_SPELL_POWER_SHORT..":", currentsp.." ("..format("%.2f", currentspcrit).."% "..CRIT_ABBR..")", 1,1,1, 1,1,1)
	GameTooltip:AddDoubleLine(ITEM_MOD_HASTE_SPELL_RATING_SHORT..":", format("%.2f", GetCombatRatingBonus(20)).."%")
	GameTooltip:AddDoubleLine(ITEM_MOD_SPELL_PENETRATION_SHORT..":", GetSpellPenetration())
	GameTooltip:AddDoubleLine(iconpath2..iconcrop.." "..BONUS_HEALING..": ", sp[0])
	GameTooltip:AddDoubleLine(iconpath.."2"..iconcrop.." "..SPELL_SCHOOL1_CAP..": ", sp[2])
	GameTooltip:AddDoubleLine(iconpath.."3"..iconcrop.." "..SPELL_SCHOOL2_CAP..": ", sp[3])
	GameTooltip:AddDoubleLine(iconpath.."4"..iconcrop.." "..SPELL_SCHOOL3_CAP..": ", sp[4])
	GameTooltip:AddDoubleLine(iconpath.."5"..iconcrop.." "..SPELL_SCHOOL4_CAP..": ", sp[5])
	GameTooltip:AddDoubleLine(iconpath.."6"..iconcrop.." "..SPELL_SCHOOL5_CAP..": ", sp[6])
	GameTooltip:AddDoubleLine(iconpath.."7"..iconcrop.." "..SPELL_SCHOOL6_CAP..": ", sp[7])

	GameTooltip:Show()
end)

RegisterTooltip("RangedStats", function(self) 
	GameTooltip:AddDoubleLine(ITEM_MOD_RANGED_ATTACK_POWER_SHORT..":", ("[rap]"):tag().." ("..("[rcrit]"):tag().."% "..CRIT_ABBR..")", 1,1,1, 1,1,1)
	GameTooltip:AddDoubleLine(ITEM_MOD_HASTE_RANGED_RATING_SHORT..":", GetCombatRating(19).." ("..("[rhaste]"):tag().."%)")
	GameTooltip:AddDoubleLine(ITEM_MOD_HIT_RANGED_RATING_SHORT..":", GetCombatRating(13).." ("..format("%.2f", GetCombatRatingBonus(13)).."%)")

	GameTooltip:Show()
end)

RegisterTooltip("Experience", function(self)
	local reststate, xpgain

	if (GetXPExhaustion() or 0) > 0 then 
		reststate = ERR_EXHAUSTION_RESTED
		xpgain = 200
		GameTooltip:AddLine("|cFFFFFFFF"..UnitXP("player").." / "..UnitXPMax("player").." - "..(floor(UnitXP("player")/UnitXPMax("player")*100).."%").." ("..floor((GetXPExhaustion() or 0)/UnitXPMax("player")*100).."% "..L["PANEL_RESTED"]..")|r|n")
	else
		reststate = ERR_EXHAUSTION_NORMAL
		xpgain = 100
		GameTooltip:AddLine("|cFFFFFFFF"..UnitXP("player").." / "..UnitXPMax("player").." - "..(floor(UnitXP("player")/UnitXPMax("player")*100).."%").."|r|n")
	end

	GameTooltip:AddLine(EXHAUST_TOOLTIP1:format(reststate, xpgain))
	GameTooltip:Show()
end)

RegisterTooltip("Reputation", function(self)
	local RPName, RPStanding, RPMin, RPMax, RPValue = GetWatchedFactionInfo()
	if RPName then
		GameTooltip:AddLine(RPName.." (".._G["FACTION_STANDING_LABEL"..RPStanding]..")", 1, 1, 1)
		GameTooltip:AddLine((RPValue - RPMin).." / "..(RPMax - RPMin).." - ("..floor((RPValue - RPMin) / (RPMax - RPMin) * 100).."%)") 
		GameTooltip:Show()
	end
end)
