﻿local addon, ns = ...

local MAJOR, MINOR = "gPanel-2.0", 3
local gPanel, oldminor = LibStub:NewLibrary(MAJOR, MINOR)

if (not gPanel) then return end 

assert(LibStub("gFrameHandler-1.0"), MAJOR .. " couldn't find an instance of gFrameHandler-1.0")
assert(LibStub("gString-1.0"), MAJOR .. " couldn't find an instance of gString-1.0")
assert(LibStub("gMedia-3.0"), MAJOR .. " couldn't find an instance of gMedia-3.0")

-- Lua API
local type = type
local strmatch = string.match
local floor, min, max = math.floor, math.min, math.max
local tonumber, tostring = tonumber, tostring
local pairs, unpack = pairs, unpack
local getmetatable, setmetatable = getmetatable, setmetatable

-- do we really need this?
local M = LibStub("gMedia-3.0")

-- frames used for panels and plugins templates, and event handlers
gPanel.PANEL = gPanel.PANEL or CreateFrame("Frame", "gPanel20PanelFrame", UIParent)
gPanel.PLUGIN = gPanel.PLUGIN or CreateFrame("Frame", "gPanel20PluginFrame", UIParent)

gPanel.db = gPanel.db or {}
gPanel.Lib = gPanel.Lib or {}
gPanel.Panels = gPanel.Panels or {}
gPanel.Plugins = gPanel.Plugins or {}
gPanel.Templates = gPanel.Templates or {}
gPanel.Tooltips = gPanel.Tooltips or {}
gPanel.L = gPanel.L or {}

local PANEL = gPanel.PANEL
local PLUGIN = gPanel.PLUGIN

-- this is kind of silly, we can surely find a better way
local ALLOWEDJUSTIFICATION = { 
	TOP 	= true; 
	BOTTOM 	= true; 
	RIGHT 	= true; 
	LEFT 	= true; 
	CENTER 	= true; 
	FULL 	= true; 
}

-- our local color table
local C = {}

C["index"] 			= { 1.00; 1.00; 1.00; 1.00 }
C["value"] 			= { 1.00; 0.82; 0.00; 1.00 }
C["background"] 	= { 0.00; 0.00; 0.00; 1.00 }
C["border"] 		= { 0.15; 0.15; 0.15; 1.00 }
C["overlay"] 		= { 0.25; 0.25; 0.25; 1.00 }
C["shadow"] 		= { 0.00; 0.00; 0.00; 3/4 }

-- grabbing WoWs default colors here
C["RAID_CLASS_COLORS"] = setmetatable({}, { __index = RAID_CLASS_COLORS })
C["FACTION_BAR_COLORS"] = setmetatable({}, { __index = FACTION_BAR_COLORS })

-- textures and colors used by the panels
gPanel.db.texture = M["Background"]["Blank"]
gPanel.db.vertexcolor = C["overlay"]
gPanel.db.backdropcolor = C["background"]
gPanel.db.backdropbordercolor = C["border"]
gPanel.db.backdrop = M["Backdrop"]["Blank-Inset"]
gPanel.db.fontobject = Tooltip_Small

------------------------------------------------------------------------------------------------------------
-- 	Local functions
------------------------------------------------------------------------------------------------------------
--
-- embeds every key from source into target
--
local Embed = function(target, source)
	if not target or not source then return end
	if type(source) ~= "table" or type(target) ~= "table" then return end 

	for i,v in pairs(source) do
		target[i] = v
	end
end

--
-- transforms X into actual pixels, dependant on the 'parent' scale
local scale = function(x, parent)
	return x and floor(x * (768/strmatch(GetCVar("gxResolution"), "%d+x(%d+)") / (parent or UIParent):GetEffectiveScale()) + 0.5)
end

------------------------------------------------------------------------------------------------------------
-- 	Functions available to all gPanel.Plugins
------------------------------------------------------------------------------------------------------------
gPanel.Lib["text"] = function(text, isValue)
	if type(text) == "number" then
		text = tostring(text)
	elseif type(text) ~= "string" then 
		text = ""
	end

	local color
	if isValue then
		color = RGBToHex(C["value"][1], C["value"][2], C["value"][3])
	else
		color = RGBToHex(C["index"][1], C["index"][2], C["index"][3])
	end
	
	return "|cFF" .. color .. text .. "|r"
end

------------------------------------------------------------------------------------------------------------
-- 	Backdrops
------------------------------------------------------------------------------------------------------------
--
-- SetBackdrop has some problems with texture coordinates, 
-- so we're making a better version here
--
-- all parameters match the WoWLua counterparts 
local ResizeUIBackdrop = function(bd, w, h)
	local edgeSize = bd.trueScale and scale(bd.edgeSize) or bd.edgeSize
	local tileSize = bd.trueScale and scale(bd.tileSize) or bd.tileSize
	local left = bd.trueScale and scale(bd.insets.left) or bd.insets.left
	local right = bd.trueScale and scale(bd.insets.right) or bd.insets.right
	local top = bd.trueScale and scale(bd.insets.top) or bd.insets.top
	local bottom = bd.trueScale and scale(bd.insets.bottom) or bd.insets.bottom
	
	local w = max(1, w or ( ((bd:GetRight() or 0) - (bd:GetLeft()or 0)) or 0) - (bd.edgeSize or 0)*2)
	local h = max(1, h or ( ((bd:GetTop() or 0) - (bd:GetBottom()or 0)) or 0) - (bd.edgeSize or 0)*2)

	bd.TEXTURES.BG:SetTexture(bd.bgFile, bd.tile)
	bd.TEXTURES.BG:ClearAllPoints()
	bd.TEXTURES.BG:SetPoint("TOPLEFT", bd, "TOPLEFT", (left or 0), -(top or 0))
	bd.TEXTURES.BG:SetPoint("BOTTOMRIGHT", bd, "BOTTOMRIGHT", -(right or 0), (bottom or 0))

	bd.TEXTURES.TOPLEFT:SetTexture(bd.edgeFile)
	bd.TEXTURES.TOPLEFT:ClearAllPoints()
	bd.TEXTURES.TOPLEFT:SetPoint("TOPLEFT", bd)
	bd.TEXTURES.TOPLEFT:SetSize((edgeSize or 1/1e4), (edgeSize or 1/1e4))
	bd.TEXTURES.TOPLEFT:SetTexCoord(4/8, 5/8, 0, 1)

	bd.TEXTURES.TOPRIGHT:SetTexture(bd.edgeFile)
	bd.TEXTURES.TOPRIGHT:ClearAllPoints()
	bd.TEXTURES.TOPRIGHT:SetPoint("TOPRIGHT", bd)
	bd.TEXTURES.TOPRIGHT:SetSize((edgeSize or 1/1e4), (edgeSize or 1/1e4))
	bd.TEXTURES.TOPRIGHT:SetTexCoord(5/8, 6/8, 0, 1)

	bd.TEXTURES.BOTTOMLEFT:SetTexture(bd.edgeFile)
	bd.TEXTURES.BOTTOMLEFT:ClearAllPoints()
	bd.TEXTURES.BOTTOMLEFT:SetPoint("BOTTOMLEFT", bd)
	bd.TEXTURES.BOTTOMLEFT:SetSize((edgeSize or 1/1e4), (edgeSize or 1/1e4))
	bd.TEXTURES.BOTTOMLEFT:SetTexCoord(6/8, 7/8, 0, 1)

	bd.TEXTURES.BOTTOMRIGHT:SetTexture(bd.edgeFile)
	bd.TEXTURES.BOTTOMRIGHT:ClearAllPoints()
	bd.TEXTURES.BOTTOMRIGHT:SetPoint("BOTTOMRIGHT", bd)
	bd.TEXTURES.BOTTOMRIGHT:SetSize((edgeSize or 1/1e4), (edgeSize or 1/1e4))
	bd.TEXTURES.BOTTOMRIGHT:SetTexCoord(7/8, 1, 0, 1)

	bd.TEXTURES.LEFT:SetTexture(bd.edgeFile, true)
	bd.TEXTURES.LEFT:ClearAllPoints()
	bd.TEXTURES.LEFT:SetPoint("LEFT", bd)
	bd.TEXTURES.LEFT:SetPoint("BOTTOM", bd, "BOTTOM", 0, (edgeSize or 0))
	bd.TEXTURES.LEFT:SetPoint("TOP", bd, "TOP", 0, -(edgeSize or 0))
	bd.TEXTURES.LEFT:SetWidth(edgeSize or 1/1e4)

	bd.TEXTURES.RIGHT:SetTexture(bd.edgeFile, true)
	bd.TEXTURES.RIGHT:ClearAllPoints()
	bd.TEXTURES.RIGHT:SetPoint("RIGHT", bd)
	bd.TEXTURES.RIGHT:SetPoint("BOTTOM", bd, "BOTTOM", 0, (edgeSize or 0))
	bd.TEXTURES.RIGHT:SetPoint("TOP", bd, "TOP", 0, -(edgeSize or 0))
	bd.TEXTURES.RIGHT:SetWidth(edgeSize or 1/1e4)

	bd.TEXTURES.TOP:SetTexture(bd.edgeFile, true)
	bd.TEXTURES.TOP:ClearAllPoints()
	bd.TEXTURES.TOP:SetPoint("TOP", bd)
	bd.TEXTURES.TOP:SetPoint("LEFT", bd, "LEFT", (edgeSize or 0), 0)
	bd.TEXTURES.TOP:SetPoint("RIGHT", bd, "RIGHT", -(edgeSize or 0), 0)
	bd.TEXTURES.TOP:SetHeight(edgeSize or 1/1e4)

	bd.TEXTURES.BOTTOM:SetTexture(bd.edgeFile, true)
	bd.TEXTURES.BOTTOM:ClearAllPoints()
	bd.TEXTURES.BOTTOM:SetPoint("BOTTOM", bd)
	bd.TEXTURES.BOTTOM:SetPoint("LEFT", bd, "LEFT", (edgeSize or 0), 0)
	bd.TEXTURES.BOTTOM:SetPoint("RIGHT", bd, "RIGHT", -(edgeSize or 0), 0)
	bd.TEXTURES.BOTTOM:SetHeight(edgeSize or 1/1e4)

	w, h = scale(w), scale(h)
	
	bd.TEXTURES.BG:SetTexCoord(0, w/(tileSize or w), 0, h/(tileSize or h))
	bd.TEXTURES.LEFT:SetTexCoord(0, 1/8, 0, h/(edgeSize or h))
	bd.TEXTURES.RIGHT:SetTexCoord(1/8, 2/8, 0, h/(edgeSize or h))
	bd.TEXTURES.TOP:SetTexCoord(2/8, 0, 3/8, 0, 2/8, w/(edgeSize or w), 3/8, w/(edgeSize or w))
	bd.TEXTURES.BOTTOM:SetTexCoord(3/8, 0, 4/8, 0, 3/8, w/(edgeSize or w), 4/8, w/(edgeSize or w))
end

local SetUIBackdrop = function(object, backdrop, trueScale)
	if (backdrop) and (type(backdrop) ~= "table") then
		geterrorhandler()((MAJOR .. ": frame:SetUIBackdrop(backdrop) - 'backdrop' table or nil expected, got %s"):format(type(backdrop)), 2)
		return
	end
	
	if (object:GetBackdrop()) then
		object:SetBackdrop(nil)
	end
	
	if (not object._BACKDROP) then
		local bd = CreateFrame("Frame", nil, object)
		bd:SetAllPoints(object)
		
		bd.TEXTURES = {}
		bd.TEXTURES.BG = object:CreateTexture()
		bd.TEXTURES.LEFT = object:CreateTexture()
		bd.TEXTURES.RIGHT = object:CreateTexture()
		bd.TEXTURES.TOP = object:CreateTexture()
		bd.TEXTURES.BOTTOM = object:CreateTexture()
		bd.TEXTURES.TOPLEFT = object:CreateTexture()
		bd.TEXTURES.TOPRIGHT = object:CreateTexture()
		bd.TEXTURES.BOTTOMLEFT = object:CreateTexture()
		bd.TEXTURES.BOTTOMRIGHT = object:CreateTexture()

		bd.TEXTURES.BG:SetDrawLayer("BACKGROUND", -5) -- setting to -5 in case of a shadow
		bd.TEXTURES.LEFT:SetDrawLayer("BORDER", -7)
		bd.TEXTURES.RIGHT:SetDrawLayer("BORDER", -7)
		bd.TEXTURES.TOP:SetDrawLayer("BORDER", -7)
		bd.TEXTURES.BOTTOM:SetDrawLayer("BORDER", -7)
		bd.TEXTURES.TOPLEFT:SetDrawLayer("BORDER", -6)
		bd.TEXTURES.TOPRIGHT:SetDrawLayer("BORDER", -6)
		bd.TEXTURES.BOTTOMLEFT:SetDrawLayer("BORDER", -6)
		bd.TEXTURES.BOTTOMRIGHT:SetDrawLayer("BORDER", -6)

		object._BACKDROP = bd
	end
	
	local bd = object._BACKDROP
	bd.bgFile = backdrop and backdrop.bgFile
	bd.edgeFile = backdrop and backdrop.edgeFile
	bd.edgeSize = backdrop and backdrop.edgeSize
	bd.tile = backdrop and backdrop.tile
	bd.tileSize = backdrop and backdrop.tileSize
	bd.insets = {
		left = (backdrop and backdrop.insets) and backdrop.insets.left;
		right = (backdrop and backdrop.insets) and backdrop.insets.right;
		top = (backdrop and backdrop.insets) and backdrop.insets.top;
		bottom = (backdrop and backdrop.insets) and backdrop.insets.bottom;
	}
	bd.trueScale = trueScale

	bd:SetScript("OnSizeChanged", ResizeUIBackdrop)
	ResizeUIBackdrop(bd)
end

local SetUIBackdropColor = function(object, r, g, b, a)
	if (not object._BACKDROP) then
		geterrorhandler()(MAJOR .. ": frame:SetUIBackdropColor(r, g, b, a) - 'frame' hasn't got a backdrop", 2)
		return
	end
	
	if (type(r) ~= "number") then
		geterrorhandler()((MAJOR .. ": frame:SetUIBackdropColor(r, g, b, a) - 'r' number expected, got %s"):format(type(r)), 2)
		return
	end

	if (type(g) ~= "number") then
		geterrorhandler()((MAJOR .. ": frame:SetUIBackdropColor(r, g, b, a) - 'g' number expected, got %s"):format(type(g)), 2)
		return
	end

	if (type(b) ~= "number") then
		geterrorhandler()((MAJOR .. ": frame:SetUIBackdropColor(r, g, b, a) - 'b' number expected, got %s"):format(type(b)), 2)
		return
	end

	if a and (type(a) ~= "number") then
		geterrorhandler()((MAJOR .. ": frame:SetUIBackdropColor(r, g, b, a) - 'a' number expected, got %s"):format(type(a)), 2)
		return
	end
	
	object._BACKDROP.TEXTURES.BG:SetVertexColor(r, g, b, a)
end

local SetUIBackdropBorderColor = function(object, r, g, b, a)
	if not object._BACKDROP then
		geterrorhandler()(MAJOR .. ": frame:SetUIBackdropBorderColor(r, g, b, a) - 'frame' hasn't got a backdrop", 2)
		return
	end
	
	if (type(r) ~= "number") then
		geterrorhandler()((MAJOR .. ": frame:SetUIBackdropColor(r, g, b, a) - 'r' number expected, got %s"):format(type(r)), 2)
		return
	end

	if (type(g) ~= "number") then
		geterrorhandler()((MAJOR .. ": frame:SetUIBackdropColor(r, g, b, a) - 'g' number expected, got %s"):format(type(g)), 2)
		return
	end

	if (type(b) ~= "number") then
		geterrorhandler()((MAJOR .. ": frame:SetUIBackdropColor(r, g, b, a) - 'b' number expected, got %s"):format(type(b)), 2)
		return
	end

	if a and (type(a) ~= "number") then
		geterrorhandler()((MAJOR .. ": frame:SetUIBackdropColor(r, g, b, a) - 'a' number expected, got %s"):format(type(a)), 2)
		return
	end

	object._BACKDROP.TEXTURES.LEFT:SetVertexColor(r, g, b, a)
	object._BACKDROP.TEXTURES.RIGHT:SetVertexColor(r, g, b, a)
	object._BACKDROP.TEXTURES.TOP:SetVertexColor(r, g, b, a)
	object._BACKDROP.TEXTURES.BOTTOM:SetVertexColor(r, g, b, a)
	object._BACKDROP.TEXTURES.TOPLEFT:SetVertexColor(r, g, b, a)
	object._BACKDROP.TEXTURES.TOPRIGHT:SetVertexColor(r, g, b, a)
	object._BACKDROP.TEXTURES.BOTTOMLEFT:SetVertexColor(r, g, b, a)
	object._BACKDROP.TEXTURES.BOTTOMRIGHT:SetVertexColor(r, g, b, a)
end

local GetUIBackdrop = function(object)
	return (object._BACKDROP) and {
		bgFile = object._BACKDROP.bgFile;
		edgeFile = object._BACKDROP.edgeFile;
		edgeSize = object._BACKDROP.edgeSize;
		tile = object._BACKDROP.tile;
		tileSize = object._BACKDROP.tileSize;
		insets = object._BACKDROP.insets and {
			left = object._BACKDROP.insets.left;
			right = object._BACKDROP.insets.right;
			top = object._BACKDROP.insets.top;
			bottom = object._BACKDROP.insets.bottom;
		} or nil;
	} or nil
end

local GetUIBackdropFrame = function(object)
	return object._BACKDROP
end

local GetUIBackdropColor = function(object)
	if (not object._BACKDROP) then
		geterrorhandler()(MAJOR .. ": frame:GetUIBackdropColor() - 'frame' hasn't got a backdrop", 2)
		return
	end
	
	return object._BACKDROP.TEXTURES.BG:GetVertexColor()
end

local GetUIBackdropBorderColor = function(object)
	if (not object._BACKDROP) then
		geterrorhandler()(MAJOR .. ": frame:GetUIBackdropBorderColor() - 'frame' hasn't got a backdrop", 2)
		return
	end

	-- 
	-- any of our border textures will do, as they all have the same color
	return object._BACKDROP.TEXTURES.LEFT:GetVertexColor()
end

local ResizeUIShadow = function(s, w, h)
	local edgeSize = s.edgeSize
	
	local w = max(1, w or ( ((s:GetRight() or 0) - (s:GetLeft()or 0)) or 0) - (s.edgeSize or 0)*2)
	local h = max(1, h or ( ((s:GetTop() or 0) - (s:GetBottom()or 0)) or 0) - (s.edgeSize or 0)*2)

	s.TEXTURES.TOPLEFT:SetTexture(s.edgeFile)
	s.TEXTURES.TOPLEFT:ClearAllPoints()
	s.TEXTURES.TOPLEFT:SetPoint("TOPLEFT", s)
	s.TEXTURES.TOPLEFT:SetSize((s.edgeSize or 1/1e4), (s.edgeSize or 1/1e4))
	s.TEXTURES.TOPLEFT:SetTexCoord(4/8, 5/8, 0, 1)

	s.TEXTURES.TOPRIGHT:SetTexture(s.edgeFile)
	s.TEXTURES.TOPRIGHT:ClearAllPoints()
	s.TEXTURES.TOPRIGHT:SetPoint("TOPRIGHT", s)
	s.TEXTURES.TOPRIGHT:SetSize((s.edgeSize or 1/1e4), (s.edgeSize or 1/1e4))
	s.TEXTURES.TOPRIGHT:SetTexCoord(5/8, 6/8, 0, 1)

	s.TEXTURES.BOTTOMLEFT:SetTexture(s.edgeFile)
	s.TEXTURES.BOTTOMLEFT:ClearAllPoints()
	s.TEXTURES.BOTTOMLEFT:SetPoint("BOTTOMLEFT", s)
	s.TEXTURES.BOTTOMLEFT:SetSize((s.edgeSize or 1/1e4), (s.edgeSize or 1/1e4))
	s.TEXTURES.BOTTOMLEFT:SetTexCoord(6/8, 7/8, 0, 1)

	s.TEXTURES.BOTTOMRIGHT:SetTexture(s.edgeFile)
	s.TEXTURES.BOTTOMRIGHT:ClearAllPoints()
	s.TEXTURES.BOTTOMRIGHT:SetPoint("BOTTOMRIGHT", s)
	s.TEXTURES.BOTTOMRIGHT:SetSize((s.edgeSize or 1/1e4), (s.edgeSize or 1/1e4))
	s.TEXTURES.BOTTOMRIGHT:SetTexCoord(7/8, 1, 0, 1)

	s.TEXTURES.LEFT:SetTexture(s.edgeFile, true)
	s.TEXTURES.LEFT:ClearAllPoints()
	s.TEXTURES.LEFT:SetPoint("LEFT", s)
	s.TEXTURES.LEFT:SetPoint("BOTTOM", s, "BOTTOM", 0, (s.edgeSize or 0))
	s.TEXTURES.LEFT:SetPoint("TOP", s, "TOP", 0, -(s.edgeSize or 0))
	s.TEXTURES.LEFT:SetWidth(s.edgeSize or 1/1e4)

	s.TEXTURES.RIGHT:SetTexture(s.edgeFile, true)
	s.TEXTURES.RIGHT:ClearAllPoints()
	s.TEXTURES.RIGHT:SetPoint("RIGHT", s)
	s.TEXTURES.RIGHT:SetPoint("BOTTOM", s, "BOTTOM", 0, (s.edgeSize or 0))
	s.TEXTURES.RIGHT:SetPoint("TOP", s, "TOP", 0, -(s.edgeSize or 0))
	s.TEXTURES.RIGHT:SetWidth(s.edgeSize or 1/1e4)

	s.TEXTURES.TOP:SetTexture(s.edgeFile, true)
	s.TEXTURES.TOP:ClearAllPoints()
	s.TEXTURES.TOP:SetPoint("TOP", s)
	s.TEXTURES.TOP:SetPoint("LEFT", s, "LEFT", (s.edgeSize or 0), 0)
	s.TEXTURES.TOP:SetPoint("RIGHT", s, "RIGHT", -(s.edgeSize or 0), 0)
	s.TEXTURES.TOP:SetHeight(s.edgeSize or 1/1e4)

	s.TEXTURES.BOTTOM:SetTexture(s.edgeFile, true)
	s.TEXTURES.BOTTOM:ClearAllPoints()
	s.TEXTURES.BOTTOM:SetPoint("BOTTOM", s)
	s.TEXTURES.BOTTOM:SetPoint("LEFT", s, "LEFT", (s.edgeSize or 0), 0)
	s.TEXTURES.BOTTOM:SetPoint("RIGHT", s, "RIGHT", -(s.edgeSize or 0), 0)
	s.TEXTURES.BOTTOM:SetHeight(s.edgeSize or 1/1e4)

	w, h = scale(w), scale(h)
	
	s.TEXTURES.LEFT:SetTexCoord(0, 1/8, 0, h/(edgeSize or h))
	s.TEXTURES.RIGHT:SetTexCoord(1/8, 2/8, 0, h/(edgeSize or h))
	s.TEXTURES.TOP:SetTexCoord(2/8, 0, 3/8, 0, 2/8, w/(edgeSize or w), 3/8, w/(edgeSize or w))
	s.TEXTURES.BOTTOM:SetTexCoord(3/8, 0, 4/8, 0, 3/8, w/(edgeSize or w), 4/8, w/(edgeSize or w))
end

local CreateUIShadow = function(object)
	if (object._SHADOW) or (not(M and M["Backdrop"]["Glow"])) then 
		return 
	end

	if (not object._SHADOW) then
		local s = CreateFrame("Frame", nil, object)
		s.edgeFile = M["Border"]["Glow"]
		s.edgeSize = 3

		s:SetPoint("LEFT", object, -(s.edgeSize - 0.5), 0)
		s:SetPoint("RIGHT", object, (s.edgeSize - 0.5), 0)
		s:SetPoint("TOP", object, 0, (s.edgeSize - 0.5))
		s:SetPoint("BOTTOM", object, 0, -(s.edgeSize - 0.5))
		
		s.TEXTURES = {}
		s.TEXTURES.LEFT = object:CreateTexture()
		s.TEXTURES.RIGHT = object:CreateTexture()
		s.TEXTURES.TOP = object:CreateTexture()
		s.TEXTURES.BOTTOM = object:CreateTexture()
		s.TEXTURES.TOPLEFT = object:CreateTexture()
		s.TEXTURES.TOPRIGHT = object:CreateTexture()
		s.TEXTURES.BOTTOMLEFT = object:CreateTexture()
		s.TEXTURES.BOTTOMRIGHT = object:CreateTexture()

		s.TEXTURES.LEFT:SetDrawLayer("BACKGROUND", -7)
		s.TEXTURES.RIGHT:SetDrawLayer("BACKGROUND", -7)
		s.TEXTURES.TOP:SetDrawLayer("BACKGROUND", -7)
		s.TEXTURES.BOTTOM:SetDrawLayer("BACKGROUND", -7)
		s.TEXTURES.TOPLEFT:SetDrawLayer("BACKGROUND", -6)
		s.TEXTURES.TOPRIGHT:SetDrawLayer("BACKGROUND", -6)
		s.TEXTURES.BOTTOMLEFT:SetDrawLayer("BACKGROUND", -6)
		s.TEXTURES.BOTTOMRIGHT:SetDrawLayer("BACKGROUND", -6)

		local r,g,b,a = unpack(C["shadow"])
		s.TEXTURES.LEFT:SetVertexColor(r, g, b, a)
		s.TEXTURES.RIGHT:SetVertexColor(r, g, b, a)
		s.TEXTURES.TOP:SetVertexColor(r, g, b, a)
		s.TEXTURES.BOTTOM:SetVertexColor(r, g, b, a)
		s.TEXTURES.TOPLEFT:SetVertexColor(r, g, b, a)
		s.TEXTURES.TOPRIGHT:SetVertexColor(r, g, b, a)
		s.TEXTURES.BOTTOMLEFT:SetVertexColor(r, g, b, a)
		s.TEXTURES.BOTTOMRIGHT:SetVertexColor(r, g, b, a)

		object._SHADOW = s

		s:SetScript("OnSizeChanged", ResizeUIShadow)

		ResizeUIShadow(s)
	end
	
	return object._SHADOW
end

local SetUIShadowColor = function(object, r, g, b, a)
	if (not object._SHADOW) then 
		return 
	end
	
	if (type(r) ~= "number") then
		geterrorhandler()((MAJOR .. ": frame:SetUIShadowColor(r, g, b, a) - 'r' number expected, got %s"):format(type(r)), 2)
		return
	end

	if (type(g) ~= "number") then
		geterrorhandler()((MAJOR .. ": frame:SetUIShadowColor(r, g, b, a) - 'g' number expected, got %s"):format(type(g)), 2)
		return
	end

	if (type(b) ~= "number") then
		geterrorhandler()((MAJOR .. ": frame:SetUIShadowColor(r, g, b, a) - 'b' number expected, got %s"):format(type(b)), 2)
		return
	end

	if a and (type(a) ~= "number") then
		geterrorhandler()((MAJOR .. ": frame:SetUIShadowColor(r, g, b, a) - 'a' number expected, got %s"):format(type(a)), 2)
		return
	end
	
	object._SHADOW.TEXTURES.LEFT:SetVertexColor(r, g, b, a)
	object._SHADOW.TEXTURES.RIGHT:SetVertexColor(r, g, b, a)
	object._SHADOW.TEXTURES.TOP:SetVertexColor(r, g, b, a)
	object._SHADOW.TEXTURES.BOTTOM:SetVertexColor(r, g, b, a)
	object._SHADOW.TEXTURES.TOPLEFT:SetVertexColor(r, g, b, a)
	object._SHADOW.TEXTURES.TOPRIGHT:SetVertexColor(r, g, b, a)
	object._SHADOW.TEXTURES.BOTTOMLEFT:SetVertexColor(r, g, b, a)
	object._SHADOW.TEXTURES.BOTTOMRIGHT:SetVertexColor(r, g, b, a)
end
------------------------------------------------------------------------------------------------------------
-- 	Panel template
------------------------------------------------------------------------------------------------------------
--
-- A plugin can only be shown once in a panel
-- Spawning another with the same name will only return
-- a pointer to the first one
--
-- SpawnPlugin(name[, justify])
-- @param name <string> name of the plugin from the library
-- @param justify <string> 
-- @param <table> pointer to the plugin
PANEL.SpawnPlugin = function(self, name, justify)
	self.Plugins = self.Plugins or {}
	
	local f
	
	if (not self.Plugins[name]) then
		f = setmetatable(CreateFrame("Frame", nil, self), { __index = gPanel.PLUGIN })

		f.overlay = CreateFrame("Frame", nil, f)
		f.overlay:SetAllPoints(f)

		f.lib = gPanel.Lib

		Embed(f, gPanel.Plugins[name])
	else
		f = self.Plugins[name]
		f:SetJustify(justify)
	end

	if (not f.active) then
		local object = f.overlay and f.overlay or f
	
		f.justify = ALLOWEDJUSTIFICATION[justify] and justify or "CENTER"

		if not f.text then
			f.text = f:CreateFontString(nil, "OVERLAY")
		end

		f.text:SetFontObject(gPanel.db.fontobject)
		f.text:ClearAllPoints()
		f.text:SetPoint((f.justify == "FULL") and "CENTER" or f.justify, self)
		
		if (f.justify == "RIGHT") or (f.justify == "LEFT") then
			f.text:SetJustifyH(f.justify)
		elseif (f.justify == "BOTTOM") or (f.justify == "TOP") then
			f.text:SetJustifyV(f.justify)
		elseif (f.justify == "FULL") then
			f.text:SetJustifyH("CENTER")
			f.text:SetJustifyV("MIDDLE")
		else
			f.text:SetJustifyH(f.justify)
			f.text:SetJustifyV(f.justify == "CENTER" and "MIDDLE" or f.justify)
		end

		f:SetAllPoints(f.text)
		
		object:EnableMouse(true)

		if (f.variables) then
			for i,v in pairs(f.variables) do
				f[i] = v
			end
		end

		f:UnregisterAllEvents()
		if (f.events) then
			for i,v in pairs(f.events) do
				f:RegisterEvent(v)
			end
		end
		
		f:SetScript("OnEvent", nil)
		if (f.onevent) then
			f.ForceEvent = function(self, event, ...)
				f.onevent(self, event, ...)
			end
			
			f:SetScript("OnEvent", function(self, event, ...) 
				if (not self:IsVisible()) then 
					return 
				end
				
				f.onevent(self, event, ...)
			end)
		end
		
		f:SetScript("OnMouseDown", nil)
		if (f.onclick) then
			object:SetScript("OnMouseDown", function(self, button) 
				if (not self:IsVisible()) then 
					return 
				end
				
				f.onclick(self, button)
			end)
		end
		
		f:SetScript("OnUpdate", nil)
		if (f.onupdate) then
			f.interval = max(0.2, tonumber(f.interval or 1))
			f:SetScript("OnUpdate", function(self, elapsed) 
				if (not self:IsVisible()) then 
					return 
				end
				
				self.elapsed = (self.elapsed or 0) + elapsed
				
				if (self.elapsed >= self.interval) then
					f.onupdate(self)
					self.elapsed = self.elapsed - self.interval
				end
			end)
		end
		
		if (f.tooltip) then
			f:HookTooltip(f.tooltip, f.horizontalTooltip)
		end
		
		if (f.oninit) then
			f.oninit (f)
		end
		
		self.Plugins[name] = f
	end

	self.Plugins[name].active = true
	self.Plugins[name]:Refresh()
	
	return self.Plugins[name]
end

--
-- 	Killing a plugin doesn't really remove it, it only hides it and halts its updating
-- 	The frame will remain in memory until reload or logout
--
-- @param name <string> 
PANEL.KillPlugin = function(self, name)
	if (not self.Plugins) or (not self.Plugins[name]) then 
		return 
	end

	self.Plugins[name].active = false
	self.Plugins[name]:Refresh()
end

--
-- set the font object of your panel
PANEL.SetFontObject = function(self, fontObject)
	if (not self.text) then 
		return 
	end
	
	self.text:SetFontObject(fontObject)
end

--
-- 	Refresh the panel and it's contents
--	Call this to update positions, fonts, etc
--
PANEL.Refresh = function(self)
	for i,v in pairs(self.Plugins) do
		v:Refresh()
	end
end

------------------------------------------------------------------------------------------------------------
-- 	Plugin template
------------------------------------------------------------------------------------------------------------
--
-- 	Sets which part of the panel the plugin should be aligned to
-- 		justify 	= "TOP", "BOTTOM", "RIGHT", "LEFT" or "CENTER" 
--
PLUGIN.SetJustify = function(self, justify)
	self.justify = ALLOWEDJUSTIFICATION[justify] and justify or "CENTER"
end

--
-- 	Refresh the plugin
--	Call this to update positions, fonts, etc
--
PLUGIN.Refresh = function(self)
	if (self.active) and (not self:IsVisible()) then
		self:Show()
	elseif (not self.active) and (self:IsVisible()) then
		self:Hide()
	end
	
	if (self.active) then
		if (self.justify == "FULL") then
			self.text:SetAllPoints(self:GetParent())
		else
			local padding = 12

			local x = (self.justify == "RIGHT") and -padding or (self.justify == "LEFT") and padding or 0
			local y = (self.justify == "TOP") and -padding or (self.justify == "BOTTOM") and padding or 0
			
			self.text:ClearAllPoints()
			self.text:SetPoint(self.justify, self:GetParent(), self.justify, x, y)
		end
	end
end

--
-- 	Hook/Unhook a tooltip to the plugin
-- 	Will replace any other existing tooltip
--
local PlaceGameTooltip = function(anchor, horTip)
	GameTooltip:SetOwner(anchor, "ANCHOR_NONE")
	GameTooltip:ClearAllPoints()
	
	if (horTip) then
		if (GetScreenWidth() - anchor:GetRight()) > anchor:GetLeft() then
			GameTooltip:SetPoint("TOPLEFT", anchor, "TOPRIGHT", 8, 0)
		else
			GameTooltip:SetPoint("TOPRIGHT", anchor, "TOPLEFT", -8, 0)
		end
	else
		if (GetScreenHeight() - anchor:GetTop()) > anchor:GetBottom() then
			GameTooltip:SetPoint("BOTTOM", anchor, "TOP", 0, 8)
		else
			GameTooltip:SetPoint("TOP", anchor, "BOTTOM", 0, -8)
		end
	end
end

PLUGIN.HookTooltip = function(self, name, horTip)
	local object = self.overlay and self.overlay or self
	local parent = self;
	
	object:SetScript("OnEnter", function(self) 
		if (S and S.tooltip and S.tooltip.showincombat) or not InCombatLockdown() then
			PlaceGameTooltip(self, horTip)
			gPanel.Tooltips[name](parent)
		end
	end)
	
	object:SetScript("OnLeave", function(self) GameTooltip:Hide(); end)
end

PLUGIN.UnhookTooltip = function(self)
	self:SetScript("OnEnter", nil)
	self:SetScript("OnLeave", nil)
end



------------------------------------------------------------------------------------------------------------
-- 	Panels
------------------------------------------------------------------------------------------------------------
--
-- 	Register a new plugin with library, which will be available for all panel objects to use
-- 	Will overwrite any exisiting plugin with the same name
--
gPanel.RegisterPlugin = function(self, name, plugin)
	if (type(name) ~= "string") then 
		geterrorhandler()((MAJOR .. ": RegisterPlugin(name, plugin) 'name' - string expected, got %s"):format(type(name)), 2)
		return 
	end

	if (type(plugin) ~= "table") then 
		geterrorhandler()((MAJOR .. ": RegisterPlugin(name, plugin) 'plugin' - table expected, got %s"):format(type(plugin)), 2)
		return 
	end

	gPanel.Plugins[name] = plugin
end

--
-- 	Register a new tooltip
--
gPanel.RegisterTooltip = function(self, name, func)
	if (type(name) ~= "string") then 
		geterrorhandler()((MAJOR .. ": RegisterTooltip(name[, func]) 'name' - string expected, got %s"):format(type(name)), 2)
		return 
	end

	gPanel.Tooltips[name] = type(func) == "function" and func or noop
end

--
-- Sets the font object of all panels
--
gPanel.SetFontObject = function(self, fontObject)
	gPanel.db.fontobject = fontObject
	for name,p in pairs(gPanel.Panels) do
		p.text:SetFontObject(fontObject)
	end
end


--
-- 	Get a pointer to an existing panel based on it's name
--
gPanel.GetPanel = function(self, name)
	return gPanel.Panels[name]
end

--
-- Set one of the stored default colors for the panel
--
gPanel.SetColor = function(self, key, r, g, b, a)
	if (type(key) ~= "string") then
		geterrorhandler()((MAJOR .. ": SetColor(key, r[, g, b[, a]]) 'key' - string expected, got %s"):format(type(key)), 2)
		return 
	end
	
	if (not C[key]) then
		geterrorhandler()((MAJOR .. ": SetColor(key, r[, g, b[, a]]) 'key' - cannot find any colors named '%s'"):format(type(key)), 2)
		return 
	end
	
	if (type(r) ~= "number") and (type(r) ~= "table") then
		geterrorhandler()((MAJOR .. ": SetColor(key, r[, g, b[, a]]) 'r' - number or table expected, got %s"):format(type(r)), 2)
		return 
	end

	if (type(r) == "number") then
		if (type(g) ~= "number") then
			geterrorhandler()((MAJOR .. ": SetColor(key, r[, g, b[, a]]) 'g' - number expected, got %s"):format(type(g)), 2)
			return 
		end

		if (type(b) ~= "number") then
			geterrorhandler()((MAJOR .. ": SetColor(key, r[, g, b[, a]]) 'b' - number expected, got %s"):format(type(b)), 2)
			return 
		end
		
		if (type(a) ~= "number") then
			geterrorhandler()((MAJOR .. ": SetColor(key, r[, g, b[, a]]) 'a' - number expected, got %s"):format(type(a)), 2)
			return 
		end
		
		C[key][1] = r
		C[key][2] = g
		C[key][3] = b
		
		if a then
			C[key][4] = a
		end
	elseif (type(r) == "table") then
		C[key] = r
	end
end

gPanel.GetColor = function(self, key)
	if (type(key) ~= "string") then
		geterrorhandler()((MAJOR .. ": SetColor(key, r[, g, b[, a]]) 'key' - string expected, got %s"):format(type(key)), 2)
		return 
	end
	
	if (not C[key]) then
		geterrorhandler()((MAJOR .. ": SetColor(key, r[, g, b[, a]]) 'key' - cannot find any colors named '%s'"):format(type(key)), 2)
		return 
	end

	return C[key]
end

--
-- 	Create a new panel object, or refresh it if object exists
--	 *anchor is both the parent and anchor, if given
--
local panelcount = 0
local CreateUIPanel = function(anchor, name)
	if (gPanel.Panels[name]) then
		gPanel.Panels[name]:Refresh()
	else

		panelcount = max(#gPanel.Panels + 1, panelcount + 1)

		if (not name) then
			name = "gPanel20_UserPanel_" .. panelcount
		end
		
		local parent
		if (type(anchor) == "string") then
			if (_G[anchor]) and (_G[anchor].IsObjectType) and (_G[anchor]:IsObjectType("Frame")) then
				parent = _G[anchor]
			end
		elseif (anchor) and (anchor.IsObjectType) and (anchor:IsObjectType("Frame")) then
			parent = anchor
		else
			parent = UIParent
		end

		local panel = setmetatable(CreateFrame("Frame", name, parent), { __index = PANEL })
		panel:SetAllPoints(parent)
		panel:SetFrameStrata(parent:GetFrameStrata())
		panel:SetFrameLevel(parent:GetFrameLevel())
		
		if (parent == UIParent) then
			panel:SetFrameLevel(20)
			panel:SetFrameStrata("LOW")
		end
		
		panel.Plugins = panel.Plugins or {}
		
		if (parent ~= UIParent) then
			parent.panel = panel
		end

		gPanel.Panels[name] = panel
	end
	
	return gPanel.Panels[name]
end

local SetUITemplate = function(object, name, ...)
	if (not gPanel.Templates[name]) then 
		return 
	end

	return gPanel.Templates[name](object, ...)
end

gPanel.CreateUIPanel = function(self, ...)
	return CreateUIPanel(...)
end

gPanel.RegisterUITemplate = function(self, name, data)
	gPanel.Templates[name] = data 
end

------------------------------------------------------------------------------------------------------------
-- 	API calls
------------------------------------------------------------------------------------------------------------
do
	local setAPI = function(frame)
		local meta = getmetatable(frame).__index

		if not meta.CreateUIPanel then meta.CreateUIPanel = CreateUIPanel end
		if not meta.CreateUIShadow then meta.CreateUIShadow = CreateUIShadow end
		if not meta.SetUIShadowColor then meta.SetUIShadowColor = SetUIShadowColor end
		if not meta.SetUIBackdrop then meta.SetUIBackdrop = SetUIBackdrop end
		if not meta.SetUIBackdropColor then meta.SetUIBackdropColor = SetUIBackdropColor end
		if not meta.SetUIBackdropBorderColor then meta.SetUIBackdropBorderColor = SetUIBackdropBorderColor end
		if not meta.GetUIBackdrop then meta.GetUIBackdrop = GetUIBackdrop end
		if not meta.GetUIBackdropColor then meta.GetUIBackdropColor = GetUIBackdropColor end
		if not meta.GetUIBackdropBorderColor then meta.GetUIBackdropBorderColor = GetUIBackdropBorderColor end
		if not meta.GetUIBackdropFrame then meta.GetUIBackdropFrame = GetUIBackdropFrame end
		if not meta.GetUIShadowColor then meta.GetUIShadowColor = GetUIShadowColor end
		if not meta.SetUITemplate then meta.SetUITemplate = SetUITemplate end
	end

	local done = {}

	local frames = EnumerateFrames()
	while frames do
		if (not done[frames:GetObjectType()]) then
			setAPI(frames)
			done[frames:GetObjectType()] = true
		end

		frames = EnumerateFrames(frames)
	end
	
	done = nil
end

