﻿local addon, ns = ...
local cargBags = cargBags or ns.cargBags

local module = LibStub("gCore-3.0"):NewModule("gBags: Bags")

-- Lua API
local ceil = math.ceil
local ipairs, pairs, select, unpack = ipairs, pairs, select, unpack
local format, strfind = string.format, string.find
local tonumber = tonumber

local M = LibStub("gMedia-3.0")
local L = LibStub("gLocale-1.0"):GetLocale(addon)
local C = LibStub("gDB-1.0"):GetDataBase("gBags: Colors")
local F = LibStub("gDB-1.0"):GetDataBase("gBags: Functions")
local CreateChatCommand = function(...) module:CreateChatCommand(...) end
local GetPoint = function(...) return LibStub("gFrameHandler-1.0"):GetPoint(...) end
local ScheduleTimer = function(...) module:ScheduleTimer(...) end 
local CancelTimer = function(...) module:CancelTimer(...) end 
local StyleActionButton = function(...) LibStub("gActionButtons-2.0").StyleActionButton(...) end

local Types, Bags, Container, Button, BagButton
local buttonSize = 29
local ContainerList = {}

local defaults = {
	autorestack 		= 0; -- automatically restack bags/bank when opened
	autorestackcrafted 	= 1; -- automatically restack crafted items like Gems and Glyphs
	compressemptyspace 	= 1; -- compress the empty space to a maximum one row
	locked 				= 1; -- whether or not the bags are locked into place
	scale 				= 1; -- scale of the bags
	
	-- what categories should be hidden
	hide = {
		["Bank_Sets"] 			= false;
		["Bank_Armor"] 			= false;
		["Bank_Weapons"] 		= false;
		["Bank_Gizmos"] 		= false;
		["Bank_Quest"] 			= false;
		["Bank_Glyphs"] 		= false;
		["Bank_Gems"] 			= false;
		["Bank_Consumables"] 	= false;
		["Bank_Trade"] 			= false;
		["Bank_Misc"] 			= false;
		["Main_Sets"] 			= false;
		["Main_Armor"] 			= false;
		["Main_Weapons"] 		= false;
		["Main_Gizmos"] 		= false;
		["Main_Quest"] 			= false;
		["Main_Glyphs"] 		= false;
		["Main_Gems"] 			= false;
		["Main_Consumables"] 	= false;
		["Main_Trade"] 			= false;
		["Main_Misc"] 			= false;
		["Main_Junk"] 			= false;
	};

	-- the bags don't use the global position saving functions
	-- this is because some users prefer to have the bags unlocked at all times
	points = {
		Main = { "BOTTOMRIGHT", 	"UIParent", 	"BOTTOMRIGHT", 	-6, 170 };
		Bank = { "TOPLEFT", 		"UIParent", 	"TOPLEFT", 		 6,  -6 };
	}
}

-- we grab the localized names here, as we need them for the equiplist
Types = cargBags:GetLocalizedTypes()

local db = {
	-- this is terrible. I _must_ find a better way!
	["equiplist"] = {
		["EXCEPTIONS"] = {
			["HUNTER"] 		= { [Types["Mail"]] 	= 40; };
			["PALADIN"] 	= { [Types["Plate"]] 	= 40; };
			["SHAMAN"] 		= { [Types["Mail"]] 	= 40; };
			["WARRIOR"] 	= { [Types["Plate"]] 	= 40; };
		};
		
		["DEATHKNIGHT"] =  {
			[Types["Armor"]] = {
				[Types["Miscellaneous"]] 		= true;
				[Types["Cloth"]] 				= true;
				[Types["Leather"]] 				= true; 
				[Types["Mail"]] 				= true;
				[Types["Plate"]] 				= true;
				[Types["Shields"]] 				= false;
				[Types["Relic"]] 				= true;
			};
			[Types["Weapon"]] = {
				[Types["One-Handed Axes"]] 		= true;
				[Types["Two-Handed Axes"]] 		= true;
				[Types["Bows"]] 				= false;
				[Types["Guns"]] 				= false;
				[Types["One-Handed Maces"]] 	= true;
				[Types["Two-Handed Maces"]] 	= true;
				[Types["Polearms"]] 			= true;
				[Types["One-Handed Swords"]] 	= true;
				[Types["Two-Handed Swords"]] 	= true;
				[Types["Staves"]] 				= false;
				[Types["Fist Weapons"]] 		= false;
				[Types["Miscellaneous"]] 		= true;
				[Types["Daggers"]] 				= false;
				[Types["Thrown"]] 				= false;
				[Types["Crossbows"]] 			= false;
				[Types["Wands"]] 				= false;
				[Types["Fishing Poles"]] 		= true;
			};
		};
		["DRUID"] =  {
			[Types["Armor"]] = {
				[Types["Miscellaneous"]] 		= true;
				[Types["Cloth"]] 				= true;
				[Types["Leather"]] 				= true; 
				[Types["Mail"]] 				= false;
				[Types["Plate"]] 				= false;
				[Types["Shields"]] 				= false;
				[Types["Relic"]] 				= true;
			};
			[Types["Weapon"]] = {
				[Types["One-Handed Axes"]] 		= false;
				[Types["Two-Handed Axes"]] 		= false;
				[Types["Bows"]] 				= false;
				[Types["Guns"]] 				= false;
				[Types["One-Handed Maces"]] 	= true;
				[Types["Two-Handed Maces"]] 	= true;
				[Types["Polearms"]] 			= true;
				[Types["One-Handed Swords"]] 	= false;
				[Types["Two-Handed Swords"]] 	= false;
				[Types["Staves"]] 				= true;
				[Types["Fist Weapons"]] 		= true;
				[Types["Miscellaneous"]] 		= true;
				[Types["Daggers"]] 				= true;
				[Types["Thrown"]] 				= false;
				[Types["Crossbows"]] 			= false;
				[Types["Wands"]] 				= false;
				[Types["Fishing Poles"]] 		= true;
			};
		};
		["HUNTER"] =  {
			[Types["Armor"]] = {
				[Types["Miscellaneous"]] 		= true;
				[Types["Cloth"]] 				= true;
				[Types["Leather"]] 				= true; 
				[Types["Mail"]] 				= true;
				[Types["Plate"]] 				= false;
				[Types["Shields"]] 				= false;
				[Types["Relic"]] 				= false;
			};
			[Types["Weapon"]] = {
				[Types["One-Handed Axes"]] 		= true;
				[Types["Two-Handed Axes"]] 		= true;
				[Types["Bows"]] 				= true;
				[Types["Guns"]] 				= true;
				[Types["One-Handed Maces"]] 	= false;
				[Types["Two-Handed Maces"]] 	= false;
				[Types["Polearms"]] 			= true;
				[Types["One-Handed Swords"]] 	= false;
				[Types["Two-Handed Swords"]] 	= false;
				[Types["Staves"]] 				= true;
				[Types["Fist Weapons"]] 		= true;
				[Types["Miscellaneous"]] 		= true;
				[Types["Daggers"]] 				= true;
				[Types["Thrown"]] 				= false;
				[Types["Crossbows"]] 			= true;
				[Types["Wands"]] 				= false;
				[Types["Fishing Poles"]] 		= true;
			};
		};
		["MAGE"] =  {
			[Types["Armor"]] = {
				[Types["Miscellaneous"]] 		= true;
				[Types["Cloth"]] 				= true;
				[Types["Leather"]] 				= false; 
				[Types["Mail"]] 				= false;
				[Types["Plate"]] 				= false;
				[Types["Shields"]] 				= false;
				[Types["Relic"]] 				= false;
			};
			[Types["Weapon"]] = {
				[Types["One-Handed Axes"]] 		= false;
				[Types["Two-Handed Axes"]] 		= false;
				[Types["Bows"]] 				= false;
				[Types["Guns"]] 				= false;
				[Types["One-Handed Maces"]] 	= false;
				[Types["Two-Handed Maces"]] 	= false;
				[Types["Polearms"]] 			= false;
				[Types["One-Handed Swords"]] 	= true;
				[Types["Two-Handed Swords"]] 	= false;
				[Types["Staves"]] 				= true;
				[Types["Fist Weapons"]] 		= false;
				[Types["Miscellaneous"]] 		= true;
				[Types["Daggers"]] 				= true;
				[Types["Thrown"]] 				= false;
				[Types["Crossbows"]] 			= false;
				[Types["Wands"]] 				= true;
				[Types["Fishing Poles"]] 		= true;
			};
		};
		["PALADIN"] =  {
			[Types["Armor"]] = {
				[Types["Miscellaneous"]] 		= true;
				[Types["Cloth"]] 				= true;
				[Types["Leather"]] 				= true; 
				[Types["Mail"]] 				= true;
				[Types["Plate"]] 				= true;
				[Types["Shields"]] 				= true;
				[Types["Relic"]] 				= true;
			};
			[Types["Weapon"]] = {
				[Types["One-Handed Axes"]] 		= true;
				[Types["Two-Handed Axes"]] 		= true;
				[Types["Bows"]] 				= false;
				[Types["Guns"]] 				= false;
				[Types["One-Handed Maces"]] 	= true;
				[Types["Two-Handed Maces"]] 	= true;
				[Types["Polearms"]] 			= true;
				[Types["One-Handed Swords"]] 	= false;
				[Types["Two-Handed Swords"]] 	= false;
				[Types["Staves"]] 				= false;
				[Types["Fist Weapons"]] 		= false;
				[Types["Miscellaneous"]] 		= true;
				[Types["Daggers"]] 				= false;
				[Types["Thrown"]] 				= false;
				[Types["Crossbows"]] 			= false;
				[Types["Wands"]] 				= false;
				[Types["Fishing Poles"]] 		= true;
			};
		};
		["PRIEST"] =  {
			[Types["Armor"]] = {
				[Types["Miscellaneous"]] 		= true;
				[Types["Cloth"]] 				= true;
				[Types["Leather"]] 				= false; 
				[Types["Mail"]] 				= false;
				[Types["Plate"]] 				= false;
				[Types["Shields"]] 				= false;
				[Types["Relic"]] 				= false;
			};
			[Types["Weapon"]] = {
				[Types["One-Handed Axes"]] 		= false;
				[Types["Two-Handed Axes"]] 		= false;
				[Types["Bows"]] 				= false;
				[Types["Guns"]] 				= false;
				[Types["One-Handed Maces"]] 	= true;
				[Types["Two-Handed Maces"]] 	= false;
				[Types["Polearms"]] 			= false;
				[Types["One-Handed Swords"]] 	= false;
				[Types["Two-Handed Swords"]] 	= false;
				[Types["Staves"]] 				= true;
				[Types["Fist Weapons"]] 		= false;
				[Types["Miscellaneous"]] 		= true;
				[Types["Daggers"]] 				= true;
				[Types["Thrown"]] 				= false;
				[Types["Crossbows"]] 			= false;
				[Types["Wands"]] 				= true;
				[Types["Fishing Poles"]] 		= true;
			};
		};
		["ROGUE"] =  {
			[Types["Armor"]] = {
				[Types["Miscellaneous"]] 		= true;
				[Types["Cloth"]] 				= true;
				[Types["Leather"]] 				= true; 
				[Types["Mail"]] 				= false;
				[Types["Plate"]] 				= false;
				[Types["Shields"]] 				= false;
				[Types["Relic"]] 				= false;
			};
			[Types["Weapon"]] = {
				[Types["One-Handed Axes"]] 		= true;
				[Types["Two-Handed Axes"]] 		= false;
				[Types["Bows"]] 				= true;
				[Types["Guns"]] 				= true;
				[Types["One-Handed Maces"]] 	= true;
				[Types["Two-Handed Maces"]] 	= false;
				[Types["Polearms"]] 			= true;
				[Types["One-Handed Swords"]] 	= true;
				[Types["Two-Handed Swords"]] 	= false;
				[Types["Staves"]] 				= false;
				[Types["Fist Weapons"]] 		= true;
				[Types["Miscellaneous"]] 		= true;
				[Types["Daggers"]] 				= true;
				[Types["Thrown"]] 				= true;
				[Types["Crossbows"]] 			= true;
				[Types["Wands"]] 				= false;
				[Types["Fishing Poles"]] 		= true;
			};
		};
		["SHAMAN"] =  {
			[Types["Armor"]] = {
				[Types["Miscellaneous"]] 		= true;
				[Types["Cloth"]] 				= true;
				[Types["Leather"]] 				= true; 
				[Types["Mail"]] 				= true;
				[Types["Plate"]] 				= false;
				[Types["Shields"]] 				= true;
				[Types["Relic"]] 				= true;
			};
			[Types["Weapon"]] = {
				[Types["One-Handed Axes"]] 		= true;
				[Types["Two-Handed Axes"]] 		= true;
				[Types["Bows"]] 				= false;
				[Types["Guns"]] 				= false;
				[Types["One-Handed Maces"]] 	= true;
				[Types["Two-Handed Maces"]] 	= true;
				[Types["Polearms"]] 			= true;
				[Types["One-Handed Swords"]] 	= false;
				[Types["Two-Handed Swords"]] 	= false;
				[Types["Staves"]] 				= true;
				[Types["Fist Weapons"]] 		= true;
				[Types["Miscellaneous"]] 		= true;
				[Types["Daggers"]] 				= true;
				[Types["Thrown"]] 				= false;
				[Types["Crossbows"]] 			= false;
				[Types["Wands"]] 				= false;
				[Types["Fishing Poles"]] 		= true;
			};
		};
		["WARLOCK"] =  {
			[Types["Armor"]] = {
				[Types["Miscellaneous"]] 		= true;
				[Types["Cloth"]] 				= true;
				[Types["Leather"]] 				= false; 
				[Types["Mail"]] 				= false;
				[Types["Plate"]] 				= false;
				[Types["Shields"]] 				= false;
				[Types["Relic"]] 				= false;
			};
			[Types["Weapon"]] = {
				[Types["One-Handed Axes"]] 		= false;
				[Types["Two-Handed Axes"]] 		= false;
				[Types["Bows"]] 				= false;
				[Types["Guns"]] 				= false;
				[Types["One-Handed Maces"]] 	= false;
				[Types["Two-Handed Maces"]] 	= false;
				[Types["Polearms"]] 			= false;
				[Types["One-Handed Swords"]] 	= true;
				[Types["Two-Handed Swords"]] 	= false;
				[Types["Staves"]] 				= true;
				[Types["Fist Weapons"]] 		= false;
				[Types["Miscellaneous"]] 		= true;
				[Types["Daggers"]] 				= true;
				[Types["Thrown"]] 				= false;
				[Types["Crossbows"]] 			= false;
				[Types["Wands"]] 				= true;
				[Types["Fishing Poles"]] 		= true;
			};
		};
		["WARRIOR"] =  {
			[Types["Armor"]] = {
				[Types["Miscellaneous"]] 		= true;
				[Types["Cloth"]] 				= true;
				[Types["Leather"]] 				= true; 
				[Types["Mail"]] 				= true;
				[Types["Plate"]] 				= true;
				[Types["Shields"]] 				= true;
				[Types["Relic"]] 				= false;
			};
			[Types["Weapon"]] = {
				[Types["One-Handed Axes"]] 		= true;
				[Types["Two-Handed Axes"]] 		= true;
				[Types["Bows"]] 				= true;
				[Types["Guns"]] 				= true;
				[Types["One-Handed Maces"]] 	= true;
				[Types["Two-Handed Maces"]] 	= true;
				[Types["Polearms"]] 			= true;
				[Types["One-Handed Swords"]] 	= true;
				[Types["Two-Handed Swords"]] 	= true;
				[Types["Staves"]] 				= true;
				[Types["Fist Weapons"]] 		= true;
				[Types["Miscellaneous"]] 		= true;
				[Types["Daggers"]] 				= true;
				[Types["Thrown"]] 				= true;
				[Types["Crossbows"]] 			= true;
				[Types["Wands"]] 				= false;
				[Types["Fishing Poles"]] 		= true;
			};
		};
	};
	
	-- items we want in different categories
	-- sorted alphabetically by their English names
	["extra"] = {
		["keys"] = {
			[30688] 	= true; -- Deathforge Key
			[18250] 	= true; -- Gordok Shackle Key
			[45798] 	= true; -- Heroic Celestial Planetarium Key
		};
		["misc"] = {
			[6219] 		= true; -- Arclight Spanner
			[5956] 		= true; -- Blacksmith Hammer
			[20815] 	= true; -- Jeweler's Kit
			[2901] 		= true; -- Mining Pick
			[22462] 	= true; -- Runed Adamantite Rod
			[16207] 	= true; -- Runed Arcanite Rod
			[52723] 	= true; -- Runed Elementium Rod
			[22463] 	= true; -- Runed Eternium Rod
			[22461] 	= true; -- Runed Fel Iron Rod
			[11130] 	= true; -- Runed Golden Rod
			[6339] 		= true; -- Runed Silver Rod
			[44452] 	= true; -- Runed Titanium Rod
			[11145] 	= true; -- Runed Truesilver Rod
			[20824] 	= true; -- Simple Grinder
			[7005] 		= true; -- Skinning Knife
			[39505] 	= true; -- Virtuoso Inking Set
		};
		["gizmos"] = {
			[18606] 	= true; -- Alliance Battle Standard
			[31666] 	= true; -- Battered Steam Tonk Controller
			[54343] 	= true; -- Blue Crashin' Thrashin' Racer Controller
			[49917] 	= true; -- Brazie's Gnomish Pleasure Device
			[34686] 	= true; -- Brazier of Dancing Flames
			[33927] 	= true; -- Brewfest Pony Keg
			[71137] 	= true; -- Brewfest Keg Pony
			[49704] 	= true; -- Carved Ogre Idol
			[37710] 	= true; -- Crashin' Thrashin' Racer Controller
			[23767] 	= true; -- Crashin' Thrashin' Robot
			[38301] 	= true; -- D.I.S.C.O.
			[36863] 	= true; -- Decahedral Dwarven Dice
			[30542] 	= true; -- Dimensional Ripper - Area 52
			[18984] 	= true; -- Dimensional Ripper - Everlook
			[37863] 	= true; -- Direbrew's Remote
			[21745] 	= true; -- Elder's Moonstone
			[21540] 	= true; -- Elune's Lantern
			[13508] 	= true; -- Eye of Arachnida
			[18232] 	= true; -- Field Repair Bot 74A
			[34113] 	= true; -- Field Repair Bot 110G
			[33223] 	= true; -- Fishing Chair
			[44719] 	= true; -- Frenzyheart Brew
			[54651] 	= true; -- Gnomeregan Pride
			[40772] 	= true; -- Gnomish Army Knife
			[18645] 	= true; -- Gnomish Alarm-o-Bot
			[18587] 	= true; -- Goblin Jumper Cables XL
			[18258] 	= true; -- Gordok Ogre Suit
			[44481] 	= true; -- Grindgear Toy Gorilla
			[70725] 	= true; -- Hallowed Hunter Wand - Squashling
			[20410] 	= true; -- Hallowed Wand - Bat
			[20409] 	= true; -- Hallowed Wand - Ghost
			[20399] 	= true; -- Hallowed Wand - Leper Gnome
			[20398] 	= true; -- Hallowed Wand - Ninja
			[20397] 	= true; -- Hallowed Wand - Pirate
			[20413] 	= true; -- Hallowed Wand - Random
			[20411] 	= true; -- Hallowed Wand - Skeleton
			[20414] 	= true; -- Hallowed Wand - Wisp
			[40110] 	= true; -- Haunted Memento
			[44601] 	= true; -- Heavy Copper Racer
			[45631] 	= true; -- High-powered Flashlight
			[43499] 	= true; -- Iron Boot Flask
			[52251] 	= true; -- Jaina's Locket
			[49040] 	= true; -- Jeeves
			[46709] 	= true; -- MiniZep Controller
			[40768] 	= true; -- MOLL-E
			[52201] 	= true; -- Muradin's Favor
			[35275] 	= true; -- Orb of the Sin' dorei
			[34499] 	= true; -- Paper Flying Machine Kit
			[49703] 	= true; -- Perpetual Purple Firework
			[32566] 	= true; -- Picnic Basket
			[46725] 	= true; -- Red Rider Air Rifle
			[34480] 	= true; -- Romantic Picnic Basket
			[40769] 	= true; -- Scrapbot Construction Kit
			[19045] 	= true; -- Stormpike Battle Standard
			[52253] 	= true; -- Sylvanas' Music Box
			[38578] 	= true; -- The Flag of Ownership
			[43824] 	= true; -- The Schools of Arcane Magic - Mastery
			[54438] 	= true; -- Tiny Blue Ragdoll
			[44430] 	= true; -- Titanium Seal of Dalaran
			[44606] 	= true; -- Toy Train Set
			[44482] 	= true; -- Trusty Copper Racer
			[45984] 	= true; -- Unusual Compass
			[18986] 	= true; -- Ultrasafe Transporter: Gadgetzan
			[30544] 	= true; -- Ultrasafe Transporter: Toshley's Station
			[34068] 	= true; -- Weighted Jack-o'-Lantern
			[45057] 	= true; -- Wind-Up Train Wrecker
			[17712] 	= true; -- Winter Veil Disguise Kit
			[18660] 	= true; -- World Enlarger
			[48933] 	= true; -- Wormhole Generator: Northrend
			[36862] 	= true; -- Worn Troll Dice
			[23821] 	= true; -- Zapthrottle Mote Extractor
			[44599] 	= true; -- Zippy Copper Racer

		};
	};
}

local styleFunc = function(self)
	if (self.Border) then
		self.Border:Hide()
	end

	if (self.Frame) and (self.Frame.SetTexture) then
		self.Frame:SetTexture("")
	end
	
	if (self.TotemBorder) then
		self.TotemBorder:SetTexture("")
		self.TotemBorder.SetTexture = noop
	end
	
	if (self.NormalTexture) then
		self.NormalTexture:ClearAllPoints()
		self.NormalTexture:SetPoint("TOPLEFT", self.Button, "TOPLEFT", -F.scale(3), F.scale(3))
		self.NormalTexture:SetPoint("BOTTOMRIGHT", self.Button, "BOTTOMRIGHT", F.scale(3), -F.scale(3))
		self.NormalTexture:SetVertexColor(C["normal"].r, C["normal"].g, C["normal"].b, 1)
	end
	
	if (self.Icon) then
		self.Icon:ClearAllPoints()
		self.Icon:SetPoint("TOPLEFT", self.Button, "TOPLEFT", F.scale(3), -F.scale(3))
		self.Icon:SetPoint("BOTTOMRIGHT", self.Button, "BOTTOMRIGHT", -F.scale(3), F.scale(3))
		self.Icon:SetTexCoord(5/64, 59/64, 5/64, 59/64)
	end

	if (not self:GetUIBackdrop()) then
		self:SetUITemplate(addon, "itembutton")
	end
	
	if (self.Highlight) then
		self.Highlight:Hide()
	end
	
	if (self.Overlay) then
		self.Overlay:ClearAllPoints()
		self.Overlay:SetPoint("TOPLEFT", self.Button, "TOPLEFT", F.scale(3), -F.scale(3))
		self.Overlay:SetPoint("BOTTOMRIGHT", self.Button, "BOTTOMRIGHT", -F.scale(3), F.scale(3))
	end
	
	if (self.Background) then
		self.Background:ClearAllPoints()
		self.Background:SetPoint("TOPLEFT", self.Button, "TOPLEFT", F.scale(3), -F.scale(3))
		self.Background:SetPoint("BOTTOMRIGHT", self.Button, "BOTTOMRIGHT", -F.scale(3), F.scale(3))
	end

	if (self.Autocast) then
		self.Autocast:ClearAllPoints()
		self.Autocast:SetPoint("TOPLEFT", self.Button, "TOPLEFT", -F.scale(5), F.scale(5))
		self.Autocast:SetPoint("BOTTOMRIGHT", self.Button, "BOTTOMRIGHT", F.scale(5), -F.scale(5))
	end

	if (self.Shine) then
		self.Shine:ClearAllPoints()
		self.Shine:SetPoint("TOPLEFT", self.Button, "TOPLEFT", F.scale(3), -F.scale(3))
		self.Shine:SetPoint("BOTTOMRIGHT", self.Button, "BOTTOMRIGHT", -F.scale(3), F.scale(3))
	end

	if (self.Cooldown) then
		self.Cooldown:ClearAllPoints()
		self.Cooldown:SetPoint("TOPLEFT", self.Button, "TOPLEFT", F.scale(3), -F.scale(3))
		self.Cooldown:SetPoint("BOTTOMRIGHT", self.Button, "BOTTOMRIGHT", -F.scale(3), F.scale(3))
	end
	
	if (self.Count) then
		self.Count:ClearAllPoints()
		self.Count:SetPoint("BOTTOMRIGHT", self.Button, "BOTTOMRIGHT", -F.scale(2), F.scale(2))
		self.Count:SetFontObject(gBAGS_NumberFontTiny)
		self.Count:SetJustifyH("RIGHT")
		self.Count:SetJustifyV("BOTTOM")
		self.Count:SetDrawLayer("OVERLAY")
	end

	if (self.Hotkey) then
		self.Hotkey:ClearAllPoints()
		self.Hotkey:SetPoint("TOPRIGHT", self.Button, "TOPRIGHT", 0, 0)
		self.Hotkey:SetWidth(self.Button:GetWidth())
		self.Hotkey:SetFontObject(gBAGS_ItemButtonHotKeyFont)
		self.Hotkey:SetJustifyH("RIGHT")
		self.Hotkey:SetJustifyV("TOP")
		self.Hotkey:SetDrawLayer("OVERLAY")
	end

	if (self.Macro) then
		self.Macro:ClearAllPoints()
		self.Macro:SetPoint("BOTTOMLEFT", self.Button, "BOTTOMLEFT", F.scale(2), F.scale(2))
		self.Macro:SetWidth(self.Button:GetWidth())
		self.Macro:SetFontObject(gBAGS_ItemButtonNameFont)
		self.Macro:SetJustifyH("LEFT")
		self.Macro:SetJustifyV("BOTTOM")
		self.Macro:SetDrawLayer("OVERLAY")
	end

	if (self.Flash)  then
		self.Flash:SetTexture(1, 0, 0, 1/3)
		self.Flash:SetPoint("TOPLEFT", self.Button, "TOPLEFT", F.scale(3), -F.scale(3))
		self.Flash:SetPoint("BOTTOMRIGHT", self.Button, "BOTTOMRIGHT", -F.scale(3), F.scale(3))
	end
	
	if (self.Button.SetHighlightTexture) and (not _G[self.Name.."HoverTex"]) then
		local hover = self.Button:CreateTexture(self.Name.."HoverTex", nil, self)
		hover:SetTexture(C["hover"].r, C["hover"].g, C["hover"].b, 0.3)
		hover:ClearAllPoints()
		hover:SetPoint("TOPLEFT", self.Button, "TOPLEFT", F.scale(3), -F.scale(3))
		hover:SetPoint("BOTTOMRIGHT", self.Button, "BOTTOMRIGHT", -F.scale(3), F.scale(3))
		self.Button:SetHighlightTexture(hover)
	end
 
	if (self.Button.SetPushedTexture) and (not _G[self.Name.."PushedTex"]) then
		local pushed = self.Button:CreateTexture(self.Name.."PushedTex", nil, self)
		pushed:SetTexture(C["pushed"].r, C["pushed"].g, C["pushed"].b, 0.3)
		pushed:ClearAllPoints()
		pushed:SetPoint("TOPLEFT", self.Button, "TOPLEFT", F.scale(3), -F.scale(3))
		pushed:SetPoint("BOTTOMRIGHT", self.Button, "BOTTOMRIGHT", -F.scale(3), F.scale(3))
		self.Button:SetPushedTexture(pushed)
	end
 
	if (self.Button.SetCheckedTexture) and (not _G[self.Name.."CheckedTex"]) then
		local checked = self.Button:CreateTexture(self.Name.."CheckedTex", nil, self)
		checked:SetTexture(C["checked"].r, C["checked"].g, C["checked"].b, 0.3)
		checked:ClearAllPoints()
		checked:SetPoint("TOPLEFT", self.Button, "TOPLEFT", F.scale(3), -F.scale(3))
		checked:SetPoint("BOTTOMRIGHT", self.Button, "BOTTOMRIGHT", -F.scale(3), F.scale(3))
		self.Button:SetCheckedTexture(checked)
	end
	
	self.Button:SetNormalTexture("")

	--[[
	if (self.NormalTexture) then
		if self.action and IsEquippedAction(self.action) and self.ItemRarity and self.ItemRarity > 1 then
			local r, g, b, _ = GetItemQualityColor(self.ItemRarity)
			self.NormalTexture:SetVertexColor(r, g, b, 1)
		else
			self.NormalTexture:SetVertexColor(C["normal"].r, C["normal"].g, C["normal"].b, 1)
		end
	end
	]]--

	if (self.action) and (IsEquippedAction(self.action)) and (self.ItemRarity) and (self.ItemRarity > 1) then  
		local r, g, b, _ = GetItemQualityColor(self.ItemRarity)
		self:SetUIBackdropBorderColor(r * 4/5, g * 4/5, b * 4/5, 1)
	else
		self:SetUIBackdropBorderColor(C["normal"].r, C["normal"].g, C["normal"].b, 1)
	end
	
	if (self.TotemBorder) then
		local tc = C["totem"][(((self:GetID() - 1) % 4) + 1)]
		
		--[[
		if (self.NormalTexture) then
			self.NormalTexture:SetVertexColor(tc.r, tc.g, tc.b, 1)
		end
		]]--
		
		self:SetUIBackdropBorderColor(tc.r * 4/5, tc.g * 4/5, tc.b * 4/5, 1)
	end
end

-- register the style with gActionButtons, but add the 'cold' flag to prevent it from initializing
LibStub("gActionButtons-2.0"):SetStyleFunction(styleFunc, true)


--
-- 	Hook/Unhook a tooltip to the plugin
-- 	Will replace any other existing tooltip
--
local PlaceGameTooltip = function(anchor, horTip)
	GameTooltip:SetOwner(anchor, "ANCHOR_NONE")
	GameTooltip:ClearAllPoints()
	
	if (horTip) then
		if (GetScreenWidth() - anchor:GetRight()) > anchor:GetLeft() then
			GameTooltip:SetPoint("TOPLEFT", anchor, "TOPRIGHT", 8, 0)
		else
			GameTooltip:SetPoint("TOPRIGHT", anchor, "TOPLEFT", -8, 0)
		end
	else
		if (GetScreenHeight() - anchor:GetTop()) > anchor:GetBottom() then
			GameTooltip:SetPoint("BOTTOM", anchor, "TOP", 0, 8)
		else
			GameTooltip:SetPoint("TOP", anchor, "BOTTOM", 0, -8)
		end
	end
end

local restackThread, restackTimer

-- define containers using Blizzard globals, for safety and future compability
local stackContainers = { bags = { 0 }; bank = { BANK_CONTAINER }; }
for i = 1, NUM_BAG_SLOTS, 1 do 
	tinsert(stackContainers.bags, i) 
end

for i = NUM_BAG_SLOTS + 1, NUM_BAG_SLOTS + NUM_BANKBAGSLOTS, 1 do 
	tinsert(stackContainers.bank, i) 
end

local yBagID, ySlot
local stackYield = function(bagID, slot)
	yBagID, ySlot = bagID, slot
	restackTimer = restackTimer or ScheduleTimer(function()
		if (type(restackThread) == "thread") and (coroutine.status(restackThread) == "suspended") then
			local locked = true
			locked = select(3, GetContainerItemInfo(yBagID, ySlot))
			if not(locked) then 
				coroutine.resume(restackThread) 
			end
		end
	end, 0.1)
	
	coroutine.yield()
end

local RestackRun = function(location, item)
	if (location == "resume") then
		if (type(restackThread) == "thread") and (coroutine.status(restackThread) == "suspended") then
			print(L["RESTACK_RESUMING"])
			coroutine.resume(restackThread)
		else
			print(L["RESTACK_NOTHING_TO_RESUME"])
		end
	end
	
	if (type(restackThread) ~= "thread") or (coroutine.status(restackThread) == "dead") then
		restackThread = coroutine.create(function()
			local changed = true
			while (changed) do
				changed = false
				
				for bagNum = 1, #stackContainers[location], 1 do
					local bagID = stackContainers[location][bagNum]
					
					for slot = 1, GetContainerNumSlots(bagID) do
						local itemID = GetContainerItemID(bagID, slot)
						if (itemID) then
							
							local _, itemCount, locked, _, _ = GetContainerItemInfo(bagID, slot)
							local name, _, _, _, _, itemType, itemSubType, itemStackCount, _, _, _ = GetItemInfo(itemID) 
							
							if (locked) then
								stackYield(bagID, slot)
							end
							
							-- partial stack discovered
							local found, fBagID, fSlot, fLocked, fDone
							if (itemStackCount > itemCount) then
								
								while true do
									-- backtrack through the bags for another partial match
									for rBagNum = #stackContainers[location], 1, -1 do
										local rBagID = stackContainers[location][rBagNum]
										if (found) or (fDone) then 
											break 
										end
										
										for rSlot = GetContainerNumSlots(rBagID), 1, -1 do
											if (rBagID == bagID) and (rSlot == slot) then
												fDone = true
												break
											end
											
											local rItemID = GetContainerItemID(rBagID, rSlot)
											if (rItemID) and (rItemID == itemID) then
												local _, rItemCount, rLocked, _, _ = GetContainerItemInfo(rBagID, rSlot)
												local _, _, _, _, _, rItemType, rItemSubType, rItemStackCount, _, _, _ = GetItemInfo(rItemID) 
												
												if (rItemStackCount > rItemCount) then
													found = true
													fLocked = rLocked
													fBagID, fSlot = rBagID, rSlot
													break
												end
											end
										end
									end
									
									fLocked = (found) and (select(3, GetContainerItemInfo(fBagID, fSlot))) or false
									if (fLocked) then
										stackYield(fBagID, fSlot)
									else
										break
									end
								end

								if (found) then
									ClearCursor()
									
									PickupContainerItem(bagID, slot)
									PickupContainerItem(fBagID, fSlot)
									
									ClearCursor()
									
									changed = true
									break
								end
							end
						end
					end
				end
			end
			if (restackTimer) then
				CancelTimer(restackTimer)
				restackTimer = nil
			end
		end)
		coroutine.resume(restackThread)
	else
		print(L["RESTACK_IN_SESSION"])
	end
end


--
-- a function to restack all the items in the given bags
-- bags: 
local RestackBags = function(location)
	if not(location) then return end

	RestackRun(location)
end

local CreateButton = function(button)
	button:SetSize(buttonSize, buttonSize)

	-- attempt to use gActionButtons styling for the bags
	-- any style registered will be applied
	StyleActionButton(button)
	
	button.Border 		= _G[button:GetName().."NormalTexture"]
	button.Cooldown 	= _G[button:GetName().."Cooldown"]
	button.Count 		= _G[button:GetName().."Count"]
	button.Icon 		= _G[button:GetName().."IconTexture"]
	button.Quest 		= _G[button:GetName().."IconQuestTexture"]
		
	if (button.Quest) then
		button.Quest:SetTexCoord(5/64, 59/64, 5/64, 59/64)
		button.Quest:SetAllPoints(button.Icon)
		button.Quest:SetBlendMode("BLEND")
	end
	
	button.glowAlpha = 0.55
	button.glowBlend = "ADD"
	button.glowCoords = { 16/64, 47/64, 16/64, 47/64 }
end

local highlightFunction = function(button, match)
	button:SetAlpha(match and 1 or 0.1)
end

--
-- our own quite glorious sort function
-- ItemRarity > ItemLevel > ItemID > StackSize > BagType > BagID
local sorts = cargBags.classes.Container.sorts
sorts.smartSort = function(a, b)
	local aItemID, bItemID = GetContainerItemID(a.bagID, a.slotID) or 0, GetContainerItemID(b.bagID, b.slotID) or 0
	local aRarity, bRarity = aItemID and (select(3, GetItemInfo(aItemID))) or 0, bItemID and (select(3, GetItemInfo(bItemID))) or 0
	if (aRarity == bRarity) then
		local aLevel, bLevel = aItemID and (select(4, GetItemInfo(aItemID))) or 1, bItemID and (select(4, GetItemInfo(bItemID))) or 1
		if (aLevel == bLevel) then
			if (aItemID == bItemID) then
				local aCount, bCount = (select(2,GetContainerItemInfo(a.bagID, a.slotID))) or 1, (select(2,GetContainerItemInfo(b.bagID, b.slotID))) or 1
				if (aCount == bCount) then
					local aBagType, bBagType = (select(2, GetContainerNumFreeSlots(a.bagID))), (select(2, GetContainerNumFreeSlots(b.bagID)))
					if (aBagType == bBagType) then
						return (a.bagID > b.bagID)
					else
						return (aBagType > bBagType) 
					end
				else
					return (aCount > bCount)
				end
			else
				return (aItemID > bItemID)
			end
		else
			return (aLevel > bLevel)
		end
	else
		return (aRarity > bRarity)
	end
end

--
-- our new layout function
-- * empty spaces are compressed
local layouts = cargBags.classes.Container.layouts
layouts.smartGrid = function(self, columns, spacing, xOffset, yOffset)
	columns, spacing = columns or 8, spacing or 5
	xOffset, yOffset = xOffset or 0, yOffset or 0

	local width, height = 0, 0
	local col, row = 0, 0
	local shown = 0
	
	local bagType, lastBagType
	local itemID, lastItemID
	
	for i, button in ipairs(self.buttons) do

		if (i == 1) then 
			width, height = button:GetSize()
		end
		
		itemID = GetContainerItemID(button.bagID, button.slotID) or 0
		bagType = (select(2, GetContainerNumFreeSlots(button.bagID)))
		
		local newButton
		if ((itemID == 0) and (itemID == lastItemID) and (bagType == lastBagType)) then
			if (shown ~= 0) and not(shown % columns == 0) then
				newButton = true
			end
		else
			newButton = true
		end
		
		if (newButton) or (gBAGS_DB["bags"].compressemptyspace == 0) then
			shown = shown + 1
			
			col = shown % columns
			row = ceil(shown/columns)

			if (col == 0) then col = columns end

			local xPos = (col-1) * (width + spacing)
			local yPos = -1 * (row-1) * (height + spacing)

			button:ClearAllPoints()
			button:SetPoint("TOPLEFT", self, "TOPLEFT", xPos+xOffset, yPos+yOffset)
			button:Show()
		else
			button:Hide()
		end
		
		lastBagType = bagType
		lastItemID = itemID
	end

	return columns * (width+spacing)-spacing, row * (height+spacing)-spacing
end

local updateAllContainers = function() 
	for name,container in pairs(ContainerList) do
		container:OnContentsChanged()
	end
end

local createCloseButton = function(parent, tooltip, closeFunction, ...)
	tooltip = tooltip or CLOSE

	local close = CreateFrame("Button", parent:GetName() and parent:GetName() .. "CloseButton" or nil, parent)
	close:SetNormalTexture(M["Button"]["UICloseButton"])
	close:SetPushedTexture(M["Button"]["UICloseButtonDown"])
	close:SetHighlightTexture(M["Button"]["UICloseButtonHighlight"])
	close:SetDisabledTexture(M["Button"]["UICloseButtonDisabled"])
	close:EnableMouse(true)
	close:SetSize(16, 16)

	close:SetScript("OnClick", closeFunction)
	
	close:SetScript("OnEnter", function(self)
		if (not InCombatLockdown()) then
			PlaceGameTooltip(self, false)
			GameTooltip:AddLine(tooltip)
			GameTooltip:Show()
		end
	end)
	
	close:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
	end)

	close:SetScript("OnShow", function(self)
		self:SetFrameLevel(self:GetParent():GetFrameLevel() + 5)
	end)

	if (...) then
		close:ClearAllPoints()
		close:SetPoint(...)
	end
	
	
	parent.UICloseButton = close
	
	return close
end

module["TRADE_SKILL_UPDATE"] = function(self, event)
	updateAllContainers()
end

module["CHAT_MSG_LOOT"] = function(self, event)
	updateAllContainers()
end

module.Init = function(self)
	gBAGS_DB["bags"] = gBAGS_DB["bags"] or {}
	gBAGS_DB["bags"] = ValidateTable(gBAGS_DB["bags"], defaults)
end

module.OnInit = function(self)
	InterfaceOptionsDisplayPanelShowFreeBagSpace:SetScale(1/1e4)
	InterfaceOptionsDisplayPanelShowFreeBagSpaceText:SetSize(1/1e4, 1/1e4)
end

module.OnEnable = function(self)

	Bags = cargBags:NewImplementation(self:GetName() .. "Frame"); 
	Bags:RegisterBlizzard()
	
	Container = Bags:GetContainerClass()
	Button = Bags:GetItemButtonClass()
	BagButton = Bags:GetBagButtonClass()

	Button:Scaffold("Default")
	
	Button.OnCreate = function(self, tpl)
		CreateButton(self)
	end

	Button.OnUpdate = function(self, item)
		local bagType = (select(2, GetContainerNumFreeSlots(self.bagID)))
		
		if (self.Quest) then
			self.Quest:SetTexCoord(5/64, 59/64, 5/64, 59/64)
			self.Quest:SetAllPoints(self.Icon)
			self.Quest:SetBlendMode("BLEND")
		end

		local hex, texture
		local blend 			= "BLEND"
		local r, g, b, a 		= unpack(C["border"])
		local tL, tR, tT, tB 	= 5/64, 59/64, 5/64, 59/64

		if ((item) and ((item.questID) and (not item.questActive))) then
			texture = TEXTURE_ITEM_QUEST_BANG
			r, g, b, a = 1, 1, 1, 1

		elseif ((item) and ((item.questID) or (item.isQuestItem))) then
			texture = TEXTURE_ITEM_QUEST_BORDER
			r, g, b, a = 1, 1, 1, 1
			
		elseif ((item) and ((item.rarity) and (item.rarity > 1) and (self.glowTex))) then
			a, r, g, b, hex = self.glowAlpha, GetItemQualityColor(item.rarity)
			texture = self.glowTex
			blend = self.glowBlend
			tL, tR, tT, tB = unpack(self.glowCoords)
		end

		if (item) and (self.glowTex) then
			local itemID = GetContainerItemID(item.bagID, item.slotID)
			
			if (itemID) then
				local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(itemID) 
				
				local canUse = true
				if (_G[itemEquipLoc]) and ((itemType == Types["Armor"]) or (itemType == Types["Weapon"])) then
					local classEquiplist = db.equiplist[(select(2, UnitClass("player")))]
					local classException = db.equiplist.EXCEPTIONS[(select(2, UnitClass("player")))]
					local canWear = (classEquiplist[itemType]) and (classEquiplist[itemType][itemSubType])
					
					if (canWear) then
						if (itemMinLevel) and (itemMinLevel > UnitLevel("player")) then
							canUse = false
						end
						
						if (classException) and (classException[itemSubType]) and (classException[itemSubType] > UnitLevel("player")) then
							canUse = false
						end
					else
						canUse = false
					end
				end
				
				if (not canUse) then
					self.Icon:SetVertexColor(C["range"].r, C["range"].g, C["range"].b)

					a, r, g, b = self.glowAlpha, C["range"].r, C["range"].g, C["range"].b
					texture = self.glowTex
					blend = self.glowBlend
					tL, tR, tT, tB = unpack(self.glowCoords)
				else
					self.Icon:SetVertexColor(1, 1, 1)
				end
			end
		end

		if (texture) then
			self.Quest:SetTexture(texture)
			self.Quest:SetTexCoord(tL, tR, tT, tB)
			self.Quest:SetBlendMode(blend)
			self.Quest:SetVertexColor(r, g, b, a)
			self.Quest:Show()
		else
			self.Quest:Hide()
		end
		
		--
		-- color the borders, backdrop and overlay of the slots
		if (item) and (((item.isQuestItem) or (item.questID)) or ((item.questID) and (not item.questActive))) then
			self:SetUIBackdropBorderColor(0.8, 0.8, 0.5, 1)
			self:SetUIBackdropColor(C["backdrop"][1], C["backdrop"][2], C["backdrop"][3], 1)
			
		elseif (item) and (item.rarity) and (item.rarity > 1) then
			local r, g, b, hex = GetItemQualityColor(item.rarity)
			self:SetUIBackdropBorderColor(r * 4/5, g * 4/5, b * 4/5, 1)
			self:SetUIBackdropColor(C["backdrop"][1], C["backdrop"][2], C["backdrop"][3], 1)
			
		elseif (bagType == 256) then
			self:SetUIBackdropBorderColor(1, 0.7, 0.1, 1)
			self:SetUIBackdropColor(C["backdrop"][1], C["backdrop"][2], C["backdrop"][3], 1)
			
		elseif (bagType and bagType > 4) then
			self:SetUIBackdropBorderColor(1, 0.65, 0.1, 1)
			self:SetUIBackdropColor(C["backdrop"][1], C["backdrop"][2], C["backdrop"][3], 1)
			
		elseif (self.bagID >= 0) and (self.bagID <= 4) then
			self:SetUIBackdropBorderColor(unpack(C["border"]))
			self:SetUIBackdropColor(C["backdrop"][1], C["backdrop"][2], C["backdrop"][3], 1)
			
		else 
			self:SetUIBackdropBorderColor(C["border"][1], C["border"][2], C["border"][3] * 2, 1)
			self:SetUIBackdropColor(C["backdrop"][1], C["backdrop"][2], C["backdrop"][3] * 2, 1)
		end
		
	end

	BagButton.OnCreate = function(self)
		CreateButton(self)
		
		self:GetCheckedTexture():SetVertexColor(1, 0.8, 0, 0.8)
		self:GetCheckedTexture():SetTexCoord(5/64, 59/64, 5/64, 59/64)
	end
	
	local top, bottom, space = 24, 32, 8
	Container.OnContentsChanged = function(self)
		self:SortButtons("smartSort")

		local extra = 0
		if (self.Settings.Name == L["MAIN"]) then
			extra = (self.currencies:GetText() ~= "") and (self.currencies:GetHeight() + 4) or 0
		end

		local _, height = self:LayoutButtons("smartGrid", self.Settings.Columns, 2, space, -(space + top + extra))
		local width = (2 + buttonSize) * self.Settings.Columns - 2
		
		if (not self:IsShown()) then
			height = 0
		end

		if (self.Settings.Name == L["MAIN"]) or (self.Settings.Name == L["BANK"]) then
			self:SetSize(width + space * 2, height + top + bottom + space * 2 + extra)
			self:SetAlpha(1)

			self.UICloseButton:SetFrameStrata(self:GetFrameStrata())
			
		elseif (height <= 0) then
			self:SetSize(width + space * 2, 0.01);
			self:SetAlpha(0)
			
			self.UICloseButton:SetFrameStrata("BACKGROUND")
						
		else
			self:SetSize(width + space * 2, height + top + space * 2)
			self:SetAlpha(1)

			self.UICloseButton:SetFrameStrata(self:GetFrameStrata())
		end
	end

	Container.OnCreate = function(self, name, settings)
		self.Settings = settings
		
		self:EnableMouse(true)
		self:SetClampedToScreen(true)
		self:SetParent(settings.Parent or Bags)
		self:SetFrameStrata("HIGH")
		self:SetFrameLevel(50)

		self.Panel = self:CreateUIPanel()
		self.Panel:SetUITemplate(addon)
		self.Panel:SetFrameLevel(40)
		self.Panel:ClearAllPoints()
		self.Panel:SetPoint("TOPLEFT", self, "TOPLEFT", 2, -2)
		self.Panel:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", -2, 2)
	
		if strfind(name, "Bank") then
			self.Panel:SetUIBackdropColor(unpack(C["bank"]))
			self.Panel:SetUIBackdropBorderColor(C["border"][1] * 5/4, C["border"][2] * 5/4, C["border"][3] * 2 * 5/4, 1)
		else
			self.Panel:SetUIBackdropColor(unpack(C["overlay"]))
			self.Panel:SetUIBackdropBorderColor(unpack(C["border"]))
		end
		
		if settings.Movable then
			self:SetMovable(true)
			self:RegisterForClicks("LeftButton", "RightButton")
			
			self:SetScript("OnMouseDown", function(self)
				if (gBAGS_DB["bags"].locked == 0) then
					self:StartMoving()
				end
			end)
			
			self:SetScript("OnMouseUp",  function(self)
				self:StopMovingOrSizing()
				gBAGS_DB["bags"].points[settings.Name and settings.Name or name] = {GetPoint(self)}
			end)
		end

		settings.Columns = settings.Columns or 8
		self:SetScale(gBAGS_DB["bags"].scale and gBAGS_DB["bags"].scale or settings.Scale and settings.Scale or 1)
		
		local title = self:CreateFontString(nil, "OVERLAY")
		title:SetFontObject(gBAGS_SystemFontNormal)
		title:SetPoint("TOPLEFT", self, "TOPLEFT", 10, -12)
		title:SetText("|cFF"..(format("%02x%02x%02x", C["value"][1] * 255, C["value"][2] * 255, C["value"][3] * 255))..settings.Name.."|r")
		
		self:SetScript("OnHide", function(self) 
			self:OnContentsChanged()
		end)

		self:SetScript("OnShow", function(self) 
			self:OnContentsChanged()
		end)
		
		if not(ContainerList[name]) then
			ContainerList[name] = self
		end

		if (self.Settings.Name == L["MAIN"]) or (self.Settings.Name == L["BANK"]) then
			local close = createCloseButton(self, L["BAGS_CLOSE_ALL_TOOLTIP"], function() 
				if (Bags:AtBank()) then 
					CloseBankFrame()
				else 
					CloseAllBags()
				end 
			end, "TOPRIGHT", -6, -8)
				
			local infoFrame = CreateFrame("Button", nil, self)
			infoFrame:SetPoint("BOTTOMLEFT", 10, 3)
			infoFrame:SetPoint("BOTTOMRIGHT", -10, 3)
			infoFrame:SetHeight(bottom)

			local money = self:SpawnPlugin("TagDisplay", "[money]", self)
			money:SetFontObject(gBAGS_NumberFontNormal)
			money.iconValues = "0:0:0:0"
			money:SetPoint("TOPRIGHT", self, "TOPRIGHT", -40, -12)
			
			local freespace = self:SpawnPlugin("TagDisplay", "[space:free/max]", infoFrame)
			freespace:SetFontObject(gBAGS_NumberFontHuge) 
			freespace:SetPoint("LEFT", infoFrame, "LEFT")
			freespace.bags = cargBags:ParseBags(settings.Bags)

			--[[
			local text = infoFrame:CreateFontString(nil, "OVERLAY")
			text:SetFontObject(gBAGS_SystemFontTiny)
			text:SetPoint("TOPLEFT", freespace, "TOPRIGHT", 2, 0)
			text:SetText(L["FREE"])
			]]--
			
			local spacegraph = CreateFrame("StatusBar", nil, infoFrame)
			spacegraph:SetBackdrop({ bgFile = M["StatusBar"]["ProgressBar"] })
			spacegraph:SetBackdropColor(1, 1, 1, 0.2)
			spacegraph:SetSize(80, 20)
			spacegraph:SetPoint("BOTTOMLEFT", infoFrame, "BOTTOMLEFT", 72, 8)

			spacegraph.bar = spacegraph:CreateTexture(nil, "OVERLAY")
			spacegraph.bar:SetSize(80, 20)
			spacegraph.bar:SetPoint("TOPLEFT", spacegraph, "TOPLEFT", 0, 0)
			spacegraph.bar:SetTexture(M["StatusBar"]["ProgressBar"])

			local bags = (self.Settings.Name == L["MAIN"] and { 0, 1, 2, 3, 4 }) or (self.Settings.Name == L["BANK"] and { -1, 5, 6, 7, 8, 9, 10, 11 }) or {}
			local function setFree()
				local free, total, used = 0, 0, 0
				for _,i in pairs(bags) do
					free, total, used = free + GetContainerNumFreeSlots(i), total + GetContainerNumSlots(i), total - free
				end
				free = free / total
				spacegraph.bar:SetWidth(((free > 0) and free or 0.0001) * 80)
				spacegraph.bar:SetTexCoord(0, 0, 0,1, free, 0, free, 1)
			end
			setFree()

			spacegraph:SetScript("OnEvent", setFree)
			spacegraph:RegisterEvent("BAG_UPDATE")
			if (self.Settings.Name == L["BANK"]) then 
				spacegraph:RegisterEvent("BANKFRAME_OPENED");
			end

			local search = self:SpawnPlugin("SearchBar", infoFrame)
			search.highlightFunction = highlightFunction
			search.isGlobal = true
		
			local bagBar = self:SpawnPlugin("BagBar", settings.Bags)
			bagBar:SetFrameStrata(self:GetFrameStrata())
			bagBar:SetFrameLevel(self:GetFrameLevel() + 20)
			bagBar:Hide()

			local width, height = bagBar:LayoutButtons("grid", (self.Settings.Name == L["MAIN"]) and 4 or (self.Settings.Name == L["BANK"]) and 7, 4, space * 1/2, -space * 1/2)
			bagBar:SetSize(width + space, height + space)
			bagBar:SetPoint("TOPRIGHT", self, "BOTTOMRIGHT", -space * 1/2, -space * 1/2)
			bagBar.highlightFunction = highlightFunction
			bagBar.isGlobal = true

			bagBar:CreateUIShadow()
			bagBar:SetUIBackdrop(self.Panel:GetUIBackdrop())
			bagBar:SetUIBackdropColor(self.Panel:GetUIBackdropColor())
			bagBar:SetUIBackdropBorderColor(self.Panel:GetUIBackdropBorderColor())

			bagBar.Button = CreateFrame("Button", nil, infoFrame)
			bagBar.Button:SetNormalTexture(M["Icon"]["BagIcon"])
			bagBar.Button:SetPushedTexture(M["Icon"]["BagIcon"])
			bagBar.Button:SetHighlightTexture(M["Icon"]["BagIconHighlight"])
			bagBar.Button:SetDisabledTexture(M["Icon"]["BagIconDisabled"])
			bagBar.Button:EnableMouse(true)
			bagBar.Button:SetSize(32, 32)
			bagBar.Button:SetPoint("BOTTOMRIGHT", infoFrame, "BOTTOMRIGHT", 3, 1)
			bagBar.Button:SetScript("OnClick", function() 
				if (bagBar:IsShown()) then
					bagBar:Hide()
				else
					bagBar:Show()
				end
			end)
			bagBar.Button:SetScript("OnEnter", function(self)
				if (not InCombatLockdown()) then
					PlaceGameTooltip(self, false)
					GameTooltip:AddLine(L["BAGS_BAGBAR_TOOLTIP"])
					GameTooltip:Show()
				end
			end)
			bagBar.Button:SetScript("OnLeave", function(self)
				GameTooltip:Hide()
			end)
			
			local restackButton = CreateFrame("Button", nil, infoFrame)
			restackButton:SetNormalTexture(M["Icon"]["BagRestack"])
			restackButton:SetDisabledTexture(M["Icon"]["BagRestackDisabled"])
			restackButton:SetHighlightTexture(M["Icon"]["BagRestackHighlight"])
			restackButton:SetPushedTexture(M["Icon"]["BagRestack"])
			
			restackButton:SetSize(64, 32)
			restackButton:SetPoint("BOTTOMRIGHT", infoFrame, "BOTTOMRIGHT", -38, -1)
			
			local this = self
			restackButton:SetScript("OnClick", function(self, button) 
				if (button == "LeftButton") then
					local location = (this.Settings.Name == L["MAIN"]) and "bags" or (this.Settings.Name == L["BANK"]) and "bank"
					
					RestackBags(location)
				end
			end)
			restackButton:SetScript("OnEnter", function(self) 
				if (not InCombatLockdown()) then
					PlaceGameTooltip(self, false)
					GameTooltip:AddLine((self:IsEnabled()) and L["RESTACK_TOOLTIP"] or L["RESTACK_IN_SESSION"])
					GameTooltip:Show()
				end
			end)
			restackButton:SetScript("OnLeave", function(self)
				GameTooltip:Hide()
			end)
			
			
			if (self.Settings.Name == L["MAIN"]) then
				local currencies = self:SpawnPlugin("TagDisplay", "[currencies]", infoFrame)
				currencies:SetFontObject(gBAGS_NumberFontSmall)
				currencies.iconValues = "0:0:0:0"
				currencies:SetPoint("TOPRIGHT", money, "BOTTOMRIGHT", -4, -4)

				currencies.clickframe = CreateFrame("Button", nil, infoFrame)
				currencies.clickframe:SetFrameLevel(infoFrame:GetFrameLevel() + 5)
				currencies.clickframe:SetAllPoints(currencies)
			
				currencies.clickframe:SetScript("OnClick", function(self) 
					ToggleCharacter("TokenFrame")
				end)
				
				currencies.clickframe:SetScript("OnEnter", function(self)
					if (not InCombatLockdown()) then
						PlaceGameTooltip(self, false)
						GameTooltip:AddLine(L["BAGS_CURRENCIES_TOOLTIP"])
						GameTooltip:Show()
					end
				end)
				
				currencies.clickframe:SetScript("OnLeave", function(self)
					GameTooltip:Hide()
				end)
				

				self.currencies = currencies
				
				-- this little magic is to make sure the currency display as well as the height of the bag
				-- is updated dynamically when we change our tracked currencies
				local main = self
				local update = function()
					if main.updating then return end
					main.updating = true

					currencies.tokens = GetNumWatchedTokens()
					currencies.forceEvent("CURRENCY_DISPLAY_UPDATE") -- we wrote this little function into cargBags. We're outlaws!

					main:ScheduleContentCallback()

					main.updating = nil
				end
				hooksecurefunc("ManageBackpackTokenFrame", update)
			end

			if (self.Settings.Name == L["BANK"]) then
				local purchase = CreateFrame("Frame", nil, infoFrame)
				
				BankFramePurchaseButton:SetParent(purchase)
				BankFramePurchaseButton:SetFrameStrata(self:GetFrameStrata())
				BankFramePurchaseButton:SetFrameLevel(self:GetFrameLevel() + 5)
				BankFramePurchaseButton:ClearAllPoints()
				BankFramePurchaseButton:SetPoint("RIGHT", bagBar.text, "LEFT", -36, 2)
				BankFramePurchaseButton:SetScript("OnClick", function(self) 
					PlaySound("igMainMenuOption")
					StaticPopup_Show("CONFIRM_BUY_BANK_SLOT")
				end)
				
				local numSlots, full = GetNumBankSlots()
				local cost = GetBankSlotCost(numSlots)
				local purchasecolor
				BankFrame.nextSlotCost = cost

				if (GetMoney() >= cost) then
					purchasecolor = "|cFFFFFFFF"
				else
					purchasecolor = "|cFFFF0000"
				end			
				purchase.text = purchase:CreateFontString(nil, "OVERLAY")
				purchase.text:SetFontObject(Tooltip_Med)
				purchase.text:SetPoint("RIGHT", BankFramePurchaseButton, "LEFT", -12, 0)
				purchase.text:SetText(purchasecolor..COSTS_LABEL.."  "..("[money:%d]"):format(cost):tag().."|r")
				
				purchase:RegisterEvent("PLAYERBANKBAGSLOTS_CHANGED")
				purchase:SetScript("OnEvent", function(self) 
					local numSlots, full = GetNumBankSlots()
					local cost = GetBankSlotCost(numSlots)
					local purchasecolor
					BankFrame.nextSlotCost = cost

					if (GetMoney() >= cost) then
						purchasecolor = "|cFFFFFFFF"
					else
						purchasecolor = "|cFFFF0000"
					end	
					
					self.text:SetText(purchasecolor..COSTS_LABEL.."  "..("[money:%d]"):format(cost):tag().."|r")

					if full then
						self:Hide()
					end

					UpdateBagSlotStatus()
				end)

				if (full) then
					purchase:Hide()
				end
			end
			
			local categoryButton = CreateFrame("Frame", nil, self)
			categoryButton.Text = categoryButton:CreateFontString(nil, "OVERLAY")
			categoryButton.Text:SetFontObject(gBAGS_SystemFontSmall)
			categoryButton.Text:SetText(">>")
			categoryButton.Text:SetPoint("BOTTOMLEFT", title, "BOTTOMRIGHT", 8, 0)
			categoryButton:SetAllPoints(categoryButton.Text)
			categoryButton:SetHitRectInsets(-8, -8, -8, -8)

			local hideOrShow = function(name) 
				if gBAGS_DB["bags"].hide[name] then
					gBAGS_DB["bags"].hide[name] = false
					Bags:GetContainer(name):Show()
				else
					gBAGS_DB["bags"].hide[name] = true
					Bags:GetContainer(name):Hide()
				end
			end
			
			local menuFrame = CreateFrame("Frame", "gBags_Bags_" .. name .. "CategorySelectionDropdown", self, "UIDropDownMenuTemplate")
			local menuList = function() return {
				{
					text = L["EQUIPMENT_SETS"]; 
					checked = not(gBAGS_DB["bags"].hide[name .. "_Sets"]);
					keepShownOnClick = true;
					isNotRadio = true;
					func = function()
						hideOrShow(name .. "_Sets")
					end
				};
				{
					text = Types["Armor"];
					checked = not(gBAGS_DB["bags"].hide[name .. "_Armor"]);
					keepShownOnClick = true;
					isNotRadio = true;
					func = function() 
						hideOrShow(name .. "_Armor")
					end
				};
				{
					text = Types["Weapon"];
					checked = not(gBAGS_DB["bags"].hide[name .. "_Weapons"]);
					keepShownOnClick = true;
					isNotRadio = true;
					func = function() 
						hideOrShow(name .. "_Weapons")
					end
				};
				{
					text = L["GIZMOS"];
					checked = not(gBAGS_DB["bags"].hide[name .. "_Gizmos"]);
					keepShownOnClick = true;
					isNotRadio = true;
					func = function() 
						hideOrShow(name .. "_Gizmos")
					end
				};
				{
					text = Types["Quest"];
					checked = not(gBAGS_DB["bags"].hide[name .. "_Quest"]);
					keepShownOnClick = true;
					isNotRadio = true;
					func = function() 
						hideOrShow(name .. "_Quest")
					end
				};
				{
					text = Types["Glyph"];
					checked = not(gBAGS_DB["bags"].hide[name .. "_Glyphs"]);
					keepShownOnClick = true;
					isNotRadio = true;
					func = function() 
						hideOrShow(name .. "_Glyphs")
					end
				};
				{
					text = Types["Gem"];
					checked = not(gBAGS_DB["bags"].hide[name .. "_Gems"]);
					keepShownOnClick = true;
					isNotRadio = true;
					func = function() 
						hideOrShow(name .. "_Gems")
					end
				};
				{
					text = Types["Consumable"];
					checked = not(gBAGS_DB["bags"].hide[name .. "_Consumables"]);
					keepShownOnClick = true;
					isNotRadio = true;
					func = function() 
						hideOrShow(name .. "_Consumables")
					end
				};
				{
					text = Types["Trade Goods"];
					checked = not(gBAGS_DB["bags"].hide[name .. "_Trade"]);
					keepShownOnClick = true;
					isNotRadio = true;
					func = function() 
						hideOrShow(name .. "_Trade")
					end
				};
				{
					text = Types["Miscellaneous"];
					checked = not(gBAGS_DB["bags"].hide[name .. "_Misc"]);
					keepShownOnClick = true;
					isNotRadio = true;
					func = function() 
						hideOrShow(name .. "_Misc")
					end
				};
				{
					text = Types["Junk"];
					checked = not(gBAGS_DB["bags"].hide[name .. "_Junk"]);
					keepShownOnClick = true;
					isNotRadio = true;
					func = function() 
						hideOrShow(name .. "_Junk")
					end
				};
			} end

			categoryButton:SetScript("OnMouseUp", function(self, button)
				if (button == "LeftButton") then
					EasyMenu(menuList(), menuFrame, "cursor", 0, 0, "MENU", 2)
				end
			end)
			
			categoryButton:SetScript("OnEnter", function(self)
				if (not InCombatLockdown()) then
					PlaceGameTooltip(self, false)
					GameTooltip:AddLine(L["BAGS_CATEGORIES_TOOLTIP"])
					GameTooltip:Show()
				end
			end)
			
			categoryButton:SetScript("OnLeave", function(self)
				GameTooltip:Hide()
			end)
			
		else

			local bName = name
			local close = createCloseButton(self, L["BAGS_CLOSE_TOOLTIP"], function(self) 
				self:GetParent():Hide()
				gBAGS_DB["bags"].hide[bName] = true
			end, "TOPRIGHT", -6, -8)

			if (gBAGS_DB["bags"].hide[name]) then
				self:Hide()
			else
				if (not strfind(name, "Bank")) then
					self:Show()
				end
			end
		end
	end

	Bags.OnInit = function(self)

		local INVERTED = -1
		
		local hideEmpty 			= function(item) return item.texture ~= nil end
		local hideJunk 				= function(item) return not item.rarity or item.rarity > 0 end

		local onlyArmor 			= function(item) return item.type and (item.type == Types["Armor"]) end 
		local onlyBags 				= function(item) return item.bagID >= 0 and item.bagID <= 4 end
		local onlyBank 				= function(item) return item.bagID == -1 or item.bagID >= 5 and item.bagID <= 11 end
		local onlyConsumables 		= function(item) return item.type and (item.type == Types["Consumable"]) end
		local onlyEpics 			= function(item) return item.rarity and item.rarity > 3 end
		local onlyGems 				= function(item) return item.type and (item.type == Types["Gem"]) end
		local onlyGlyphs 			= function(item) return item.type and (item.type == Types["Glyph"]) end
		local onlyJunk 				= function(item) return item.rarity and (item.rarity == 0) end
		local onlyQuest 			= function(item) return item.type and (item.type == Types["Quest"]) end
		local onlyRareEpics 		= function(item) return item.rarity and item.rarity > 3 end
		local onlyTradeGoods 		= function(item) 
			return (item.type and (item.type == Types["Trade Goods"])) or (item.type and (item.type == Types["Recipe"])) 
		end
		local onlyWeapon 			= function(item) return item.type and (item.type == Types["Weapon"]) end
		
		local onlyGizmos 			= function(item) 
			local itemID = GetContainerItemID(item.bagID, item.slotID)
			return (itemID and (db.extra.gizmos[itemID]))
		end
		
		local onlyMisc 				= function(item) 
			local itemID = GetContainerItemID(item.bagID, item.slotID);
			return (item.type and (item.type == Types["Miscellaneous"])) or (item.type and (item.type == Types["Reagent"])) or (itemID and (db.extra.misc[itemID]))
		end
		
		local onlyItemSets 			= function(item)
			return cargBags.itemKeys["setID"](item)
		end

		local banksets = Container:New("Bank_Sets", { Bags = "bankframe+bank"; Name = L["EQUIPMENT_SETS"]; Columns = 8; })
			banksets:SetFilter(onlyBank, true)
			banksets:SetFilter(onlyItemSets, true)
			banksets:SetFilter(hideEmpty, true)
			banksets:Hide()

		local bankgizmos = Container:New("Bank_Gizmos", { Bags = "bankframe+bank"; Name = L["GIZMOS"]; Columns = 8; })
			bankgizmos:SetFilter(onlyBank, true)
			bankgizmos:SetFilter(onlyGizmos, true)
			bankgizmos:SetFilter(hideEmpty, true)
			bankgizmos:Hide()

		local bankmisc = Container:New("Bank_Misc", { Bags = "backpack+bags"; Name = Types["Miscellaneous"]; Columns = 8; })
			bankmisc:SetFilter(onlyBank, true)
			bankmisc:SetFilter(onlyMisc, true)
			bankmisc:SetFilter(hideEmpty, true)
			bankmisc:Hide()

		local bankarmor = Container:New("Bank_Armor", { Bags = "bankframe+bank"; Name = Types["Armor"]; Columns = 8; })
			bankarmor:SetFilter(onlyBank, true)
			bankarmor:SetFilter(onlyArmor, true)
			bankarmor:SetFilter(hideEmpty, true)
			bankarmor:Hide()

		local bankweapons = Container:New("Bank_Weapons", { Bags = "bankframe+bank"; Name = Types["Weapon"]; Columns = 8; })
			bankweapons:SetFilter(onlyBank, true)
			bankweapons:SetFilter(onlyWeapon, true)
			bankweapons:SetFilter(hideEmpty, true)
			bankweapons:Hide()

		local bankgems = Container:New("Bank_Gems", { Bags = "bankframe+bank"; Name = Types["Gem"]; Columns = 8; })
			bankgems:SetFilter(onlyBank, true)
			bankgems:SetFilter(onlyGems, true)
			bankgems:SetFilter(hideEmpty, true)
			bankgems:Hide()

		local bankglyphs = Container:New("Bank_Glyphs", { Bags = "bankframe+bank"; Name = Types["Glyph"]; Columns = 8; })
			bankglyphs:SetFilter(onlyBank, true)
			bankglyphs:SetFilter(onlyGlyphs, true)
			bankglyphs:SetFilter(hideEmpty, true)
			bankglyphs:Hide()

		local bankquest = Container:New("Bank_Quest", { Bags = "bankframe+bank"; Name = Types["Quest"]; Columns = 8; })
			bankquest:SetFilter(onlyBank, true)
			bankquest:SetFilter(onlyQuest, true)
			bankquest:SetFilter(hideEmpty, true)
			bankquest:Hide()

		local bankconsumables = Container:New("Bank_Consumables", { Bags = "bankframe+bank"; Name = Types["Consumable"]; Columns = 8; })
			bankconsumables:SetFilter(onlyBank, true)
			bankconsumables:SetFilter(onlyConsumables, true)
			bankconsumables:SetFilter(hideEmpty, true)
			bankconsumables:Hide()

		local banktrade = Container:New("Bank_Trade", { Bags = "backpack+bags"; Name = Types["Trade Goods"]; Columns = 8; })
			banktrade:SetFilter(onlyBank, true)
			banktrade:SetFilter(onlyTradeGoods, true)
			banktrade:SetFilter(hideEmpty, true)
			banktrade:Hide()

		local bank = Container:New("Bank", { Bags = "bankframe+bank"; Name = L["BANK"]; Columns = 17; Movable = true; })
			bank:SetFilter(onlyBank, true)
			bank:Hide()

		local mainsets = Container:New("Main_Sets", { Bags = "backpack+bags"; Name = L["EQUIPMENT_SETS"]; Columns = 9; })
			mainsets:SetFilter(onlyBags, true)
			mainsets:SetFilter(onlyItemSets, true)
			mainsets:SetFilter(hideEmpty, true)

		local maingizmos = Container:New("Main_Gizmos", { Bags = "backpack+bags"; Name = L["GIZMOS"]; Columns = 9; })
			maingizmos:SetFilter(onlyBags, true)
			maingizmos:SetFilter(onlyGizmos, true)
			maingizmos:SetFilter(hideEmpty, true)

		local mainjunk = Container:New("Main_Junk", { Bags = "backpack+bags"; Name = Types["Junk"]; Columns = 9; })
			mainjunk:SetFilter(onlyBags, true)
			mainjunk:SetFilter(onlyJunk, true)
			mainjunk:SetFilter(hideEmpty, true)

		local mainmisc = Container:New("Main_Misc", { Bags = "backpack+bags"; Name = Types["Miscellaneous"]; Columns = 9; })
			mainmisc:SetFilter(onlyBags, true)
			mainmisc:SetFilter(onlyMisc, true)
			mainmisc:SetFilter(hideEmpty, true)

		local mainarmor = Container:New("Main_Armor", { Bags = "backpack+bags"; Name = Types["Armor"]; Columns = 9; })
			mainarmor:SetFilter(onlyBags, true)
			mainarmor:SetFilter(onlyArmor, true)
			mainarmor:SetFilter(hideJunk, true)
			mainarmor:SetFilter(hideEmpty, true)

		local mainweapons = Container:New("Main_Weapons", { Bags = "backpack+bags"; Name = Types["Weapon"]; Columns = 9; })
			mainweapons:SetFilter(onlyBags, true)
			mainweapons:SetFilter(onlyWeapon, true)
			mainweapons:SetFilter(hideEmpty, true)

		local maingems = Container:New("Main_Gems", { Bags = "backpack+bags"; Name = Types["Gem"]; Columns = 9; })
			maingems:SetFilter(onlyBags, true)
			maingems:SetFilter(onlyGems, true)
			maingems:SetFilter(hideEmpty, true)

		local mainglyphs = Container:New("Main_Glyphs", { Bags = "backpack+bags"; Name = Types["Glyph"]; Columns = 9; })
			mainglyphs:SetFilter(onlyBags, true)
			mainglyphs:SetFilter(onlyGlyphs, true)
			mainglyphs:SetFilter(hideEmpty, true)

		local mainquest = Container:New("Main_Quest", { Bags = "backpack+bags"; Name = Types["Quest"]; Columns = 9; })
			mainquest:SetFilter(onlyBags, true)
			mainquest:SetFilter(onlyQuest, true)
			mainquest:SetFilter(hideEmpty, true)

		local mainconsumables = Container:New("Main_Consumables", { Bags = "backpack+bags"; Name = Types["Consumable"]; Columns = 9; })
			mainconsumables:SetFilter(onlyBags, true)
			mainconsumables:SetFilter(onlyConsumables, true)
			mainconsumables:SetFilter(hideEmpty, true)

		local maintrade = Container:New("Main_Trade", { Bags = "backpack+bags"; Name = Types["Trade Goods"]; Columns = 9; })
			maintrade:SetFilter(onlyBags, true)
			maintrade:SetFilter(onlyTradeGoods, true)
			maintrade:SetFilter(hideEmpty, true)

		local main = Container:New("Main", {  Bags = "backpack+bags"; Name = L["MAIN"] ;Columns = 9; Movable = true; })
			main:SetFilter(onlyBags, true)
	
		main:SetPoint(unpack(gBAGS_DB["bags"].points["Main"]))
		
		mainmisc:SetPoint(				"BOTTOM", 		maintrade, 			"TOP", 			  0, 0)
		maintrade:SetPoint(				"BOTTOM", 		mainconsumables, 	"TOP", 			  0, 0)
		mainconsumables:SetPoint(		"BOTTOM", 		main, 				"TOP", 			  0, 0)
		mainsets:SetPoint(				"BOTTOMRIGHT", 	main, 				"BOTTOMLEFT", 	  0, 0)
		mainarmor:SetPoint(				"BOTTOM", 		mainsets, 			"TOP", 			  0, 0)
		mainweapons:SetPoint(			"BOTTOM", 		mainarmor, 			"TOP", 			  0, 0)
		maingems:SetPoint(				"BOTTOM", 		mainweapons, 		"TOP", 			  0, 0)
		mainglyphs:SetPoint(			"BOTTOM", 		maingems, 			"TOP", 			  0, 0)
		mainquest:SetPoint(				"BOTTOM", 		mainglyphs, 		"TOP", 			  0, 0)
		maingizmos:SetPoint(			"BOTTOM", 		mainquest, 			"TOP", 			  0, 0)
		mainjunk:SetPoint(				"BOTTOM", 		maingizmos, 		"TOP", 			  0, 0)

		bank:SetPoint(unpack(gBAGS_DB["bags"].points["Bank"]));
		
		banksets:SetPoint(				"TOPLEFT", 		bank, 				"BOTTOMLEFT", 	  0, 0)
		bankarmor:SetPoint(				"TOP", 			banksets, 			"BOTTOM", 		  0, 0)
		bankweapons:SetPoint(			"TOP", 			bankarmor, 			"BOTTOM", 		  0, 0)
		bankgizmos:SetPoint(			"TOPLEFT", 		bank, 				"TOPRIGHT", 	  0, 0)
		bankconsumables:SetPoint(		"TOP", 			bankgizmos, 		"BOTTOM", 	 	  0, 0)
		banktrade:SetPoint(				"TOP", 			bankconsumables, 	"BOTTOM", 		  0, 0)
		bankmisc:SetPoint(				"TOP", 			banktrade, 			"BOTTOM", 		  0, 0)
		bankgems:SetPoint(				"TOPRIGHT", 	bank, 				"BOTTOMRIGHT", 	  0, 0)
		bankglyphs:SetPoint(			"TOP", 			bankgems, 			"BOTTOM", 	 	  0, 0)
		bankquest:SetPoint(				"TOP", 			bankglyphs, 		"BOTTOM", 	 	  0, 0)

	end

	Bags.OnBankOpened = function(self)
		-- the order here is crucial, as each category "grabs" their items once opened. 
		if not gBAGS_DB["bags"].hide["Bank_Sets"] then self:GetContainer("Bank_Sets"):Show() end
		if not gBAGS_DB["bags"].hide["Bank_Misc"] then self:GetContainer("Bank_Misc"):Show() end
		if not gBAGS_DB["bags"].hide["Bank_Armor"] then self:GetContainer("Bank_Armor"):Show() end
		if not gBAGS_DB["bags"].hide["Bank_Weapons"] then self:GetContainer("Bank_Weapons"):Show() end
		if not gBAGS_DB["bags"].hide["Bank_Gizmos"] then self:GetContainer("Bank_Gizmos"):Show() end
		if not gBAGS_DB["bags"].hide["Bank_Quest"] then self:GetContainer("Bank_Quest"):Show() end
		if not gBAGS_DB["bags"].hide["Bank_Glyphs"] then self:GetContainer("Bank_Glyphs"):Show() end
		if not gBAGS_DB["bags"].hide["Bank_Gems"] then self:GetContainer("Bank_Gems"):Show() end
		if not gBAGS_DB["bags"].hide["Bank_Trade"] then self:GetContainer("Bank_Trade"):Show() end
		if not gBAGS_DB["bags"].hide["Bank_Consumables"] then self:GetContainer("Bank_Consumables"):Show() end
		if not gBAGS_DB["bags"].hide["Bank"] then self:GetContainer("Bank"):Show() end
	end

	Bags.OnBankClosed = function(self)
		self:GetContainer("Bank"):Hide()
		self:GetContainer("Bank_Sets"):Hide()
		self:GetContainer("Bank_Armor"):Hide()
		self:GetContainer("Bank_Weapons"):Hide()
		self:GetContainer("Bank_Gizmos"):Hide()
		self:GetContainer("Bank_Quest"):Hide()
		self:GetContainer("Bank_Glyphs"):Hide()
		self:GetContainer("Bank_Gems"):Hide()
		self:GetContainer("Bank_Consumables"):Hide()
		self:GetContainer("Bank_Trade"):Hide()
		self:GetContainer("Bank_Misc"):Hide()
	end

	Bags.SetScale = function(self, scale)
		self:GetContainer("Bank"):SetScale(scale)
		self:GetContainer("Bank_Sets"):SetScale(scale)
		self:GetContainer("Bank_Armor"):SetScale(scale)
		self:GetContainer("Bank_Weapons"):SetScale(scale)
		self:GetContainer("Bank_Gizmos"):SetScale(scale)
		self:GetContainer("Bank_Quest"):SetScale(scale)
		self:GetContainer("Bank_Glyphs"):SetScale(scale)
		self:GetContainer("Bank_Gems"):SetScale(scale)
		self:GetContainer("Bank_Consumables"):SetScale(scale)
		self:GetContainer("Bank_Trade"):SetScale(scale)
		self:GetContainer("Bank_Misc"):SetScale(scale)

		self:GetContainer("Main"):SetScale(scale)
		self:GetContainer("Main_Sets"):SetScale(scale)
		self:GetContainer("Main_Armor"):SetScale(scale)
		self:GetContainer("Main_Weapons"):SetScale(scale)
		self:GetContainer("Main_Gizmos"):SetScale(scale)
		self:GetContainer("Main_Quest"):SetScale(scale)
		self:GetContainer("Main_Glyphs"):SetScale(scale)
		self:GetContainer("Main_Gems"):SetScale(scale)
		self:GetContainer("Main_Consumables"):SetScale(scale)
		self:GetContainer("Main_Trade"):SetScale(scale)
		self:GetContainer("Main_Misc"):SetScale(scale)
		self:GetContainer("Main_Junk"):SetScale(scale)
	end
	
	local slashHandler = function(cmd, opt)
		if (cmd == "lock") then
			gBAGS_DB["bags"].locked = 1
			
		elseif (cmd == "unlock") then
			gBAGS_DB["bags"].locked = 0
			
		elseif (cmd == "reset") then
			gBAGS_DB["bags"].locked = 1

			gBAGS_DB["bags"].points["Main"] = defaults.points["Main"]
			gBAGS_DB["bags"].points["Bank"] = defaults.points["Bank"]
			
			if (not Bags:GetContainer("Main")) then return end
			
			Bags:GetContainer("Main"):ClearAllPoints()
			Bags:GetContainer("Main"):SetPoint(unpack(gBAGS_DB["bags"].points["Main"]))
			
			Bags:GetContainer("Bank"):ClearAllPoints()
			Bags:GetContainer("Bank"):SetPoint(unpack(gBAGS_DB["bags"].points["Bank"]))
			
			for container, _ in pairs(gBAGS_DB["bags"].hide) do
				gBAGS_DB["bags"].hide[container] = defaults.hide[container]
				if strfind(container, "Bank") then 
					if Bags:GetContainer("Bank"):IsShown() then
						if gBAGS_DB["bags"].hide[container] then
							Bags:GetContainer(container):Hide()
						else
							Bags:GetContainer(container):Show()
						end
					end
				else
					if gBAGS_DB["bags"].hide[container] then
						Bags:GetContainer(container):Hide()
					else
						Bags:GetContainer(container):Show()
					end
				end
			end
		elseif (cmd == "scale") then
			opt = tonumber(opt)
			
			if (opt >= 1) and (opt <= 2) then
				gBAGS_DB["bags"].scale = opt
				
				if (Bags:GetContainer("Main")) then
					Bags:SetScale(opt)
				end
			else
				print("/scalebags x |cffff0000(1 <= x <= 2)|r")
			end
		end
	end
	
	self:RegisterEvent("CHAT_MSG_LOOT")
	self:RegisterEvent("TRADE_SKILL_UPDATE")

	CreateChatCommand(function(opt) slashHandler("scale", opt) end, "scalebags")
	CreateChatCommand(function() slashHandler("reset") end, "resetbags")
	CreateChatCommand(function() slashHandler("lock") end, "lockbags")
	CreateChatCommand(function() slashHandler("unlock") end, "unlockbags")
	CreateChatCommand(function() 
		if (gBAGS_DB["bags"].compressemptyspace == 0) then
			gBAGS_DB["bags"].compressemptyspace = 1 
			print(L["BAGS_COMPRESSION_ON"])
			updateAllContainers()
		else
			gBAGS_DB["bags"].compressemptyspace = 0 
			print(L["BAGS_COMPRESSION_OFF"])
			updateAllContainers()
		end
	end, "compressbags")
	
	CreateChatCommand(function(opt) 
		if (opt == "resume") then
			RestackBags("resume") 
		end
	end, "restackbags")
end
