﻿local addon, ns = ...;

local defaults = {
	autorepair 			= true;
	autosell 			= true;
	guildrepair 		= false;
	detailedreport 		= true;
};

gMerchant_DB = gMerchant_DB or {}

gMerchant = gLib:newAddon( addon );
gMerchant:RegisterEvent("MERCHANT_SHOW");

local L = gMerchant:GetLocale();

gMerchant:Slash( "menu", function() 
	InterfaceOptionsFrame_OpenToCategory(gMerchant.optionspanel.name); 
end, { "gmerchant", "gmerc" } );

local _G = _G;
local format, max, min, print = format, max, min, print;

-- event handlers
function gMerchant:OnInit()
	gMerchant_DB = GL_Table_Validate( gMerchant_DB, defaults );
end

function gMerchant:OnEnable()
	self.optionspanel = GL_CreateInterfaceOptionsPanel({
		title = self.NAME;
		version = self.VERSION;
		description = "";
		panel = nil;
		option = {
			[0] = {
				name = L["GUI_VENDOR_AUTOSELL"];
				type = "button";
				value = gMerchant_DB.autosell;
				panelfunction = function() gMerchant_DB.autosell = not gMerchant_DB.autosell; end;
			};
			[5] = {
				name = L["GUI_VENDOR_DETAILEDREPORT"];
				type = "button";
				value = gMerchant_DB.detailedreport;
				panelfunction = function() gMerchant_DB.detailedreport = not gMerchant_DB.detailedreport; end;
			};
			[10] = {
				name = L["GUI_VENDOR_AUTOREPAIR"];
				type = "button";
				value = gMerchant_DB.autorepair;
				panelfunction = function() gMerchant_DB.autorepair = not gMerchant_DB.autorepair; end;
			};
			[20] = {
				name = L["GUI_VENDOR_GUILDREPAIR"];
				type = "button";
				value = gMerchant_DB.guildrepair;
				panelfunction = function() gMerchant_DB.guildrepair = not gMerchant_DB.guildrepair; end;
			};
		};
	}, gMerchant_DB, self.defaults );
end

function gMerchant:MERCHANT_SHOW()

	local gain, sold 			= 0, 0;
	local repaired 				= false;
	local useGuildFunds 		= IsInGuild() and gMerchant_DB.guildrepair;
	local usedGuildFunds 		= false;
	local yourGuildFunds 		= min( GetGuildBankWithdrawMoney(), GetGuildBankMoney() );
	local repairCost 			= ( select( 1, GetRepairAllCost() ) ) or 0;

	if ( gMerchant_DB.autosell == true ) then
		for bag = 0,4,1 do 
			for slot = 1, GetContainerNumSlots( bag ), 1 do 
				local itemID = GetContainerItemID( bag, slot );
				if itemID then
					local count = ( select( 2, GetContainerItemInfo( bag, slot ) ) );
					local _, link, rarity, _, _, _, _, _, _, _, price = GetItemInfo( itemID ); 

					if ( rarity == 0 ) then
						local stack = ( price or 0 ) * ( count or 1 );
						sold = sold + stack; 
						
						if gMerchant_DB.detailedreport then
							print( L["MESSAGE_AUTOSELLING"]:format( link, count, string.tag( "[money:"..stack.."]" ) ) );
						end
						
						UseContainerItem( bag, slot );
					end
				end 
			end 
		end
		gain = gain + sold;
	end

	if ( sold > 0 ) then 
		print( L["MESSAGE_SELLVALUE"]:format( string.tag( "[money:"..sold.."]" ) ) ); 
	end

	if ( gMerchant_DB.autorepair and CanMerchantRepair() and ( repairCost > 0 ) ) then

		if ( max( GetMoney(), yourGuildFunds ) > repairCost ) then
		
			if ( useGuildFunds and ( yourGuildFunds > repairCost ) ) and ( MerchantGuildBankRepairButton:IsEnabled() ) and ( MerchantGuildBankRepairButton:IsShown() ) then
				RepairAllItems(1); 
				usedGuildFunds = true;
				repaired = true;
				print( L["MESSAGE_REPAIRCOST_USING_GUILD"]:format( string.tag( "|cffff0000[money:"..repairCost.."]|r" ) ) );
	
			elseif ( GetMoney() > repairCost ) then
				RepairAllItems(); 
				repaired = true;
				print( L["MESSAGE_REPAIRCOST"]:format( string.tag( "|cffff0000[money:"..repairCost.."]|r" ) ) );
				
				gain = gain - repairCost;
			end
			
		else
			print( L["MESSAGE_NOT_ENOUGH_TO_REPAIR"] );
		end
	end

	if ( gain > 0 ) then
		print( L["MESSAGE_VENDORPROFIT"]:format( string.tag( "[money:"..gain.."]" ) ) ); 
		
	elseif ( gain < 0 ) then 
		print( L["MESSAGE_VENDOREXPENSE"]:format( string.tag( "|cffff0000[money:"..( -gain ).."]|r" ) ) );
	end

end

