--[[
	
	sGroupLoot
	Copyright (c) 2010, Nils Ruesch
	All rights reserved.
	
]]

--[[------------------------]]--
-- ndern auf eigener Gefahr! --
--[[------------------------]]--

local TEST = false;

UIParent:UnregisterEvent("START_LOOT_ROLL");
UIParent:UnregisterEvent("CANCEL_LOOT_ROLL");

local GFHCName, GFHCHeight = GameFontHighlightCenter:GetFont();
local frames, rolls, rolltypes, rollstrings = {}, {}, { [0] = PASS, [1] = NEED, [2] = GREED, [3] = ROLL_DISENCHANT }, { [(LOOT_ROLL_PASSED_AUTO):gsub("%%1$s", "(.+)"):gsub("%%2$s", "(.+)")] = 0, [(LOOT_ROLL_PASSED_AUTO_FEMALE):gsub("%%1$s", "(.+)"):gsub("%%2$s", "(.+)")] = 0, [(LOOT_ROLL_PASSED):gsub("%%s", "(.+)")] = 0, [(LOOT_ROLL_GREED):gsub("%%s", "(.+)")] = 2, [(LOOT_ROLL_NEED):gsub("%%s", "(.+)")] = 1, [(LOOT_ROLL_DISENCHANT):gsub("%%s", "(.+)")] = 3 };

local function OnUpdate(self, elapsed)
	if ( self.rollID and not self.paused ) then
		if ( TEST ) then
			self:SetValue(self.rollID);
			self.paused = 1;
		else
			self:SetValue(GetLootRollTimeLeft(self.rollID));
		end
		
		if ( GameTooltip:IsOwned(self) ) then
			GameTooltip:SetLootRollItem(self.rollID);
			CursorUpdate(self);
		end
	end
end

local function OnClick(self)
	HandleModifiedItemClick(self.rollLink);
end

local function OnEvent(self, event, ...)
	local rollID = ...;
	if ( self.rollID and self.rollID == rollID ) then
		self.paused = 1;
		StaticPopup_Hide("CONFIRM_LOOT_ROLL", rollID);
		
		self:SetStatusBarColor(0.5, 0.5, 0.5);
		for i=0, #self.button, 1 do
			GroupLootFrame_DisableLootButton(self.button[i]);
		end
		
		local fadeInfo = {};
		fadeInfo.mode = "OUT";
		fadeInfo.timeToFade = 0.5;
		fadeInfo.startAlpha = self:GetAlpha();
		fadeInfo.endAlpha = 0;
		fadeInfo.finishedFunc = function()
			rolls[rollID] = nil;
			self.rollID = nil;
			self.paused = nil;
			self:Hide();
		end;
		UIFrameFade(self, fadeInfo);
	end
end

local function OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_BOTTOMLEFT");
	GameTooltip:SetLootRollItem(self.rollID);
	CursorUpdate(self);
end

local function OnLeave(self)
	GameTooltip:Hide();
	ResetCursor();
end

local function Button_OnClick(self)
	RollOnLoot(self:GetParent().rollID, self:GetID());
end

local function Button_OnEnter(self)
	local id = self:GetID();
	local rollID = self:GetParent().rollID;
	GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT");
	GameTooltip:SetText(rolltypes[id]);
	if ( rolls[rollID] and rolls[rollID][id] ) then
		for player, color in pairs(rolls[rollID][id]) do
			if ( player ~= "count" ) then
				GameTooltip:AddLine("|cff"..color..player.."|r");
			end
		end
	end
	if ( self:IsEnabled() ~= 1 ) then
		GameTooltip:AddLine(self.reason, RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b, true);
	end
	GameTooltip:Show();
end

local function CreateRollFrame()
	local frame = CreateFrame("StatusBar", "sGroupLootBar"..(#frames+1), sGroupLoot);
	frame.hasItem = 1; -- Brauch man fr die Funktion CursorUpdate.
	frame:SetAlpha(0);
	frame:EnableMouse(1);
	frame:SetWidth(280);
	frame:SetHeight(25);
	if ( #frames == 0 ) then
		frame:SetPoint("TOP");
	else
		frame:SetPoint("TOP", frames[#frames], "BOTTOM", 0, -6);
	end
	frame:SetStatusBarTexture("Interface\\AddOns\\sGroupLoot\\statusbar");
	frame:SetScript("OnUpdate", OnUpdate);
	frame:SetScript("OnMouseUp", OnClick);
	frame:RegisterEvent("CANCEL_LOOT_ROLL");
	frame:SetScript("OnEvent", OnEvent);
	frame:SetScript("OnEnter", OnEnter);
	frame:SetScript("OnLeave", OnLeave);
	
	frame.background = frame:CreateTexture("$parentBackground", "BORDER");
	frame.background:SetAllPoints();
	frame.background:SetTexture("Interface\\AddOns\\sGroupLoot\\statusbar");
	frame.background:SetVertexColor(0.5, 0.5, 0.5, 0.7);
	
	frame.button = {};
	frame.button[0] = CreateFrame("Button", "$perentPassButton", frame);
	frame.button[0]:SetMotionScriptsWhileDisabled(true);
	frame.button[0]:SetID(0);
	frame.button[0]:SetWidth(28);
	frame.button[0]:SetHeight(28);
	frame.button[0]:SetNormalTexture("Interface\\Buttons\\UI-GroupLoot-Pass-Up");
	frame.button[0]:SetHighlightTexture("Interface\\Buttons\\UI-GroupLoot-Pass-Down");
	frame.button[0]:SetHighlightTexture("Interface\\Buttons\\UI-GroupLoot-Pass-Highlight");
	frame.button[0]:SetPoint("RIGHT", -5, 1);
	frame.button[0]:SetScript("OnClick", Button_OnClick);
	frame.button[0]:SetScript("OnEnter", Button_OnEnter);
	frame.button[0]:SetScript("OnLeave", OnLeave);
	frame.button[0].text = frame.button[0]:CreateFontString("$perentText", "ARTWORK");
	frame.button[0].text:SetFont(GFHCName, GFHCHeight, "OUTLINE");
	frame.button[0].text:SetShadowColor(1, 1, 1, 0);
	frame.button[0].text:SetPoint("CENTER");
	
	frame.button[2] = CreateFrame("Button", "$perentGreedButton", frame);
	frame.button[2]:SetMotionScriptsWhileDisabled(true);
	frame.button[2]:SetID(2);
	frame.button[2]:SetWidth(28);
	frame.button[2]:SetHeight(28);
	frame.button[2]:SetNormalTexture("Interface\\Buttons\\UI-GroupLoot-Coin-Up");
	frame.button[2]:SetPushedTexture("Interface\\Buttons\\UI-GroupLoot-Coin-Down");
	frame.button[2]:SetHighlightTexture("Interface\\Buttons\\UI-GroupLoot-Coin-Highlight");
	frame.button[2]:SetPoint("RIGHT", frame.button[0], "LEFT", -2, -4);
	frame.button[2]:SetScript("OnClick", Button_OnClick);
	frame.button[2]:SetScript("OnEnter", Button_OnEnter);
	frame.button[2]:SetScript("OnLeave", OnLeave);
	frame.button[2].text = frame.button[2]:CreateFontString("$perentText", "ARTWORK");
	frame.button[2].text:SetFont(GFHCName, GFHCHeight, "OUTLINE");
	frame.button[2].text:SetShadowColor(1, 1, 1, 0);
	frame.button[2].text:SetPoint("CENTER", 0, 3);
	
	frame.button[1] = CreateFrame("Button", "$perentNeedButton", frame);
	frame.button[1]:SetMotionScriptsWhileDisabled(true);
	frame.button[1]:SetID(1);
	frame.button[1]:SetWidth(28);
	frame.button[1]:SetHeight(28);
	frame.button[1]:SetNormalTexture("Interface\\Buttons\\UI-GroupLoot-Dice-Up");
	frame.button[1]:SetPushedTexture("Interface\\Buttons\\UI-GroupLoot-Dice-Down");
	frame.button[1]:SetHighlightTexture("Interface\\Buttons\\UI-GroupLoot-Dice-Highlight");
	frame.button[1]:SetPoint("RIGHT", frame.button[2], "LEFT", -2, 2);
	frame.button[1]:SetScript("OnClick", Button_OnClick);
	frame.button[1]:SetScript("OnEnter", Button_OnEnter);
	frame.button[1]:SetScript("OnLeave", OnLeave);
	frame.button[1].text = frame.button[1]:CreateFontString("$perentText", "ARTWORK");
	frame.button[1].text:SetFont(GFHCName, GFHCHeight, "OUTLINE");
	frame.button[1].text:SetShadowColor(1, 1, 1, 0);
	frame.button[1].text:SetPoint("CENTER", 0, 1);
	
	frame.button[3] = CreateFrame("Button", "$perentDisenchantButton", frame);
	frame.button[3]:SetMotionScriptsWhileDisabled(true);
	frame.button[3]:SetID(3);
	frame.button[3]:SetWidth(28);
	frame.button[3]:SetHeight(28);
	frame.button[3]:SetNormalTexture("Interface\\Buttons\\UI-GroupLoot-DE-Up");
	frame.button[3]:SetPushedTexture("Interface\\Buttons\\UI-GroupLoot-DE-Down");
	frame.button[3]:SetHighlightTexture("Interface\\Buttons\\UI-GroupLoot-DE-Highlight");
	frame.button[3]:SetPoint("RIGHT", frame.button[1], "LEFT", -2, 0);
	frame.button[3]:SetScript("OnClick", Button_OnClick);
	frame.button[3]:SetScript("OnEnter", Button_OnEnter);
	frame.button[3]:SetScript("OnLeave", OnLeave);
	frame.button[3].text = frame.button[3]:CreateFontString("$perentText", "ARTWORK");
	frame.button[3].text:SetFont(GFHCName, GFHCHeight, "OUTLINE");
	frame.button[3].text:SetShadowColor(1, 1, 1, 0);
	frame.button[3].text:SetPoint("CENTER", 0, 1);
	
	frame.text = frame:CreateFontString("$perentText", "ARTWORK", "GameFontHighlightLeft");
	frame.text:SetPoint("LEFT", 5, 0);
	frame.text:SetPoint("RIGHT", frame.button[3], "LEFT");
	
	frame.border = {};
	frame.border.topleft = frame:CreateTexture(nil, "OVERLAY");
	frame.border.topleft:SetTexture("Interface\\AddOns\\sGroupLoot\\border");
	frame.border.topleft:SetPoint("TOPLEFT", -4, 4);
	frame.border.topleft:SetWidth(16);
	frame.border.topleft:SetHeight(16);
	frame.border.topleft:SetTexCoord(0, 1/3, 0, 1/3);
	frame.border.bottomleft = frame:CreateTexture(nil, "OVERLAY");
	frame.border.bottomleft:SetTexture("Interface\\AddOns\\sGroupLoot\\border");
	frame.border.bottomleft:SetPoint("BOTTOMLEFT", -4, -4);
	frame.border.bottomleft:SetWidth(16);
	frame.border.bottomleft:SetHeight(16);
	frame.border.bottomleft:SetTexCoord(0, 1/3, 2/3, 1);
	frame.border.topright = frame:CreateTexture(nil, "OVERLAY");
	frame.border.topright:SetTexture("Interface\\AddOns\\sGroupLoot\\border");
	frame.border.topright:SetPoint("TOPRIGHT", 4, 4);
	frame.border.topright:SetWidth(16);
	frame.border.topright:SetHeight(16);
	frame.border.topright:SetTexCoord(2/3, 1, 0, 1/3);
	frame.border.bottomright = frame:CreateTexture(nil, "OVERLAY");
	frame.border.bottomright:SetTexture("Interface\\AddOns\\sGroupLoot\\border");
	frame.border.bottomright:SetPoint("BOTTOMRIGHT", 4, -4);
	frame.border.bottomright:SetWidth(16);
	frame.border.bottomright:SetHeight(16);
	frame.border.bottomright:SetTexCoord(2/3, 1, 2/3, 1);
	frame.border.top = frame:CreateTexture(nil, "OVERLAY");
	frame.border.top:SetTexture("Interface\\AddOns\\sGroupLoot\\border");
	frame.border.top:SetPoint("TOPLEFT", frame.border.topleft, "TOPRIGHT");
	frame.border.top:SetPoint("TOPRIGHT", frame.border.topright, "TOPLEFT");
	frame.border.top:SetHeight(16);
	frame.border.top:SetTexCoord(1/3, 2/3, 0, 1/3);
	frame.border.bottom = frame:CreateTexture(nil, "OVERLAY");
	frame.border.bottom:SetTexture("Interface\\AddOns\\sGroupLoot\\border");
	frame.border.bottom:SetPoint("BOTTOMLEFT", frame.border.bottomleft, "BOTTOMRIGHT");
	frame.border.bottom:SetPoint("BOTTOMRIGHT", frame.border.bottomright, "BOTTOMLEFT");
	frame.border.bottom:SetHeight(16);
	frame.border.bottom:SetTexCoord(1/3, 2/3, 2/3, 1);
	frame.border.left = frame:CreateTexture(nil, "OVERLAY");
	frame.border.left:SetTexture("Interface\\AddOns\\sGroupLoot\\border");
	frame.border.left:SetPoint("TOPLEFT", frame.border.topleft, "BOTTOMLEFT");
	frame.border.left:SetPoint("BOTTOMLEFT", frame.border.bottomleft, "TOPLEFT");
	frame.border.left:SetWidth(16);
	frame.border.left:SetTexCoord(0, 1/3, 1/3, 2/3);
	frame.border.right = frame:CreateTexture(nil, "OVERLAY");
	frame.border.right:SetTexture("Interface\\AddOns\\sGroupLoot\\border");
	frame.border.right:SetPoint("TOPRIGHT", frame.border.topright, "BOTTOMRIGHT");
	frame.border.right:SetPoint("BOTTOMRIGHT", frame.border.bottomright, "TOPRIGHT");
	frame.border.right:SetWidth(16);
	frame.border.right:SetTexCoord(2/3, 1, 1/3, 2/3);
	
	table.insert(frames, frame);
	return frame;
end

local function GetRollFrame()
	for _, frame in ipairs(frames) do
		if ( not frame.rollID ) then
			return frame;
		end
	end

	return CreateRollFrame();
end

local function GetClass(player)
	local class;
	if ( UnitName("player") == player ) then
		class = RAID_CLASS_COLORS[select(2, UnitClass("player"))];
		return class.r*255, class.g*255, class.b*255;
	end
	
	local num, unit = GetNumRaidMembers();
	if ( num > 0 ) then
		for i=1, num, 1 do
			unit = "raid"..i;
			if ( UnitName(unit) == player ) then
				class = RAID_CLASS_COLORS[select(2, UnitClass(unit))];
				return class.r*255, class.g*255, class.b*255;
			end
		end
	end
	
	num = GetNumPartyMembers();
	if ( num > 0 ) then
		for i=1, num, 1 do
			unit = "party"..i;
			if ( UnitName(unit) == player ) then
				class = RAID_CLASS_COLORS[select(2, UnitClass(unit))];
				return class.r*255, class.g*255, class.b*255;
			end
		end
	end
end

local function OnEvent(self, event, ...)
	if ( event == "CHAT_MSG_LOOT" ) then
		local msg = ...;
		for string, type in pairs(rollstrings) do
			local _, _, player, item = string.find(msg, string);
			if ( player and item ) then
				local rollID;
				for _, frame in ipairs(frames) do
					rollID = frame.rollID;
					if ( rollID and frame.rollLink == item ) then
						rolls[rollID] = rolls[rollID] or {};
						rolls[rollID][type] = rolls[rollID][type] or {};
						rolls[rollID][type].count = (rolls[rollID][type].count or 0)+1;
						rolls[rollID][type][player] = ("%02x%02x%02x"):format(GetClass(player));
						
						frame.button[type].text:SetText(rolls[rollID][type].count);
						return;
					end
				end
			end
		end
		return;
	end
	
	local rollID, rollTime = ...;
	local texture, name, count, quality, bindOnPickUp, canNeed, canGreed, canDisenchant, reasonNeed, reasonGreed, reasonDisenchant, deSkillRequired = GetLootRollItemInfo(rollID);
	local color = ITEM_QUALITY_COLORS[quality];
	local frame = GetRollFrame();
	frame.rollID = rollID;
	frame.rollTime = rollTime;
	frame.rollLink = GetLootRollItemLink(rollID);
	frame:SetMinMaxValues(0, rollTime);
	frame:SetStatusBarColor(color.r, color.g, color.b, 1);
	frame.text:SetText(count > 1 and count.."x "..name or name);
	
	for i=0, #frame.button, 1 do
		frame.button[i].text:SetText("");
	end
	GroupLootFrame_EnableLootButton(frame.button[0]);
	if ( canNeed ) then
		GroupLootFrame_EnableLootButton(frame.button[1]);
		frame.button[1].reason = nil;
	else
		GroupLootFrame_DisableLootButton(frame.button[1]);
		frame.button[1].reason = _G["LOOT_ROLL_INELIGIBLE_REASON"..reasonNeed];
	end
	if ( canGreed ) then
		GroupLootFrame_EnableLootButton(frame.button[2]);
		frame.button[2].reason = nil;
	else
		GroupLootFrame_DisableLootButton(frame.button[2]);
		frame.button[2].reason = _G["LOOT_ROLL_INELIGIBLE_REASON"..reasonGreed];
	end
	if ( canDisenchant ) then
		GroupLootFrame_EnableLootButton(frame.button[3]);
		frame.button[3].reason = nil;
	else
		GroupLootFrame_DisableLootButton(frame.button[3]);
		frame.button[3].reason = format(_G["LOOT_ROLL_INELIGIBLE_REASON"..reasonDisenchant], deSkillRequired);
	end
	
	if ( bindOnPickUp ) then
		frame.border.topleft:SetVertexColor(194/255, 172/255, 114/255, 1);
		frame.border.bottomleft:SetVertexColor(194/255, 172/255, 114/255, 1);
		frame.border.topright:SetVertexColor(194/255, 172/255, 114/255, 1);
		frame.border.bottomright:SetVertexColor(194/255, 172/255, 114/255, 1);
		frame.border.top:SetVertexColor(194/255, 172/255, 114/255, 1);
		frame.border.bottom:SetVertexColor(194/255, 172/255, 114/255, 1);
		frame.border.left:SetVertexColor(194/255, 172/255, 114/255, 1);
		frame.border.right:SetVertexColor(194/255, 172/255, 114/255, 1);
	else
		frame.border.topleft:SetVertexColor(0.25, 0.25, 0.25, 1);
		frame.border.bottomleft:SetVertexColor(0.25, 0.25, 0.25, 1);
		frame.border.topright:SetVertexColor(0.25, 0.25, 0.25, 1);
		frame.border.bottomright:SetVertexColor(0.25, 0.25, 0.25, 1);
		frame.border.top:SetVertexColor(0.25, 0.25, 0.25, 1);
		frame.border.bottom:SetVertexColor(0.25, 0.25, 0.25, 1);
		frame.border.left:SetVertexColor(0.25, 0.25, 0.25, 1);
		frame.border.right:SetVertexColor(0.25, 0.25, 0.25, 1);
	end
	
	local fadeInfo = {};
	fadeInfo.mode = "IN";
	fadeInfo.timeToFade = 0.5;
	fadeInfo.startAlpha = frame:GetAlpha();
	fadeInfo.endAlpha = 1;
	UIFrameFade(frame, fadeInfo);
end

local frame = CreateFrame("Frame", "sGroupLoot", UIParent);
frame:SetWidth(280);
frame:SetHeight(25);
frame:SetPoint("CENTER", 0, -100);
frame:RegisterEvent("START_LOOT_ROLL");
frame:RegisterEvent("CHAT_MSG_LOOT");
frame:SetScript("OnEvent", OnEvent);

if ( TEST ) then
	local color = 1;
	function GetLootRollItemInfo(rollID)
		color = color+1;
		return "Interface\\Icons\\Spell_Fire_Burnout", "Test-Roll "..rollID, 1, color, nil, nil, 1, nil, 5, nil, 4, 450;
	end
	for i=1, 4, 1 do
		OnEvent(frame, "START_LOOT_ROLL", i, 4);
	end
end
